<?php namespace Stage\Account;

use Closure;
use PDO;
use Stage\Events\Event;
use Stage\Industry;
use Stage\Stage;
use Exception;
use Stage\Role;
use Stage\Lead\Lead;
use Stage\Lead\Query;
use Stage\Account\Service;
use Stage\Account\Api;
use Stage\Account\User;
use Stage\Lead\Tag\Tag;
use Stage\Lead\HardTag\HardTag;
use Stage\Account\Config\Config;

class Account
{
    const ID = "1";
    const EMAIL = "2";
    const TOKEN = "3";

    /**
     * @var
     */
    private $id;

    /**
     * @var
     */
    private $token;

    /**
     * @var
     */
    private $alias;

    /**
     * @var
     */
    private $companyName;

    /**
     * @var
     */
    private $email;

    /**
     * @var
     */
    private $phone = 0;

    /**
     * @var
     */
    private $address = "";

    /**
     * @var
     */
    private $idIndustryType;

    /**
     * @var
     */
    private $countryIso;

    /**
     * @var
     */
    private $website;

    /**
     * @var
     */
    private $timezone;

    /**
     * @var
     */
    private $verified = false;

    /**
     * @var
     */
    private $active = false;

    /**
     * @var
     */
    private $role;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'token', 'alias', 'companyName', 'email', 'phone', 'address', 'idIndustryType', 'countryIso', 'website', 'timezone', 'verified', 'active','role');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array();

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getToken()
    {
        return $this->token;
    }

    public function getAlias()
    {
        return $this->alias;
    }

    public function getCompanyName()
    {
        return $this->companyName;
    }

    public function getEmail()
    {
        return $this->email;
    }

    public function getPhone()
    {
        return $this->phone;
    }

    public function getAddress()
    {
        return $this->address;
    }

    public function getIdIndustryType()
    {
        return $this->idIndustryType;
    }

    public function getCountryIso()
    {
        return $this->countryIso;
    }

    public function getWebsite()
    {
        return $this->website ? $this->website : "";
    }

    public function getTimezone()
    {
        return $this->timezone ? $this->timezone : "";
    }

    public function getVerified()
    {
        return $this->verified;
    }

    public function getActive()
    {
        return $this->active;
    }

    public function getRole()
    {
        return $this->role;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setToken($token)
    {
        $this->token = $token;
    }

    public function setAlias($alias)
    {
        $this->alias = $alias;
    }

    public function setCompanyName($companyName)
    {
        $this->companyName = $companyName;
    }

    public function setEmail($email)
    {
        $this->email = $email;
    }

    public function setPhone($phone)
    {
        $this->phone = (int) $phone;
    }

    public function setAddress($address)
    {
        $this->address = $address;
    }

    public function setIdIndustryType($id)
    {
        $this->idIndustryType = (int) $id;
    }

    public function setCountryIso($iso)
    {
        $this->countryIso = $iso;
    }

    public function setWebsite($website)
    {
        $this->website = $website;
    }

    public function setTimezone($timezone)
    {
        $this->timezone = $timezone;
    }

    public function setVerified($verified)
    {
        $this->verified = $verified == "1" || $verified == true ? true : false;
    }

    public function setActive($active)
    {
        $this->active = $active == "1" || $active == true ? true : false;
    }

    public function setRole(Role $role)
    {
        $this->role= $role;
    }

    /**
     * getUsers
     *
     * Get all the users of this Account
     * @return array
     * @throws Exception
     */
    public function getUsers()
    {
        if (!$this->getId()) {
            throw new Exception("This instance don't has a valid account id defined.");
        }

        $id_account = $this->getId();
        $parameters = [[$id_account, PDO::PARAM_INT]];

        try {
            $name = "ACCOUNT_USERS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
        } catch (Exception $e) {
            throw $e;
        }

        $temp = [];
        foreach ($data as $user) {
            $userInstance = User::createInstance($user);
            $role = Role::createInstance($user);
            $userInstance->setRole($role);
            array_push($temp, $userInstance);
        }

        return $temp;
    }

    /**
     * getIndustry
     *
     * Return the industry of the Account
     * @param $lang
     */
    public function getIndustry($lang)
    {
        try {
            $industries = Industry::all($lang);
            foreach ($industries as $i) {
                if ($i->getId() == $this->getIdIndustryType()) {
                    return $i;
                }
            }
        } catch (Exception $e) {
            throw $e;
        }

        throw new Exception("Industry not found with id $this->getIdIndustryType()");
    }

    /**
     * find
     *
     * @param $value
     * @param string $field
     * @return Account
     * @throws Exception
     */
    public static function find($value, $field = self::ID)
    {
        $parameters = [[$field, PDO::PARAM_STR],
                       [$value, PDO::PARAM_STR]];

        try {
            $name = "ACCOUNT";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
        } catch (Exception $e) {
            throw $e;
        }

        if (sizeof($data) > 0) {
            $accInstance = self::createInstance($data[0]);
            return $accInstance;
        } else {
            throw new Exception("Account not found width $value.");
        }
    }

    /**
     * notCreated
     *
     * @return array
     * @throws Exception
     */
    private static function notCreated()
    {
        try {
            $parameters = [];
            $name = "ACCOUNTS_TO_CREATE";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            $temp = [];
            foreach ($data as $account) {
                try {
                    $temp[] = self::find($account['id_account'], Account::ID);
                } catch (Exception $e) {
                }
            }

            return $temp;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        $parameters = [[$this->getId(), PDO::PARAM_INT],
                       [$this->getToken(), PDO::PARAM_STR],
                       [$this->getAlias(), PDO::PARAM_STR],
                       [$this->getCompanyName(), PDO::PARAM_STR],
                       [$this->getEmail(), PDO::PARAM_STR],
                       [$this->getPhone(), PDO::PARAM_INT],
                       [$this->getAddress(), PDO::PARAM_STR],
                       [$this->getIdIndustryType(), PDO::PARAM_INT],
                       [$this->getCountryIso(), PDO::PARAM_STR],
                       [$this->getWebsite(), PDO::PARAM_STR],
                       [$this->getTimezone(), PDO::PARAM_STR],
                       [$this->getVerified(), PDO::PARAM_INT],
                       [$this->getActive(), PDO::PARAM_INT]];

        // Make a request to Database
        try {
            $name = "ACCOUNT";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            $return_code = $data[0]['return_code'];

            if ($return_code == "0") { // Success

                // Fire an event
                if (!$this->getId()) {
                    Event::fire("account.created", $this);
                } else {
                    Event::fire("account.updated", $this);
                }

                $this->setId($data[0]['id_account']);
            } elseif ($return_code == "-2") {
                throw new Exception("Email already registered.", $return_code);
            } else {
                throw new Exception("Some error occurred.", $return_code);
            }
        } catch (Exception $e) {
            throw $e;
        }

        return $this;
    }

    /**
     * delete
     *
     */
    public function delete()
    {
        // TODO
    }

    /**
     * setup
     * Setup a new Account
     * @throws Exception
     */
    public function setup()
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_STR]];
            $name = "ACCOUNT_SETUP";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("Account not found.", $data[0]['return_code']);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * all
     *
     * @return array
     * @throws Exception
     */
    public static function all($notCreated = false)
    {
        if ($notCreated === true) {
            return self::notCreated();
        } else {
            $parameters = [[0, PDO::PARAM_STR],
                           ["", PDO::PARAM_STR]];

            // Make a request to Database
            try {
                $name = "ACCOUNT";
                $method = "GET";
                $data = Stage::esp($parameters, $name, $method);
            } catch (Exception $e) {
                throw $e;
            }

            $temp = [];
            foreach ($data as $acc) {
                $accInstance = self::createInstance($acc);
                array_push($temp, $accInstance);
            }

            return $temp;
        }
    }

    /**
     * leads
     *
     * Return an Lead[]
     */
    public function leads(Query $q)
    {
        $idAccount = $this->getId();
        return Lead::query($q, $idAccount);
    }

    /**
     * addLead
     *
     * @param Lead $l
     * @return Closure
     */
    public function addLead(Lead $l, $overwrite = Lead::CREATE)
    {
        $idAccount = $this->getId();
        return $l->save($idAccount, $overwrite);
    }

    /**
     * addLeadTag
     *
     * @param Tag $tag
     * @throws Exception
     * @return Tag
     */
    public function addLeadTag(Tag $tag)
    {
        try {
            $idAccount = $this->getId();
            return $tag->save($idAccount);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * addLeadHardTag
     *
     * @param HardTag $hardTag
     * @return HardTag
     */
    public function addLeadHardTag(HardTag $hardTag)
    {
        try {
            $idAccount = $this->getId();
            return $hardTag->save($idAccount);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * leadCampaign
     *
     * @param Lead $lead
     * @param $field
     * @param $description
     * @return bool
     * @throws Exception
     */
    public function leadCampaign(Lead $lead, $field, $description)
    {
        try {
            $idAccount = $this->getId();
            return $lead->campaign($idAccount, $field, $description);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * removeLead
     *
     * @param $idLead
     * @return mixed
     */
    public function removeLead($idLead)
    {
        $idAccount = $this->getId();
        try {
            return Lead::delete($idAccount, $idLead);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * services
     *
     * @return array
     * @throws Exception
     */
    public function services()
    {
        return Service::all($this->getId());
    }

    /**
     * apis
     *
     * @return array
     * @throws Exception
     */
    public function apis($public = true)
    {
        return Api::all($this->getId());
    }

    /**
     * addApi
     *
     * @return array
     * @throws Exception
     */
    public function addApi(Api $api)
    {
        return $api->save($this->getId());
    }

    /**
     * removeApi
     *
     * @param Api $api
     * @return $this
     * @throws Exception
     */
    public function removeApi(Api $api)
    {
        return $api->delete($this->getId());
    }

    /**
     * isVerified
     *
     * Check if the account is verified
     *
     * @return bool
     */
    public function isVerified()
    {
        return $this->getVerified() == true;
    }

    /**
     * getConfig
     *
     * Get configurations for the account
     * @return Config
     */
    public function getConfig()
    {
        return Config::getConfig($this->getId());
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson($options = 0)
    {
        return json_encode($this->toArray(), $options);
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "role" && $this->$attribute instanceof Role) {
                $arr[$attribute] = $this->$attribute->toArray();
            } elseif ($attribute != "role") {
                $arr[$attribute] = $attribute == "companyName"  ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Account
     */
    public static function createInstance($data)
    {
        $accInstance = new self();
        $accInstance->setId($data['id_account']);
        $accInstance->setToken($data['account_token']);
        $accInstance->setAlias($data['account_alias']);
        $accInstance->setCompanyName(html_entity_decode($data['company_name'], ENT_COMPAT, 'UTF-8'));
        $accInstance->setEmail($data['email']);
        $accInstance->setPhone($data['phone']);
        $accInstance->setAddress(html_entity_decode($data['address'], ENT_COMPAT, 'UTF-8'));
        $accInstance->setIdIndustryType($data['id_type_industry']);
        $accInstance->setCountryIso($data['country_iso']);
        $accInstance->setWebsite($data['website']);
        $accInstance->setTimezone($data['timezone']);
        $accInstance->setVerified($data['verified']);
        $accInstance->setActive($data['active']);

        if (isset($data['id_role'])) {
            $role = Role::createInstance(['id_role'=>$data['id_role'],'description'=>$data['description']]);
            $accInstance->setRole($role);
        }

        return $accInstance;
    }

    /**
     * __toString
     *
     * @return string
     */
    public function __toString()
    {
        return "My id is: {$this->getId()}, My Token is: {$this->getToken()}, My Email is: {$this->getEmail()}\n";
    }
}
