<?php namespace Stage\Account\Config;

use PDO;
use Exception;
use Stage\Stage;

class Config
{
    const ASSET_IMAGE                    = "asset.image";           // Its the CDN
    const UPLOAD_ASSETS                  = "upload.asset.path";     // The path where the uploads are made
    const STORAGE_ASSETS                 = "storage.asset.path";    // Where are located the asset/image on disk
    const UPLOAD_ASSET_MAX_FILESIZE      = "upload.max.filesize";   // Allowed Upload max fileSize
    const ASSET_UPLOAD_ALLOWED_MIMETYPES = "upload.asset.mimetypes";// Allowed upload mimeTypes
    const ASSET_QUALITY                  = "asset.quality";         // The asset quality
    const UPLOAD_ASSET_CHUNKS            = "upload.asset.chunks.path";         // Asset chunk path
    const ASSET_SERVICE_PATH             = "asset.service.path";
    const ASSET_UPLOAD_PATH              = "asset.upload.path";

    const ASSET_IMAGE_ID                 = "asset.image.id";
    const STORAGE_ASSETS_ID              = "storage.asset.path.id";

    const VIDEO_IMAGE                    = "video.image";           // Its the CDN to the video thumbnail
    const STORAGE_THUMBNAILS             = "video.storage.thumbnails.path";      // Its the path where the thumbnails are located
    const UPLOAD_VIDEO                   = "upload.video.path";     // The path where the uploads chunks are made
    const STORAGE_VIDEO                  = "storage.video.path";    // Final destination of video
    const UPLOAD_VIDEO_MAX_FILESIZE      = "video.max.filesize";    // Allowed Upload max fileSize
    const UPLOAD_VIDEO_CHUNKS            = "upload.video.chunks.path";   // Video chunks path
    const VIDEO_UPLOAD_ALLOWED_MIMETYPES = "upload.video.mimetypes";// Allowed upload mimeTypes
    const VIDEO_QUALITY                  = "video.quality";         // The video quality
    const VIDEO_VERSIONS_PATH            = "video.versions.path";

    const VIDEO_H264                     = "video.h264h";
    const VIDEO_OGG                      = "video.ogg";
    const VIDEO_WEBM                     = "video.webm";
    const VIDEO_WEB_VERSION              = "video.web.version";
    const VIDEO_MOBILE_VERSION           = "video.mobile.version";
    const VIDEO_DEFAULT_VERSION          = "video.default.version";
    const VIDEO_CDN_VERSIONS             = "video.public.path";     //  Its the CDN path pointing to the video
    const VIDEO_CDN_SOURCE               = "video.cdn.source";
    const VIDEO_SERVICE_PATH             = "video.service.path";
    const VIDEO_UPLOAD_PATH              = "video.upload.path";

    const VIDEO_ABR_APPLICATION         = "video.abr.application";
    const VIDEO_CDN_SOURCE_ID           = "video.cdn.source.id";
    const VIDEO_CDN_ABR                 = "video.cdn.abr";
    const VIDEO_CDN_VERSIONS_ID         = "video.cdn.versions.id";
    const VIDEO_CDN_ABR_ID              = "video.cdn.id";
    const STORAGE_VIDEO_ID              = "video.source.id";
    const VIDEO_VERSIONS_PATH_ID        = "video.versions.id";
    const PLAY_HOST                     = "play.host";

    const ARCHIVE_CDN                    = "archive.public.path";
    const ARCHIVE_CDN_SOURCE             = "archive.cdn.source";
    const ARCHIVE_CDN_SOURCE_CLIPS       = "archive.cdn.source.clips";
    const ARCHIVE_CDN_SOURCE_CLIPS_PREVIEWS = "archive.cdn.source.clips.previews";
    const ARCHIVE_CDN_SOURCE_PREVIEWS    = "archive.cdn.source.previews";
    const STORAGE_ARCHIVE                = "storage.archive.path";
    const STORAGE_ARCHIVE_PREVIEWS       = "storage.archive.previews.path";
    const STORAGE_ARCHIVE_CLIPS          = "storage.archive.clips.path";
    const STORAGE_ARCHIVE_CLIPS_PREVIEWS = "storage.archive.clips.previews.path";
    const UPLOAD_ARCHIVE                 = "upload.archive.path";     // The path where the uploads are made
    const UPLOAD_ARCHIVE_CHUNKS          = "upload.archive.chunks.path";
    const ARCHIVE_QUALITY                = "archive.quality";         // The video quality
    const ARCHIVE_VERSIONS_PATH          = "archive.versions.path";
    const UPLOAD_ARCHIVE_MAX_FILESIZE    = "archive.max.filesize";    // Allowed Upload max fileSize
    const ARCHIVE_STORAGE_THUMBNAILS     = "archive.storage.thumbnails.path";
    const ARCHIVE_UPLOAD_ALLOWED_MIMETYPES   = "archive.video.mimetypes";
    const ARCHIVE_CDN_THUMBNAIL              = "archive.cdn.thumbnail";
    const ARCHIVE_SERVICE_PATH               = "archive.service.path";
    const ARCHIVE_UPLOAD_PATH                = "archive.upload.path";
    const ARCHIVE_TRANSFER_PATH              = "archive.transfer.path";

    const ARCHIVE_CDN_SOURCE_ID                 = "archive.cdn.source.id";
    const ARCHIVE_CDN_SOURCE_CLIPS_ID           = "archive.cdn.source.clips.id";
    const ARCHIVE_CDN_SOURCE_CLIPS_PREVIEWS_ID  = "archive.cdn.source.clips.previews.id";
    const ARCHIVE_CDN_SOURCE_PREVIEWS_ID        = "archive.cdn.source.previews.id";
    const STORAGE_ARCHIVE_ID                    = "archive.source.id";
    const STORAGE_ARCHIVE_CLIPS_ID              = "archive.source.clips.id";
    const STORAGE_ARCHIVE_PREVIEWS_ID           = "archive.source.previews.id";
    const STORAGE_ARCHIVE_CLIPS_PREVIEWS_ID     = "archive.source.clips.previews.id";

    const LOG_ENABLE                     = "log.enable";            // Log is enable?
    const LOG_STORAGE                    = "log.storage.path";      // The path to log files

    const PUBLISH_VERSIONS               = "publish.version";       // The versions to publish separated with comma

    const ACCOUNTS_SERVICE_PATH           = "accounts.service.path";
    const LEADS_UPLOAD_PATH               = "leads.service.path";
    const POSTS_UPLOAD_PATH               = "posts.upload.path";
    const POSTS_SERVICE_PATH              = "posts.service.path";

    const POSTS_SITEMAP_PATH              = "posts.sitemap.path";
    const POSTS_FEEDS_PATH                = "posts.feeds.path";

    const POSTS_INTERNAL_UPLOAD_MAX_SIZE   = "posts.internal.upload.max.size";
    const POSTS_INTERNAL_INTERNAL_UPTYPES  = "posts.internal.uptypes";
    const VIDEO_INTERNAL_UPLOAD_MAX_SIZE   = "video.internal.upload.max.size";
    const VIDEO_INTERNAL_INTERNAL_UPTYPES  = "video.internal.uptypes";
    const ASSET_INTERNAL_UPLOAD_MAX_SIZE   = "asset.internal.upload.max.size";
    const ASSET_INTERNAL_INTERNAL_UPTYPES  = "asset.internal.uptypes";

    const ASSET_INTERNAL_INTERNAL_UPLOAD_PATH  = "asset.internal.upload.path";
    const VIDEO_INTERNAL_INTERNAL_UPLOAD_PATH  = "video.internal.upload.path";
    const POSTS_INTERNAL_INTERNAL_UPLOAD_PATH  = "posts.internal.upload.path";

    const ASSETS_OIR_CACHE_PATH         = "assets.oir.cache.path";
    const ARCHIVE_OIR_CACHE_PATH        = "archive.oir.cache.path";
    const VIDEO_OIR_CACHE_PATH          = "video.oir.cache.path";
    const VIDEO_FEEDS_PATH              = "video.feeds.path";
    const VIDEO_SITEMAP_PATH            = "video.sitemap.path";

    const POSTS_SIZE_OF_LATEST          = "posts.size.latest";
    const POSTS_SIZE_OF_FEED            = "posts.size.feed";

    const ASSETS_SOURCES_RAW_PREVIEWS   = "assets.sources.raw.previews";

    const UPLOAD_ADS                        = "upload.ads.path";
    const UPLOAD_AD_MAX_FILESIZE            = "ads.max.filesize";
    const ADS_UPLOAD_ALLOWED_MIMETYPES      = "ads.video.mimetypes";
    const ADS_INTERNAL_UPLOAD_MAX_SIZE      = "ads.internal.upload.max.size";
    const ADS_INTERNAL_INTERNAL_UPTYPES     = "ads.internal.uptypes";
    const ADS_SERVICE_PATH                  = "ads.service.path";
    const ADS_UPLOAD_PATH                   = "ads.upload.path";
    const ADS_INTERNAL_INTERNAL_UPLOAD_PATH = "ads.internal.upload.path";
    const ADS_QUALITY                       = "ads.quality";
    const ADS_H264                          = "ads.h264";
    const ADS_OGG                           = "ads.ogg";
    const ADS_WEBM                          = "ads.webm";
    const ADS_WEB_VERSION                   = "ads.web.version";
    const ADS_MOBILE_VERSION                = "ads.mobile.version";
    const ADS_DEFAULT_VERSION               = "ads.default.version";
    const ADS_OIR_CACHE_PATH                = "ads.oir.cache.path";
    const ADS_VERSIONS_PATH                 = "ads.versions.path";
    const UPLOAD_ADS_CHUNKS                 = "upload.ads.chunks.path";
    const ADS_STORAGE_THUMBNAILS            = "ads.storage.thumbnails.path";
    const ADS_CDN_VERSIONS                  = "ads.cdn.versions";
    const ADS_CDN_SOURCE                    = "ads.cdn.source";
    const ADS_CDN_THUMBNAIL                 = "ads.cdn.thumbnail";
    const STORAGE_ADS                       = "storage.ads.path";
    const VAST_SERVICE_PATH                 = "vast.service.path";
    const SVAS_SERVICE_PATH                 = "svas.service.path";

    const ADS_CDN_ABR                       = "ads.cdn.abr";
    const STORAGE_ADS_ID                    = "storage.ads.path.id";
    const ADS_CDN_SOURCE_ID                 = "ads.cdn.source.id";
    const STORAGE_ADS_VERSIONS_ID           = "ads.storage.versions.id";
    const ADS_CDN_VERSIONS_ID               = "ads.cdn.versions.id";
    const ADS_PLAY_HOST                     = "ads.play.host";
    const ADS_ABR_APPLICATION               = "ads.abr.application";

    const WEBVTT_SOURCE                     = "webvtt.source.path";
    const CDN_WEBVTT_SOURCE                 = "webvtt.cdn.path";
    const WEBVTT_ALLOWED_MIMETYPES          = "webvtt.allowed.mimetypes";

    const LIVE_SERVICE_PATH                 = "live.service.path";

    const ASSET_DOWNLOADER_PATH             = "asset.downloader.path";
    const ASSET_TRANSFER_PATH               = "asset.transfer.path";

    const ARCHIVE_DOWNLOADER_PATH           = "archive.downloader.path";
    const MONGO_DB_HOST                     = "mongo.db.host";
    const MONGO_DB_PORT                     = "mongo.db.port";

    const VAST_ENABLED                     = "vast.enabled";
    const SVAS_ENABLED                     = "svas.enabled";
    const ASSET_PUBLISH_ENABLED            = "assets.publish.enabled";

    const COMPRESS_CDN                     = "compress.cdn";
    const COMPRESS_SOURCE                  = "compress.source";

    const VIDEO_PUBLISH_OTT_ENABLED        = "video.publish.ott.enabled";

    /**
     * @var
     */
    private $options = [];

    public function __construct()
    {
    }

    /**
     * config
     *
     * @param $config
     * @return string
     */
    public function get($config)
    {
        return isset($this->options[$config]) ? $this->options[$config] : "";
    }

    /**
     * getConfig
     *
     * @param $idAccount
     * @return Config
     * @throws Exception
     */
    public static function getConfig($idAccount)
    {
        try {
            $name = "ACCOUNT_SETTINGS";
            $method = "GET";
            $parameters = [[$idAccount, PDO::PARAM_STR]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == -2) {
                throw new Exception("Configuration for account $idAccount not found.");
            }

            $self = new self();

            if (isset($data[0]['asset_cdn'])) {
                $self->options[self::ASSET_IMAGE]               = $data[0]['asset_cdn'];
                $self->options[self::UPLOAD_ASSETS]             = $data[0]['asset_upload'];
                $self->options[self::STORAGE_ASSETS]            = $data[0]['asset_source'];
                $self->options[self::UPLOAD_ASSET_MAX_FILESIZE] = $data[0]['asset_upload_max_size'];
                $self->options[self::ASSET_UPLOAD_ALLOWED_MIMETYPES]  = $data[0]['asset_uptypes'];
                $self->options[self::ASSET_QUALITY]             = $data[0]['asset_quality'];
                $self->options[self::UPLOAD_ASSET_CHUNKS]        = $data[0]['asset_chunks'];
                $self->options[self::ASSET_SERVICE_PATH]        = $data[0]['asset_service_path'];
                $self->options[self::ASSET_UPLOAD_PATH]        = $data[0]['asset_upload_path'];
                $self->options[self::ASSET_IMAGE_ID]           = isset($data[0]['asset_cdn_id']) ? $data[0]['asset_cdn_id'] : "";
                $self->options[self::STORAGE_ASSETS_ID]        = isset($data[0]['asset_source_id']) ? $data[0]['asset_source_id'] : "";
                $self->options[self::ASSET_DOWNLOADER_PATH]    = isset($data[0]['asset_downloader_path']) ? $data[0]['asset_downloader_path'] : "";
                $self->options[self::ASSET_TRANSFER_PATH]      = isset($data[0]['asset_transfer_path']) ? $data[0]['asset_transfer_path'] : "";

                // $self->options[self::MONGO_DB_HOST]                    = $data[0]['ads_upload'];
                // $self->options[self::MONGO_DB_PORT]                    = $data[0]['ads_upload'];
            }

            if (isset($data[0]['video_cdn_source'])) {
                $self->options[self::VIDEO_CDN_VERSIONS]        = isset($data[0]['video_cdn_versions']) ? $data[0]['video_cdn_versions'] : "";
                $self->options[self::VIDEO_CDN_SOURCE]          = isset($data[0]['video_cdn_source']) ? $data[0]['video_cdn_source'] : "";
                $self->options[self::VIDEO_IMAGE]               = $data[0]['video_cdn_thumbnail'];
                $self->options[self::STORAGE_THUMBNAILS]        = $data[0]['video_thumbnail'];
                $self->options[self::UPLOAD_VIDEO]              = $data[0]['video_upload'];
                $self->options[self::STORAGE_VIDEO]             = $data[0]['video_source'];
                $self->options[self::UPLOAD_VIDEO_MAX_FILESIZE] = $data[0]['video_upload_max_size'];
                $self->options[self::VIDEO_UPLOAD_ALLOWED_MIMETYPES]  = $data[0]['video_uptypes'];
                $self->options[self::VIDEO_QUALITY]             = $data[0]['video_quality'];
                $self->options[self::UPLOAD_VIDEO_CHUNKS]       = $data[0]['video_chunks'];
                $self->options[self::VIDEO_VERSIONS_PATH]       = $data[0]['video_versions'];

                $self->options[self::VIDEO_H264]                = $data[0]['video_h264'];
                $self->options[self::VIDEO_OGG]                 = $data[0]['video_ogg'];
                $self->options[self::VIDEO_WEBM]                = $data[0]['video_webm'];
                $self->options[self::VIDEO_WEB_VERSION]         = $data[0]['video_web_version'];
                $self->options[self::VIDEO_MOBILE_VERSION]      = $data[0]['video_mobile_version'];
                $self->options[self::VIDEO_DEFAULT_VERSION]     = $data[0]['video_default_version'];
                $self->options[self::VIDEO_SERVICE_PATH]        = $data[0]['video_service_path'];
                $self->options[self::VIDEO_UPLOAD_PATH]         = $data[0]['video_upload_path'];

                $self->options[self::WEBVTT_SOURCE]             = $data[0]['video_webvtt_source'];
                $self->options[self::CDN_WEBVTT_SOURCE]         = $data[0]['video_cdn_webvtt_source'];
                $self->options[self::WEBVTT_ALLOWED_MIMETYPES]  = $data[0]['video_webvtt_uptypes'];

                $self->options[self::VIDEO_CDN_ABR]             = isset($data[0]['video_cdn_abr']) ? $data[0]['video_cdn_abr'] : "";
                $self->options[self::VIDEO_CDN_VERSIONS_ID]     = isset($data[0]['video_cdn_versions_id']) ? $data[0]['video_cdn_versions_id'] : "";
                $self->options[self::VIDEO_CDN_ABR_ID]          = isset($data[0]['video_cdn_abr_id']) ? $data[0]['video_cdn_abr_id'] : "";
                $self->options[self::STORAGE_VIDEO_ID]          = isset($data[0]['video_source_id']) ? $data[0]['video_source_id'] : "";
                $self->options[self::VIDEO_VERSIONS_PATH_ID]    = isset($data[0]['video_versions_id']) ? $data[0]['video_versions_id'] : "";
                $self->options[self::VIDEO_CDN_SOURCE_ID]       = isset($data[0]['video_cdn_source_id']) ? $data[0]['video_cdn_source_id'] : "";
                $self->options[self::VIDEO_ABR_APPLICATION]     = isset($data[0]['video_abr_application']) ? $data[0]['video_abr_application'] : "";
                $self->options[self::PLAY_HOST]                 = isset($data[0]['video_play_path']) ? $data[0]['video_play_path'] : "";
            }

            if (isset($data[0]['archive_source'])) {
                $self->options[self::ARCHIVE_CDN]                 = $data[0]['archive_cdn'];
                $self->options[self::ARCHIVE_CDN_SOURCE]          = $data[0]['archive_cdn_source'];
                $self->options[self::ARCHIVE_CDN_SOURCE_CLIPS]    = $data[0]['archive_cdn_source_clips'];
                $self->options[self::ARCHIVE_CDN_SOURCE_CLIPS_PREVIEWS]    = $data[0]['archive_cdn_source_clips_previews'];
                $self->options[self::ARCHIVE_CDN_SOURCE_PREVIEWS]    = $data[0]['archive_cdn_source_previews'];
                $self->options[self::ARCHIVE_CDN_THUMBNAIL]       = $data[0]['archive_cdn_thumbnail'];
                $self->options[self::STORAGE_ARCHIVE]             = $data[0]['archive_source'];
                $self->options[self::STORAGE_ARCHIVE_PREVIEWS]    = $data[0]['archive_source_previews'];
                $self->options[self::STORAGE_ARCHIVE_CLIPS]       = $data[0]['archive_source_clips'];
                $self->options[self::STORAGE_ARCHIVE_CLIPS_PREVIEWS] = $data[0]['archive_source_clips_previews'];
                $self->options[self::ARCHIVE_VERSIONS_PATH]       = $data[0]['archive_versions'];
                $self->options[self::UPLOAD_ARCHIVE_CHUNKS]       = $data[0]['archive_chunks'];
                $self->options[self::ARCHIVE_STORAGE_THUMBNAILS]  = $data[0]['archive_thumbnail'];
                $self->options[self::UPLOAD_ARCHIVE]              = $data[0]['archive_upload'];
                $self->options[self::UPLOAD_ARCHIVE_MAX_FILESIZE] = $data[0]['archive_upload_max_size'];
                $self->options[self::ARCHIVE_UPLOAD_ALLOWED_MIMETYPES]  = $data[0]['archive_uptypes'];
                $self->options[self::ARCHIVE_QUALITY]                   = $data[0]['archive_quality'];
                $self->options[self::ARCHIVE_SERVICE_PATH]               = $data[0]['archive_service_path'];
                $self->options[self::ARCHIVE_UPLOAD_PATH]                = $data[0]['archive_upload_path'];
                $self->options[self::ARCHIVE_TRANSFER_PATH]              = $data[0]['archive_transfer_path'];

                $self->options[self::ARCHIVE_CDN_SOURCE_ID]                 = isset($data[0]['archive_cdn_source_id']) ? $data[0]['archive_cdn_source_id'] : "";
                $self->options[self::ARCHIVE_CDN_SOURCE_CLIPS_ID]           = isset($data[0]['archive_cdn_source_clips_id']) ? $data[0]['archive_cdn_source_clips_id'] : "";
                $self->options[self::ARCHIVE_CDN_SOURCE_PREVIEWS_ID]        = isset($data[0]['archive_cdn_source_previews_id']) ? $data[0]['archive_cdn_source_previews_id'] : "";
                $self->options[self::ARCHIVE_CDN_SOURCE_CLIPS_PREVIEWS_ID]  = isset($data[0]['archive_cdn_source_clips_previews_id']) ? $data[0]['archive_cdn_source_clips_previews_id'] : "";
                $self->options[self::STORAGE_ARCHIVE_ID]                    = isset($data[0]['archive_source_id']) ? $data[0]['archive_source_id'] : "";
                $self->options[self::STORAGE_ARCHIVE_CLIPS_ID]              = isset($data[0]['archive_source_clips_id']) ? $data[0]['archive_source_clips_id'] : "";
                $self->options[self::STORAGE_ARCHIVE_PREVIEWS_ID]           = isset($data[0]['archive_source_previews_id']) ? $data[0]['archive_source_previews_id'] : "";
                $self->options[self::STORAGE_ARCHIVE_CLIPS_PREVIEWS_ID]     = isset($data[0]['archive_source_clips_previews_id']) ? $data[0]['archive_source_clips_previews_id'] : "";

                $self->options[self::ARCHIVE_DOWNLOADER_PATH]     = isset($data[0]['archive_downloader_path']) ? $data[0]['archive_downloader_path'] : "";
                $self->options[self::ARCHIVE_TRANSFER_PATH]     = isset($data[0]['archive_transfer_path']) ? $data[0]['archive_transfer_path'] : "";
            }

            if (isset($data[0]['live_service_path'])) {
                $self->options[self::LIVE_SERVICE_PATH]              = $data[0]['live_service_path'];
            }

            $self->options[self::LOG_ENABLE]               = (bool) $data[0]['logs_active'];
            $self->options[self::LOG_STORAGE]              = $data[0]['logs_source'];
            $self->options[self::PUBLISH_VERSIONS]         = $data[0]['api_versions'];

            $self->options[self::ACCOUNTS_SERVICE_PATH]    = $data[0]['accounts_service_path'];
            $self->options[self::LEADS_UPLOAD_PATH]        = $data[0]['leads_service_path'];
            $self->options[self::POSTS_SERVICE_PATH]       = $data[0]['posts_service_path'];
            $self->options[self::POSTS_UPLOAD_PATH]        = $data[0]['posts_upload_path'];

            $self->options[self::POSTS_SITEMAP_PATH]             = $data[0]['posts_sitemap_path'];
            $self->options[self::POSTS_FEEDS_PATH]               = $data[0]['posts_feeds_path'];

            $self->options[self::POSTS_INTERNAL_UPLOAD_MAX_SIZE] = $data[0]['posts_internal_upload_max_size'];
            $self->options[self::POSTS_INTERNAL_INTERNAL_UPTYPES]= $data[0]['posts_internal_uptypes'];
            $self->options[self::VIDEO_INTERNAL_UPLOAD_MAX_SIZE] = $data[0]['video_internal_upload_max_size'];
            $self->options[self::VIDEO_INTERNAL_INTERNAL_UPTYPES]= $data[0]['video_internal_uptypes'];
            $self->options[self::ASSET_INTERNAL_UPLOAD_MAX_SIZE] = $data[0]['asset_internal_upload_max_size'];
            $self->options[self::ASSET_INTERNAL_INTERNAL_UPTYPES]= $data[0]['asset_internal_uptypes'];

            $self->options[self::ASSET_INTERNAL_INTERNAL_UPLOAD_PATH]= $data[0]['asset_internal_upload_path'];
            $self->options[self::POSTS_INTERNAL_INTERNAL_UPLOAD_PATH]= $data[0]['posts_internal_upload_path'];
            $self->options[self::VIDEO_INTERNAL_INTERNAL_UPLOAD_PATH]= $data[0]['video_internal_upload_path'];

            $self->options[self::ASSETS_OIR_CACHE_PATH]         = $data[0]['asset_oir_cache'];
            $self->options[self::ARCHIVE_OIR_CACHE_PATH]        = $data[0]['archive_oir_cache'];
            $self->options[self::VIDEO_OIR_CACHE_PATH]          = $data[0]['video_oir_cache'];
            $self->options[self::POSTS_SIZE_OF_LATEST]          = $data[0]['posts_size_of_latest'];
            $self->options[self::POSTS_SIZE_OF_FEED]            = $data[0]['posts_size_of_feed'];   // If came zero than is all the results
            $self->options[self::ASSETS_SOURCES_RAW_PREVIEWS]   = $data[0]['asset_source_raw_previews'];

            if (isset($data[0]['ads_source'])) {
                $self->options[self::UPLOAD_ADS]                    = $data[0]['ads_upload'];
                $self->options[self::UPLOAD_AD_MAX_FILESIZE]        = $data[0]['ads_upload_max_size'];
                $self->options[self::ADS_UPLOAD_ALLOWED_MIMETYPES]  = $data[0]['ads_uptypes'];
                $self->options[self::ADS_INTERNAL_UPLOAD_MAX_SIZE]  = $data[0]['ads_internal_upload_max_size'];
                $self->options[self::ADS_INTERNAL_INTERNAL_UPTYPES] = $data[0]['ads_internal_uptypes'];
                $self->options[self::ADS_SERVICE_PATH]              = $data[0]['ads_service_path'];
                $self->options[self::ADS_UPLOAD_PATH]               = $data[0]['ads_upload_path'];
                $self->options[self::ADS_INTERNAL_INTERNAL_UPLOAD_PATH]   = $data[0]['ads_internal_upload_path'];
                $self->options[self::ADS_QUALITY]                   = $data[0]['ads_quality'];
                $self->options[self::ADS_H264]                      = $data[0]['ads_h264'];
                $self->options[self::ADS_OGG]                       = $data[0]['ads_ogg'];
                $self->options[self::ADS_WEBM]                      = $data[0]['ads_webm'];
                $self->options[self::ADS_WEB_VERSION]               = $data[0]['ads_web_version'];
                $self->options[self::ADS_MOBILE_VERSION]            = $data[0]['ads_mobile_version'];
                $self->options[self::ADS_DEFAULT_VERSION]           = $data[0]['ads_default_version'];
                $self->options[self::ADS_OIR_CACHE_PATH]            = $data[0]['ads_oir_cache'];
                $self->options[self::ADS_VERSIONS_PATH]             = $data[0]['ads_versions'];
                $self->options[self::UPLOAD_ADS_CHUNKS]             = $data[0]['ads_chunks'];
                $self->options[self::ADS_STORAGE_THUMBNAILS]        = $data[0]['ads_thumbnail'];
                $self->options[self::ADS_CDN_VERSIONS]              = $data[0]['ads_cdn_versions'];
                $self->options[self::ADS_CDN_SOURCE]                = $data[0]['ads_cdn_source'];
                $self->options[self::ADS_CDN_THUMBNAIL]             = $data[0]['ads_cdn_thumbnail'];
                $self->options[self::STORAGE_ADS]                   = $data[0]['ads_source'];
                $self->options[self::VAST_SERVICE_PATH]             = isset($data[0]['ads_vast_path']) ? $data[0]['ads_vast_path'] : "";
                $self->options[self::SVAS_SERVICE_PATH]             = isset($data[0]['ads_svas_path']) ? $data[0]['ads_svas_path'] : "";

                $self->options[self::ADS_CDN_ABR]                   = isset($data[0]['ads_cdn_abr']) ? $data[0]['ads_cdn_abr'] : "";
                $self->options[self::STORAGE_ADS_ID]                = isset($data[0]['ads_source_id']) ? $data[0]['ads_source_id'] : "";
                $self->options[self::ADS_CDN_SOURCE_ID]             = isset($data[0]['ads_cdn_source_id']) ? $data[0]['ads_cdn_source_id'] : "";
                $self->options[self::STORAGE_ADS_VERSIONS_ID]       = isset($data[0]['ads_versions_id']) ? $data[0]['ads_versions_id'] : "";
                $self->options[self::ADS_CDN_VERSIONS_ID]           = isset($data[0]['ads_cdn_versions_id']) ? $data[0]['ads_cdn_versions_id'] : "";

                $self->options[self::ADS_PLAY_HOST]                 = isset($data[0]['ads_play_path']) ? $data[0]['ads_play_path'] : "";
                $self->options[self::ADS_ABR_APPLICATION]         = isset($data[0]['ads_abr_application']) ? $data[0]['ads_abr_application'] : "";
            }

            $self->options[self::VAST_ENABLED]         = isset($data[0]['ads_vast_enabled']) ? (bool) $data[0]['ads_vast_enabled'] : "";
            $self->options[self::SVAS_ENABLED]         = isset($data[0]['ads_svas_enabled']) ? (bool) $data[0]['ads_svas_enabled'] : "";
            $self->options[self::ASSET_PUBLISH_ENABLED]         = isset($data[0]['asset_publish_enabled']) ? (bool) $data[0]['asset_publish_enabled'] : "";

            $self->options[self::COMPRESS_CDN]         = isset($data[0]['asset_compress_cdn']) ? $data[0]['asset_compress_cdn'] : "";
            $self->options[self::COMPRESS_SOURCE]      = isset($data[0]['asset_compress_source']) ? $data[0]['asset_compress_source'] : "";

            $self->options[self::VIDEO_PUBLISH_OTT_ENABLED]      = isset($data[0]['video_publish_ott_enabled']) ? $data[0]['video_publish_ott_enabled'] : "";

            return $self;
        } catch (Exception $e) {
            throw $e;
        }
    }

    public function __toString()
    {
        return implode(";", array_map(function ($keys, $option) {
            return $keys."=".$option;
        }, array_keys($this->options), $this->options));
    }
}
