<?php namespace Stage\Account\Settings;

use PDO;
use Exception;
use Stage\Stage as Stage;
use Stage\Account\Account as Account;
use Service\Connection as Connection;

class Database
{

    /**
     * @var
     */
    private static $instance;

    /**
     * @var
     */
    private $host;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $leadsHost;

    /**
     * @var
     */
    private $leadsName;

    /**
     * @var
     */
    private $created;

    private function __construct()
    {
    }

    /**
     * getInstance
     *
     * @return Database
     */
    public static function getInstance()
    {
        if (!self::$instance) {
            self::$instance = new self();
        }

        return self::$instance;
    }

    public function getHost()
    {
        return $this->host;
    }

    public function getLeadsHost()
    {
        return $this->leadsHost;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getLeadsName()
    {
        return $this->leadsName;
    }

    public function getCreated()
    {
        return $this->created;
    }

    public function setHost($host)
    {
        $this->host = $host;
    }

    public function setLeadsHost($hostname)
    {
        $this->leadsHost = $hostname;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setLeadsName($databaseName)
    {
        $this->leadsName = $databaseName;
    }

    public function setCreated($created)
    {
        $this->created = (bool) $created;
    }

    /**
     * getSettings
     *
     * @param $idAccount
     */
    public function getSettings($account)
    {
        $idAccount = $account instanceof Account ? $account->getId() : $account;
        $parameters = [[$idAccount, PDO::PARAM_INT]];

        // Make a request to Database
        try {
            $name = "ACCOUNT_ACCESS_CONFIG";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            if (sizeof($data) > 0 && !isset($data[0]['return_code'])) {
                $accInstance = self::createInstance($data[0]);
                return $accInstance;
            } else {
                throw new Exception("The database settings for the account $idAccount is not created or not exists");
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Database
     */
    public static function createInstance($data)
    {
        $accInstance = new self();
        $accInstance->setName($data['bd_name']);
        $accInstance->setHost($data['bd_host']);
        $accInstance->setLeadsHost($data['bd_leads_host']);
        $accInstance->setLeadsName($data['bd_leads_name']);
        $accInstance->setCreated($data['created']);

        return $accInstance;
    }
}
