<?php namespace Skoiy\Ott\Audience;

use PDO;
use Stage\Stage;
use DateTime;
use Stage\Helper;
use Exception;

class Audience
{
    const ID = 1;

    private $id = -1;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $total = 0;

    /**
     * @var
     */
    private $updateDate;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'name', 'total', 'updateDate', 'registerDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getTotal()
    {
        return $this->total;
    }

    /**
     * @return mixed
     */
    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    /**
     * @return mixed
     */
    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setTotal($total)
    {
        $this->total = (int)$total;
    }

    /**
     * @param mixed $updateDate
     */
    private function setUpdateDate($updateDate)
    {
        $this->updateDate = $updateDate;
    }

    /**
     * @param mixed $registerDate
     */
    private function setRegisterDate($registerDate)
    {
        $this->registerDate = $registerDate;
    }

    /**
     * find
     *
     * @param $id
     * @return Audience
     * @throws Exception
     */
    public static function find($id)
    {
        try {
            $name = "OTT_AUDIENCE";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Audience id $id not exists.", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * findGroupOptions
     *
     * @param $id
     * @return array
     * @throws Exception
     */
    public static function findGroupOptions($id)
    {
        try {
            $name = "OTT_AUDIENCES_GROUP_OPTIONS_RELATED";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];
            return Stage::esp($parameters, $name, $method);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        // table: audiences
        try {
            $name = "OTT_AUDIENCE";
            $method = "SET";
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                [$this->getName(), PDO::PARAM_STR]];

            $data = Stage::esp($parameters, $name, $method);
            if (!isset($data[0]) && !isset($data[0]['id_audience'])) {
                throw new Exception("Error saving a Audience");
            }

            $this->setId($data[0]['id_audience']);
            if (isset($data[0]['update_date']) && Helper::isValidDate($data[0]['update_date'], "Y-m-d H:i:s")) {
                $this->setRegisterDate(new DateTime($data[0]['update_date']));
            }
            if (isset($data[0]['register_date']) && Helper::isValidDate($data[0]['register_date'], "Y-m-d H:i:s")) {
                $this->setRegisterDate(new DateTime($data[0]['register_date']));
            }

            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * query
     *
     * @param Query $q
     * @return array|Audience[]
     * @throws Exception
     */
    public static function query(Query $q)
    {
        try {
            $parameters = [];
            $name = "OTT_AUDIENCES";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return array_map(function ($user) {
                return self::createInstance($user);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * @param $idAudience
     * @return bool
     * @throws Exception
     */
    public static function delete($idAudience)
    {
        try {
            $parameters = [[$idAudience, PDO::PARAM_INT]];
            $name = "OTT_AUDIENCE";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("Audience {$idAudience} not found");
            } else if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0") {
                return true;
            }
            return false;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "updateDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" || $attribute == "nameStripped" || $attribute == "imageTitle" || $attribute == "imageAlterText" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * @param $asset
     * @return Audience
     * @throws Exception
     */
    public static function createInstance($asset)
    {
        if (isset($asset[0]))
            $asset = $asset[0];

        $instance = new self();
        $instance->setId($asset['id_audience']);
        $instance->setName(html_entity_decode($asset['name'], ENT_COMPAT, 'UTF-8'));
        $instance->setTotal(isset($asset['total']) ? $asset['total'] : 0);
        if (isset($asset['update_date']) && Helper::isValidDate($asset['update_date'], "Y-m-d H:i:s")) {
            $instance->setUpdateDate(new DateTime($asset['update_date']));
        }

        if (isset($asset['register_date']) && Helper::isValidDate($asset['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($asset['register_date']));
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }

}