<?php namespace Skoiy\Ott\Audience;

use PDO;
use Stage\Stage;
use DateTime;
use Stage\Helper;
use Exception;

class Group
{
    const ID = 1;

    private $id = -1;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $updateDate;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'updateDate', 'registerDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getName()
    {
        return $this->name;
    }

    /**
     * @return mixed
     */
    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    /**
     * @return mixed
     */
    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function setId($id_plan)
    {
        $this->id = (int)$id_plan;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    /**
     * @param mixed $updateDate
     */
    private function setUpdateDate($updateDate)
    {
        $this->updateDate = $updateDate;
    }

    /**
     * @param mixed $registerDate
     */
    private function setRegisterDate($registerDate)
    {
        $this->registerDate = $registerDate;
    }

    /**
     * saveGroup
     *
     * @param $idAudience
     * @param $operatorType
     * @return $this
     * @throws Exception
     */
    public function save($idAudience, $operatorType)
    {
        // table: audiences_groups
        try {
            $name = "OTT_AUDIENCES_GROUP";
            $method = "SET";
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                [$idAudience, PDO::PARAM_INT],
                [$operatorType, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);

            $this->setId($data[0]['id_group']);
            if (isset($data[0]['update_date']) && Helper::isValidDate($data[0]['update_date'], "Y-m-d H:i:s")) {
                $this->setRegisterDate(new DateTime($data[0]['update_date']));
            }
            if (isset($data[0]['register_date']) && Helper::isValidDate($data[0]['register_date'], "Y-m-d H:i:s")) {
                $this->setRegisterDate(new DateTime($data[0]['register_date']));
            }

            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * @param $idGroup
     * @param $idAudience
     * @return bool
     * @throws Exception
     */
    public static function delete($idAudience, $idGroup)
    {
        try {
            $parameters = [[$idGroup, PDO::PARAM_INT],
                [$idAudience, PDO::PARAM_INT]];
            $name = "OTT_AUDIENCES_GROUP";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("Group {$idGroup} not found");
            } else if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0") {
                return true;
            }
            return false;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachOption
     *
     * @param $idGroup
     * @param $idAudience
     * @param $idOption
     * @param $optionValue
     * @return boolean
     * @throws Exception
     */
    public static function attachOption($idGroup, $idAudience, $idOption, $optionValue)
    {
        // table: audiences_groups_options_related
        try {
            $name = "OTT_AUDIENCES_GROUP_OPTIONS_RELATED";
            $method = "SET";
            $parameters = [[$idGroup, PDO::PARAM_INT],
                [$idAudience, PDO::PARAM_INT],
                [$idOption, PDO::PARAM_INT],
                [$optionValue, PDO::PARAM_STR]];

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code'])) {
                throw new Exception("Error", $data[0]['return_code']);
            } else {
                return isset($data[0]) && isset($data[0]['id_group']);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * detachOption
     *
     * @param $idGroup
     * @param $idAudience
     * @param $idOption
     * @return boolean
     * @throws Exception
     */
    public static function detachOption($idGroup, $idAudience, $idOption)
    {
        // table: audiences_groups_options_related
        try {
            $name = "OTT_AUDIENCES_GROUP_OPTION_RELATED";
            $method = "DEL";
            $parameters = [[$idGroup, PDO::PARAM_INT],
                [$idAudience, PDO::PARAM_INT],
                [$idOption, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] != 0) {
                throw new Exception("Error", $data[0]['return_code']);
            } else {
                return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == 0;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "updateDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" || $attribute == "nameStripped" || $attribute == "imageTitle" || $attribute == "imageAlterText" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * @param $asset
     * @return Audience
     * @throws Exception
     */
    public static function createInstance($asset)
    {
        if (isset($asset[0]))
            $asset = $asset[0];

        $instance = new self();
        $instance->setId($asset['id_audience']);
        $instance->setName(html_entity_decode($asset['name'], ENT_COMPAT, 'UTF-8'));
        if (isset($asset['update_date']) && Helper::isValidDate($asset['update_date'], "Y-m-d H:i:s")) {
            $instance->setUpdateDate(new DateTime($asset['update_date']));
        }

        if (isset($asset['register_date']) && Helper::isValidDate($asset['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($asset['register_date']));
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }

}