<?php namespace Skoiy\Ott\Audience;

use PDO;
use Stage\Stage;
use DateTime;
use Stage\Helper;
use Exception;

class Option
{
    const ID = 1;

    private $id = -1;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $optionName;

    /**
     * @var
     */
    private $subOptionName;

    /**
     * @var
     */
    private $optionType;

    /**
     * @var
     */
    private $valueType;

    /**
     * @var
     */
    private $updateDate;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'name', 'optionName', 'subOptionName', 'optionType', 'valueType', 'updateDate', 'registerDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getOptionName()
    {
        return $this->optionName;
    }

    public function getSubOptionName()
    {
        return $this->subOptionName;
    }

    public function getOptionType()
    {
        return $this->optionType;
    }

    public function getValueType()
    {
        return $this->valueType;
    }

    /**
     * @return mixed
     */
    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    /**
     * @return mixed
     */
    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setOptionName($name)
    {
        $this->optionName = $name;
    }

    public function setSubOptionName($name)
    {
        $this->subOptionName = $name;
    }

    public function setOptionType($type)
    {
        $this->optionType = $type;
    }

    public function setValueType($value)
    {
        $this->valueType = $value;
    }

    /**
     * @param mixed $updateDate
     */
    private function setUpdateDate($updateDate)
    {
        $this->updateDate = $updateDate;
    }

    /**
     * @param mixed $registerDate
     */
    private function setRegisterDate($registerDate)
    {
        $this->registerDate = $registerDate;
    }

    /**
     * query
     *
     * @param Query $q
     * @return array|Option[]
     * @throws Exception
     */
    public static function query(Query $q)
    {
        try {
            $parameters = [
                [$q->field, PDO::PARAM_INT],
                [$q->optionType, PDO::PARAM_INT]
            ];

            $name = "OTT_AUDIENCES_OPTIONS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return array_map(function ($user) {
                return self::createInstance($user);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "updateDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" || $attribute == "nameStripped" || $attribute == "imageTitle" || $attribute == "imageAlterText" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $asset
     * @return Option
     * @throws Exception
     */
    public static function createInstance($asset)
    {
        if (isset($asset[0]))
            $asset = $asset[0];

        $instance = new self();
        $instance->setId($asset['id_option']);
        $instance->setName(html_entity_decode($asset['name'], ENT_COMPAT, 'UTF-8'));
        $instance->setOptionName(html_entity_decode($asset['option_name'], ENT_COMPAT, 'UTF-8'));
        $instance->setSubOptionName(html_entity_decode($asset['sub_option_name'], ENT_COMPAT, 'UTF-8'));
        $instance->setOptionType($asset['option_type']);
        $instance->setValueType($asset['value_type']);
        if (isset($asset['update_date']) && Helper::isValidDate($asset['update_date'], "Y-m-d H:i:s")) {
            $instance->setUpdateDate(new DateTime($asset['update_date']));
        }

        if (isset($asset['register_date']) && Helper::isValidDate($asset['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($asset['register_date']));
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }

}