<?php namespace Skoiy\Ott\Avatar;

use PDO;
use Stage\Stage;
use DateTime;
use Stage\Helper;
use Exception;

class Avatar
{
    const ID = 1;

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $idGroup;

    /**
     * @var
     */
    private $name = "";

    /**
     * @var
     */
    private $avatar;

    /**
     * @var
     */
    private $active;

    /**
     * @var
     */
    private $totalSelected;

    /**
     * @var
     */
    private $isDefault;

    /**
     * @var
     */
    private $position;

    /**
     * @var
     */
    private $updateDate;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'idGroup', 'name', 'avatar', 'active', 'totalSelected', 'isDefault', 'position', 'updateDate', 'registerDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    /**
     * @return mixed
     */
    public function getIdGroup()
    {
        return $this->idGroup;
    }

    /**
     * @return mixed
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * @return mixed
     */
    public function getAvatar()
    {
        return $this->avatar;
    }

    /**
     * @return mixed
     */
    public function getActive()
    {
        return (bool)$this->active;
    }

    /**
     * @return mixed
     */
    public function getBirthdate()
    {
        return $this->birthdate;
    }

    /**
     * @return mixed
     */
    public function getTotalSelected()
    {
        return (int) $this->totalSelected;
    }

    /**
     * @return mixed
     */
    public function isDefault()
    {
        return (bool) $this->isDefault;
    }

    /**
     * @return mixed
     */
    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    /**
     * @return mixed
     */
    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    /**
     * @param mixed $id
     */
    public function setIdAvatar($id)
    {
        $this->id = $id;
    }

    /**
     * @param mixed $id
     */
    public function setIdGroup($id)
    {
        $this->idGroup = (int) $id;
    }

    /**
     * @param mixed $name
     */
    public function setName($name)
    {
        $this->name = $name;
    }

    /**
     * @param mixed $url
     */
    public function setAvatar($url)
    {
        $this->avatar = $url;
    }

    /**
     * @param mixed $active
     */
    public function setActive($active)
    {
        $this->active = (bool)$active;
    }

    /**
     * @param mixed $total
     */
    public function setTotalSelected($total)
    {
        $this->totalSelected = (int) $total;
    }

    /**
     * @param mixed $is
     */
    public function setIsDefault($is)
    {
        $this->isDefault = (bool) $is;
    }

    /**
     * @param mixed $position
     */
    public function setPosition($position)
    {
        $this->position = $position;
    }

    /**
     * @param mixed $updateDate
     */
    public function setUpdateDate($updateDate)
    {
        $this->updateDate = $updateDate;
    }

    /**
     * @param mixed $registerDate
     */
    public function setRegisterDate($registerDate)
    {
        $this->registerDate = $registerDate;
    }

    /**
     * @return array
     * @throws Exception
     */
    public static function sizeof()
    {
        $name = "OTT_AVATARS_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * @param $idUser
     * @return User
     * @throws Exception
     */
    public static function find($idUser)
    {
        try {
            $name = "OTT_AVATAR";
            $method = "GET";
            $parameters = [[$idUser, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The User id $idUser not exists.", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param Query $q
     * @return array|User[]
     * @throws Exception
     */
    public static function query(Query $q)
    {
        $field = $q->by;
        $query = $q->term;
        $filter = $q->filter;
        $limitA = $q->from;
        $limitB = $q->to;
        $orderBy = $q->orderBy;
        $order = $q->order;

        try {
            $parameters = [[$field, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_INT],
                [$orderBy, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT],
                [$limitA, PDO::PARAM_INT],
                [$limitB, PDO::PARAM_INT]];

            $name = "OTT_AVATARS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return array_map(function ($user) {
                return self::createInstance($user);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param Query $q
     * @return mixed
     * @throws Exception
     */
    public static function queryCount(Query $q)
    {
        $field = $q->by;
        $query = $q->term;
        $filter = $q->filter;

        try {
            $parameters = [[$field, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_INT]];

            $name = "OTT_AVATARS_COUNT";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]['total']) ? $data[0]['total'] : 0;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "updateDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" || $attribute == "nameStripped" || $attribute == "imageTitle" || $attribute == "imageAlterText" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * @param $asset
     * @return Avatar
     * @throws Exception
     */
    public static function createInstance($asset)
    {
        $instance = new self();
        $instance->setIdAvatar($asset['id_avatar']);
        $instance->setIdGroup($asset['id_group']);
        $instance->setName(html_entity_decode($asset['name'], ENT_COMPAT, 'UTF-8'));
        $instance->setAvatar($asset['avatar']);
        $instance->setActive($asset['active']);
        $instance->setTotalSelected($asset['total_selected']);
        $instance->setIsDefault($asset['is_default']);
        $instance->setPosition($asset['position']);

        if (isset($asset['update_date']) && Helper::isValidDate($asset['update_date'], "Y-m-d H:i:s")) {
            $instance->setUpdateDate(new DateTime($asset['update_date']));
        }

        if (isset($asset['register_date']) && Helper::isValidDate($asset['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($asset['register_date']));
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}