<?php namespace Skoiy\Ott\Freebie;

use PDO;
use Skoiy\Ott\Plan\Plan;
use Stage\Stage;
use DateTime;
use Stage\Helper;
use Exception;

class Freebie
{
    const ID = 1;

    private $id = -1;
    private $name = "";
    private $nameStripped = "";
    private $beginDate = "";
    private $endDate = "";
    private $appliedToAllPeriods;
    private $billingPeriod;
    private $oneUsePerUser;
    private $isActive = true;
    private $totalPlans = 0;
    /**
     * @var
     */
    private $updateDate;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'name', 'nameStripped', 'beginDate', 'endDate', 'appliedToAllPeriods', 'billingPeriod', 'oneUsePerUser', 'isActive', 'totalPlans', 'updateDate', 'registerDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getNameStripped()
    {
        return $this->nameStripped;
    }

    public function getBeginDate()
    {
        return $this->beginDate;
    }

    public function getEndDate()
    {
        return $this->endDate;
    }

    public function getAppliedToAllPeriods()
    {
        return $this->appliedToAllPeriods;
    }

    public function getBillingPeriod()
    {
        return $this->billingPeriod;
    }

    public function getOneUsePerUser()
    {
        return $this->oneUsePerUser;
    }

    public function isActive()
    {
        return (bool)$this->isActive;
    }

    public function getTotalPlans()
    {
        return $this->totalPlans;
    }

    /**
     * @return mixed
     */
    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    /**
     * @return mixed
     */
    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function setId($id_plan)
    {
        $this->id = (int)$id_plan;
    }

    /**
     * @param mixed $active
     */
    public function setActive($active)
    {
        $this->isActive = (bool)$active;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setNameStripped($name_stripped)
    {
        $this->nameStripped = $name_stripped;
    }

    public function setBeginDate($begin_date)
    {
        $this->beginDate = $begin_date;
    }

    public function setEndDate($end_date)
    {
        $this->endDate = $end_date;
    }

    public function setAppliedToAllPeriods($applied)
    {
        $this->appliedToAllPeriods = $applied;
    }

    public function setBillingPeriod($period)
    {
        $this->billingPeriod = $period;
    }

    public function setOneUsePerUser($perUser)
    {
        $this->oneUsePerUser = $perUser;
    }

    public function setTotalPlans($total)
    {
        $this->totalPlans = (int)$total;
    }

    /**
     * @param mixed $updateDate
     */
    private function setUpdateDate($updateDate)
    {
        $this->updateDate = $updateDate;
    }

    /**
     * @param mixed $registerDate
     */
    private function setRegisterDate($registerDate)
    {
        $this->registerDate = $registerDate;
    }

    /**
     * save
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        try {
            $name = "OTT_FREEBIE";
            $method = "SET";
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                [$this->getName(), PDO::PARAM_STR],
                [$this->getNameStripped(), PDO::PARAM_STR],
                [$this->getBeginDate(), PDO::PARAM_STR],
                [$this->getEndDate(), PDO::PARAM_STR],
                [$this->getAppliedToAllPeriods(), PDO::PARAM_BOOL],
                [$this->getBillingPeriod(), PDO::PARAM_INT],
                [$this->getOneUsePerUser(), PDO::PARAM_BOOL],
                [$this->isActive(), PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            if (!isset($data[0]) && !isset($data[0]['id_plan'])) {
                throw new Exception("Error saving a new Plan");
            }

            $this->setId($data[0]['id_freebie']);
            if (isset($data[0]['update_date']) && Helper::isValidDate($data[0]['update_date'], "Y-m-d H:i:s")) {
                $this->setRegisterDate(new DateTime($data[0]['update_date']));
            }
            if (isset($data[0]['register_date']) && Helper::isValidDate($data[0]['register_date'], "Y-m-d H:i:s")) {
                $this->setRegisterDate(new DateTime($data[0]['register_date']));
            }

            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * duplicate
     *
     * @return Freebie
     * @throws Exception
     */
    public static function duplicate($id)
    {
        try {
            $parameters = [[$id, PDO::PARAM_INT]];
            $name = "OTT_FREEBIE_CLONE";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            return self::createInstance($data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idFreebie
     * @return bool
     * @throws Exception
     */
    public static function delete($idFreebie)
    {
        try {
            $parameters = [[$idFreebie, PDO::PARAM_INT]];
            $name = "OTT_FREEBIE";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("Freebie {$idFreebie} not found");
            } else if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0") {
                return true;
            }
            return false;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * sizeof
     *
     * @return array
     * @throws Exception
     */
    public static function sizeof()
    {
        $name = "OTT_FREEBIES_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['total']) ? $data[0]['total'] : 0;
    }

    /**
     * find
     *
     * @param $idFreebie
     * @return Freebie
     * @throws Exception
     */
    public static function find($idFreebie)
    {
        try {
            $name = "OTT_FREEBIE";
            $method = "GET";
            $parameters = [[$idFreebie, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Freebie id $idFreebie not exists.", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * isoLanguages
     *
     * @param $idFreebie
     * @return array
     * @throws Exception
     */
    public static function isoLanguages($idFreebie)
    {
        try {
            $name = "OTT_FREEBIES_ISOLANGUAGES";
            $method = "GET";
            $parameters = [[$idFreebie, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Freebie id $idFreebie not exists.", $data[0]['return_code']);
            } else {
                return $data;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Get all L10n's for this Freebie
     *
     * @param $idFreebie
     * @return L10n[]
     * @throws Exception
     */
    public static function l10n($idFreebie)
    {
        try {
            return L10n::find($idFreebie);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * query
     *
     * @param FreebieQuery $q
     * @return array|Freebie[]
     * @throws Exception
     */
    public static function query(Query $q)
    {
        $field = $q->by;
        $fieldSnd = $q->sndBy;
        $fieldTrd = $q->trdBy;
        $query = $q->term;
        $filter = $q->filter;
        $filterSnd = $q->sndFilter;
        $filterTrd = $q->trdFilter;
        $limitA = $q->from;
        $limitB = $q->to;
        $orderBy = $q->orderBy;
        $order = $q->order;

        try {
            $parameters = [[$field, PDO::PARAM_INT],
                [$fieldSnd, PDO::PARAM_INT],
                [$fieldTrd, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$filterSnd, PDO::PARAM_STR],
                [$filterTrd, PDO::PARAM_STR],
                [$orderBy, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT],
                [$limitA, PDO::PARAM_INT],
                [$limitB, PDO::PARAM_INT],
            ];

            if ($q instanceof CodeQuery) {
                $name = "OTT_FREEBIE_CODES";
            } else if ($q instanceof EmailQuery) {
                $name = "OTT_FREEBIE_WHITELIST";
            } else if ($q instanceof UsedQuery) {
                $name = "OTT_FREEBIE_USED";
            } else if ($q instanceof FreebieQuery) {
                $name = "OTT_FREEBIES";
            }

            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]) && isset($data[0]['return_code'])) {
                return [];
            }

            return array_map(function ($user) use ($q) {
                if ($q instanceof CodeQuery) {
                    return Code::createInstance($user);
                } else if ($q instanceof EmailQuery) {
                    return WhiteList::createInstance($user);
                } else if ($q instanceof UsedQuery) {
                    return Used::createInstance($user);
                } else if ($q instanceof FreebieQuery) {
                    return self::createInstance($user);
                }
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * queryCount
     *
     * @param FreebieQuery $q
     * @return mixed
     * @throws Exception
     */
    public static function queryCount(Query $q)
    {
        $field = $q->by;
        $fieldSnd = $q->sndBy;
        $fieldTrd = $q->trdBy;
        $query = $q->term;
        $filter = $q->filter;
        $filterSnd = $q->sndFilter;
        $filterTrd = $q->trdFilter;

        try {
            $parameters = [[$field, PDO::PARAM_INT],
                [$fieldSnd, PDO::PARAM_INT],
                [$fieldTrd, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$filterSnd, PDO::PARAM_STR],
                [$filterTrd, PDO::PARAM_STR]];

            if ($q instanceof CodeQuery) {
                $name = "OTT_FREEBIE_CODES_COUNT";
            } else if ($q instanceof EmailQuery) {
                $name = "OTT_FREEBIE_WHITELIST_COUNT";
            } else if ($q instanceof UsedQuery) {
                $name = "OTT_FREEBIE_USED_COUNT";
            } else if ($q instanceof FreebieQuery) {
                $name = "OTT_FREEBIES_COUNT";
            }

            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]['total']) ? $data[0]['total'] : 0;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $id
     * @param $idFreebie
     * @param $value
     * @param $type // type1 = email type2=domain
     * @param $active boolean
     * @return WhiteList
     * @throws Exception
     * @description Writes on table freebies_codes_related
     */
    public static function addToWhitelist($id, $idFreebie, $value, $type, $active)
    {
        try {
            return WhiteList::save($id, $idFreebie, $value, $type, $active);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $type // type1 = email type2=domain
     * @param $idFreebie
     * @param $id
     * @return bool
     * @throws Exception
     * @description Writes on table
     */
    public static function removeFromWhitelist($type, $idFreebie, $id)
    {
        try {
            $parameters = [[$type, PDO::PARAM_INT],
                [$idFreebie, PDO::PARAM_INT],
                [$id, PDO::PARAM_INT]];
            $name = "OTT_FREEBIE_WHITELIST";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0";
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * plans
     *
     * @param $idFreebie
     * @return Plan[]
     * @throws Exception
     */
    public static function plans($idFreebie)
    {
        try {
            $name = "OTT_FREEBIE_PLANS";
            $method = "GET";
            $parameters = [[$idFreebie, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Freebie id $idFreebie not exists.", $data[0]['return_code']);
            } else {
                return array_map(function ($l10n) {
                    return Plan::createInstance($l10n);
                }, $data);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idFreebie
     * @return array|SecurityCountry[]
     * @throws Exception
     */
    public static function securityCountries($idFreebie)
    {
        try {
            $name = "OTT_FREEBIE_SECURITY_COUNTRIES";
            $method = "GET";
            $parameters = [[$idFreebie, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                return [];
            } else {
                return array_map(function ($l10n) {
                    return SecurityCountry::createInstance($l10n);
                }, $data);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * addPlan
     *
     * @return boolean
     * @throws Exception
     * @description Writes on table freebies_plans_related
     */
    public static function addPlan($idFreebie, $idPlan, $position)
    {
        try {
            $parameters = [[$idFreebie, PDO::PARAM_INT],
                [$idPlan, PDO::PARAM_INT],
                [$position, PDO::PARAM_INT]];
            $name = "OTT_FREEBIES_PLANS_RELATED";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0";
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * removePlan
     *
     * @return boolean
     * @throws Exception
     * @description Writes on table promotions_plans_related
     */
    public static function removePlan($type, $idFreebie, $idPlan)
    {
        try {
            $parameters = [[$type, PDO::PARAM_INT],
                [$idFreebie, PDO::PARAM_INT],
                [$idPlan, PDO::PARAM_INT]];
            $name = "OTT_FREEBIE_PLAN";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0";
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * addCode
     *
     * @param $idFreebie
     * @param $code
     * @param $quantity
     * @param $unlimited boolean
     * @param $active boolean
     * @return Code
     * @throws Exception
     * @description Writes on table freebies_codes_related
     */
    public static function addCode($idFreebie, $code, $quantity, $unlimited, $active, $idCode = -1)
    {
        try {
            return Code::save($idCode, $idFreebie, $code, $quantity, $unlimited, $active);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * removeCode
     *
     * @param $type
     * @param $idFreebie
     * @param $idCode
     * @return bool
     * @throws Exception
     * @description Writes on table
     */
    public static function removeCode($type, $idFreebie, $idCode)
    {
        try {
            $parameters = [[$type, PDO::PARAM_INT],
                [$idFreebie, PDO::PARAM_INT],
                [$idCode, PDO::PARAM_INT]];
            $name = "OTT_FREEBIE_CODE";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0";
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * codeExist
     *
     * @param $code
     * @return array
     * @throws Exception
     */
    public static function codeExist($code)
    {
        try {
            $parameters = [[$code, PDO::PARAM_STR]];
            $name = "OTT_FREEBIE_CODE_NUM";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * item
     *
     * @param $id
     * @throws Exception
     */
    public static function item($id)
    {
        // TODO
        try {
            $name = "OTT_FREEBIE_WHITELIST_ITEM";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "updateDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" || $attribute == "nameStripped" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * @param $asset
     * @return Freebie
     * @throws Exception
     */
    public static function createInstance($asset)
    {
        if (isset($asset[0]))
            $asset = $asset[0];

        $instance = new self();
        $instance->setId($asset['id_freebie']);
        $instance->setName(html_entity_decode($asset['name'], ENT_COMPAT, 'UTF-8'));
        $instance->setNameStripped(html_entity_decode($asset['name_stripped'], ENT_COMPAT, 'UTF-8'));
        $instance->setBeginDate($asset['begin_date']);
        $instance->setEndDate($asset['end_date']);
        $instance->setAppliedToAllPeriods($asset['applied_to_all_periods']);
        $instance->setBillingPeriod($asset['billing_period']);
        $instance->setOneUsePerUser($asset['one_use_per_user']);
        if (isset($asset['total_plans'])) {
            $instance->setTotalPlans($asset['total_plans']);
        }
        $instance->setActive($asset['active']);
        if (isset($asset['update_date']) && Helper::isValidDate($asset['update_date'], "Y-m-d H:i:s")) {
            $instance->setUpdateDate(new DateTime($asset['update_date']));
        }

        if (isset($asset['register_date']) && Helper::isValidDate($asset['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($asset['register_date']));
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }

}