<?php namespace Skoiy\Ott\Freebie;

use PDO;
use Stage\Stage;
use DateTime;
use Stage\Helper;
use Exception;

class WhiteList
{
    const ID = 1;

    /**
     * @var
     */
    private $id = -1;
    private $idFreebie;
    private $value;
    private $type;
    private $active;

    /**
     * @var
     */
    private $updateDate;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'idFreebie', 'value', 'type', 'active', 'updateDate', 'registerDate');

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdFreebie()
    {
        return $this->idFreebie;
    }

    public function getValue()
    {
        return $this->value;
    }

    public function getType()
    {
        return $this->type;
    }

    public function isActive()
    {
        return (bool)$this->active;
    }

    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function setId($id)
    {
        $this->id = $id;
    }

    public function setIdFreebie($id)
    {
        $this->idFreebie = $id;
    }

    public function setValue($value)
    {
        $this->value = $value;
    }

    public function setType($type)
    {
        $this->type = $type;
    }

    public function setActive($active)
    {
        $this->active = (bool)$active;
    }

    /**
     * @param mixed $updateDate
     */
    public function setUpdateDate($updateDate)
    {
        $this->updateDate = $updateDate;
    }

    /**
     * @param mixed $registerDate
     */
    public function setRegisterDate($registerDate)
    {
        $this->registerDate = $registerDate;
    }

    /**
     * save
     *
     * @param $id
     * @param $idFreebie
     * @param $value
     * @param $type
     * @param $active boolean
     * @return Email|WhiteList
     * @throws Exception
     * @description Writes on table $id
     */
    public static function save($id, $idFreebie, $value, $type, $active)
    {
        try {
            $parameters = [[$id, PDO::PARAM_INT],
                [$idFreebie, PDO::PARAM_INT],
                [$value, PDO::PARAM_STR],
                [$type, PDO::PARAM_STR],
                [$active, PDO::PARAM_BOOL]];
            $name = "OTT_FREEBIES_WHITELIST_RELATED";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) ) {
                throw new Exception("Return code {$data[0]['return_code']}.", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "updateDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" || $attribute == "nameStripped" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * @param $asset
     * @return Email
     * @throws Exception
     */
    public static function createInstance($asset)
    {
        if (isset($asset[0]) && isset($asset[0]['id']))
            $asset = $asset[0];

        $instance = new self();
        $instance->setId($asset['id']);
        $instance->setIdFreebie($asset['id_freebie']);
        $instance->setValue($asset['value']);
        if(isset($asset['type'])) {
            $instance->setType($asset['type']);
        }
        $instance->setActive($asset['active']);

        if (isset($asset['update_date']) && Helper::isValidDate($asset['update_date'], "Y-m-d H:i:s")) {
            $instance->setUpdateDate(new DateTime($asset['update_date']));
        }

        if (isset($asset['register_date']) && Helper::isValidDate($asset['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($asset['register_date']));
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }

}