<?php namespace Skoiy\Ott\Leaderboard;

use DateTime;
use Stage\Helper;
use Exception;

class L10n
{
    private $id;
    private $idLeaderboard;
    private $isoLanguage;
    private $singularUnit = "";
    private $pluralUnits = "";

    /**
     * @var
     */
    private $updateDate;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'idLeaderboard', 'isoLanguage', 'singularUnit', 'pluralUnits', 'updateDate', 'registerDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdLeaderboard()
    {
        return $this->idLeaderboard;
    }

    public function getIsoLanguage()
    {
        return $this->isoLanguage;
    }

    public function getSingularUnit()
    {
        return $this->singularUnit;
    }

    public function getPluralUnits()
    {
        return $this->pluralUnits;
    }

    /**
     * @return mixed
     */
    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    /**
     * @return mixed
     */
    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setIdLeaderboard($id)
    {
        $this->idLeaderboard = (int)$id;
    }

    public function setIsoLanguage($iso)
    {
        $this->isoLanguage = $iso;
    }

    public function setSingularUnit($name)
    {
        $this->singularUnit = $name;
    }

    public function setPluralUnits($desc)
    {
        $this->pluralUnits = $desc;
    }

    /**
     * @param mixed $updateDate
     */
    private function setUpdateDate($updateDate)
    {
        $this->updateDate = $updateDate;
    }

    /**
     * @param mixed $registerDate
     */
    private function setRegisterDate($registerDate)
    {
        $this->registerDate = $registerDate;
    }

    /**
     * Convert the model instance to JSON.
     *
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "updateDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" || $attribute == "nameStripped" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * @param $asset
     * @return L10n
     * @throws Exception
     */
    public static function createInstance($asset)
    {
        if (isset($asset[0]))
            $asset = $asset[0];

        $instance = new self();
        $instance->setId($asset['id_l10n']);
        $instance->setIdLeaderboard($asset['id_leaderboard']);
        $instance->setIsoLanguage($asset['iso_language']);
        $instance->setSingularUnit($asset['singular_unit']);
        $instance->setPluralUnits($asset['plural_units']);

        if (isset($asset['update_date']) && Helper::isValidDate($asset['update_date'], "Y-m-d H:i:s")) {
            $instance->setUpdateDate(new DateTime($asset['update_date']));
        }

        if (isset($asset['register_date']) && Helper::isValidDate($asset['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($asset['register_date']));
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }

}