<?php namespace Skoiy\Ott\Leaderboard;

use PDO;
use Skoiy\Ott\MyArrayObject;
use Stage\Paginator;
use Stage\Stage;
use DateTime;
use Stage\Helper;
use Exception;

class Leaderboard
{
    const ID = 1;

    private $id = -1;
    private $name;
    private $nameStripped = "";
    private $token;
    private $allowTeams;
    private $allowFriends;
    private $allowOptimizeResults;
    private $maxUsersPerTeam;
    private $totalTeamsPerUser;
    private $maxFriendsPerUser;
    private $maxOptimizationAttempts;
    private $isActive;

    /**
     * @var
     */
    private $updateDate;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'name', 'nameStripped', 'token', 'allowTeams', 'allowFriends', 'allowOptimizeResults', 'maxUsersPerTeam', 'totalTeamsPerUser', 'maxFriendsPerUser', 'maxOptimizationAttempts', 'isActive', 'updateDate', 'registerDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getNameStripped()
    {
        return $this->nameStripped;
    }

    public function getToken()
    {
        return $this->token;
    }

    public function getAllowTeams()
    {
        return $this->allowTeams;
    }

    public function getAllowFriends()
    {
        return $this->allowFriends;
    }

    public function getAllowOptimizeResults()
    {
        return $this->allowOptimizeResults;
    }

    public function getMaxUsersPerTeam()
    {
        return $this->maxUsersPerTeam;
    }

    public function getTotalTeamsPerUser()
    {
        return $this->totalTeamsPerUser;
    }

    public function getMaxFriendsPerUser()
    {
        return $this->maxFriendsPerUser;
    }

    public function getMaxOptimizationAttempts()
    {
        return $this->maxOptimizationAttempts;
    }

    public function isActive()
    {
        return (bool)$this->isActive;
    }

    /**
     * @return mixed
     */
    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    /**
     * @return mixed
     */
    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function setId($id_plan)
    {
        $this->id = (int)$id_plan;
    }

    /**
     * @param mixed $active
     */
    public function setActive($active)
    {
        $this->isActive = (bool)$active;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setNameStripped($name)
    {
        $this->nameStripped = $name;
    }

    public function setToken($token)
    {
        $this->token = $token;
    }

    public function setAllowTeams($is)
    {
        $this->allowTeams = (bool)$is;
    }

    public function setAllowFriends($pos)
    {
        $this->allowFriends = (bool)$pos;
    }

    public function setAllowOptimizeResults($allowOptimizeResults)
    {
        $this->allowOptimizeResults = (bool)$allowOptimizeResults;
    }

    public function setMaxUsersPerTeam($maxUsersPerTeam)
    {
        $this->maxUsersPerTeam = (int)$maxUsersPerTeam;
    }

    public function setTotalTeamsPerUser($totalTeamsPerUser)
    {
        $this->totalTeamsPerUser = (int)$totalTeamsPerUser;
    }

    public function setMaxFriendsPerUser($grace_period)
    {
        $this->maxFriendsPerUser = (int)$grace_period;
    }

    public function setMaxOptimizationAttempts($maxOptimizationAttempts)
    {
        $this->maxOptimizationAttempts = (int)$maxOptimizationAttempts;
    }

    /**
     * @param mixed $updateDate
     */
    private function setUpdateDate($updateDate)
    {
        $this->updateDate = $updateDate;
    }

    /**
     * @param mixed $registerDate
     */
    private function setRegisterDate($registerDate)
    {
        $this->registerDate = $registerDate;
    }

    /**
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        try {
            $name = "OTT_LEADERBOARDS";
            $method = "SET";
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                [$this->getName(), PDO::PARAM_STR],
                [$this->getNameStripped(), PDO::PARAM_STR],
                [$this->getAllowTeams(), PDO::PARAM_INT],
                [$this->getAllowFriends(), PDO::PARAM_INT],
                [$this->getAllowOptimizeResults(), PDO::PARAM_INT],
                [$this->getMaxUsersPerTeam(), PDO::PARAM_INT],
                [$this->getTotalTeamsPerUser(), PDO::PARAM_INT],
                [$this->getMaxFriendsPerUser(), PDO::PARAM_INT],
                [$this->getMaxOptimizationAttempts(), PDO::PARAM_INT],
                [$this->isActive(), PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            if (!isset($data[0]) && !isset($data[0]['id_leaderboard'])) {
                throw new Exception("Error saving a new Plan");
            }

            $this->setId($data[0]['id_leaderboard']);
            if (isset($data[0]['update_date']) && Helper::isValidDate($data[0]['update_date'], "Y-m-d H:i:s")) {
                $this->setRegisterDate(new DateTime($data[0]['update_date']));
            }
            if (isset($data[0]['register_date']) && Helper::isValidDate($data[0]['register_date'], "Y-m-d H:i:s")) {
                $this->setRegisterDate(new DateTime($data[0]['register_date']));
            }

            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @return array
     * @throws Exception
     */
    public static function sizeof()
    {
        $name = "OTT_LEADERBOARDS_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * @param $id
     * @return Leaderboard
     * @throws Exception
     */
    public static function find($id)
    {
        try {
            $name = "OTT_LEADERBOARD";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Leaderboard id $id not exists.", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * isoLanguages
     *
     * @param $idLeaderboard
     * @return array
     * @throws Exception
     */
    public static function isoLanguages($idLeaderboard)
    {
        try {
            $name = "OTT_LEADERBOARDS_ISOLANGUAGES";
            $method = "GET";
            $parameters = [[$idLeaderboard, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Leaderboard id $idLeaderboard not exists.", $data[0]['return_code']);
            } else {
                return $data;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * query
     *
     * @param Query $q
     * @param bool $count
     * @param int $page
     * @param int $perPage
     * @return MyArrayObject
     * @throws Exception
     */
    public static function query(Query $q, $count = true, $page = 1, $perPage = 15)
    {
        $method = "GET";

        $field = $q->by;
        $fieldSnd = $q->sndBy;
        $fieldTrd = $q->trdBy;
        $query = $q->term;
        $filter = $q->filter;
        $filterSnd = $q->sndFilter;
        $filterTrd = $q->trdFilter;
        $limitA = $q->from;
        $limitB = $q->to;
        $orderBy = $q->orderBy;
        $order = $q->order;

        try {
            if ($count == true) {
                $parameters = [
                    [$field, PDO::PARAM_INT],
                    [$fieldSnd, PDO::PARAM_INT],
                    [$fieldTrd, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_STR],
                    [$filterSnd, PDO::PARAM_STR],
                    [$filterTrd, PDO::PARAM_STR],
                ];

                $name = "OTT_LEADERBOARDS_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limitA = isset($p) ? $p->from() : $q->from;
            $limitB = isset($p) ? $p->to() : $q->to;

            $parameters = [[$field, PDO::PARAM_INT],
                [$fieldSnd, PDO::PARAM_INT],
                [$fieldTrd, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$filterSnd, PDO::PARAM_STR],
                [$filterTrd, PDO::PARAM_STR],
                [$orderBy, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT],
                [$limitA, PDO::PARAM_INT],
                [$limitB, PDO::PARAM_INT],
            ];

            $name = "OTT_LEADERBOARDS";
            $data = Stage::esp($parameters, $name, $method);

            $result = new MyArrayObject(array_map(function ($label) {
                return self::createInstance($label);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachL10n
     *
     * @param $idLeaderboard
     * @param $isoLanguage
     * @param $singularUnit
     * @param $pluralUnits
     * @param int $idL10n
     * @return L10n|null
     * @throws Exception
     */
    public static function attachL10n($idLeaderboard, $isoLanguage, $singularUnit, $pluralUnits, $idL10n = 0)
    {
        $method = "SET";

        try {

            $parameters = [[$idL10n, PDO::PARAM_INT],
                [$idLeaderboard, PDO::PARAM_INT],
                [$isoLanguage, PDO::PARAM_STR],
                [$singularUnit, PDO::PARAM_STR],
                [$pluralUnits, PDO::PARAM_STR]
            ];

            $name = "OTT_LEADERBOARD_L10N";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code'])) {
                throw new Exception("The L10n iso language $isoLanguage already exists.", $data[0]['return_code']);
            }

            if (!isset($data[0]) || !isset($data[0]['id_l10n'])) {
                return null;
            }

            return L10n::createInstance($data[0]);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * query
     *
     * @param $idLeaderboard
     * @return array
     * @throws Exception
     */
    public static function l10ns($idLeaderboard)
    {
        try {

            $parameters = [
                [$idLeaderboard, PDO::PARAM_INT],
            ];

            $method = "GET";
            $name = "OTT_LEADERBOARDS_L10N";
            $data = Stage::esp($parameters, $name, $method);
            return array_map(function ($user) {
                return L10n::createInstance($user);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachTeamAvatars
     *
     * @param $idLeaderboard
     * @param $name
     * @param $idAsset
     * @param $filename
     * @param $avatar
     * @param $active
     * @param int $idAvatar
     * @return TeamAvatar|null
     * @throws Exception
     */
    public static function attachTeamAvatars($idLeaderboard, $name, $idAsset, $filename, $avatar, $active, $idAvatar = 0)
    {
        $method = "SET";

        try {
            $parameters = [[$idAvatar, PDO::PARAM_INT],
                [$idLeaderboard, PDO::PARAM_INT],
                [$name, PDO::PARAM_STR],
                [$idAsset, PDO::PARAM_INT],
                [$filename, PDO::PARAM_STR],
                [$avatar, PDO::PARAM_STR],
                [$active, PDO::PARAM_BOOL]
            ];

            $name = "OTT_LEADERBOARDS_TEAMS_AVATARS";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code'])) {
                throw new Exception("The TeamAvatar id $idLeaderboard already exists.", $data[0]['return_code']);
            }

            if (!isset($data[0]) || !isset($data[0]['id_avatar'])) {
                return null;
            }

            return TeamAvatar::createInstance($data[0]);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * queryTeamsAvatars
     *
     * @param Query $q
     * @param bool $count
     * @param int $page
     * @param int $perPage
     * @return MyArrayObject
     * @throws Exception
     */
    public static function queryTeamsAvatars(Query $q, $count = true, $page = 1, $perPage = 15)
    {
        $method = "GET";

        $idLeaderboard = $q->idLeaderboard;
        $field = $q->by;
        $fieldSnd = $q->sndBy;
        $fieldTrd = $q->trdBy;
        $query = $q->term;
        $filter = $q->filter;
        $filterSnd = $q->sndFilter;
        $filterTrd = $q->trdFilter;
        $limitA = $q->from;
        $limitB = $q->to;
        $orderBy = $q->orderBy;
        $order = $q->order;

        try {
            if ($count == true) {
                $parameters = [
                    [$idLeaderboard, PDO::PARAM_INT],
                    [$field, PDO::PARAM_INT],
                    [$fieldSnd, PDO::PARAM_INT],
                    [$fieldTrd, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_STR],
                    [$filterSnd, PDO::PARAM_STR],
                    [$filterTrd, PDO::PARAM_STR],
                    [$orderBy, PDO::PARAM_INT],
                    [$order, PDO::PARAM_INT],
                    [$limitA, PDO::PARAM_INT],
                    [-1, PDO::PARAM_INT],
                ];

                $name = "OTT_LEADERBOARDS_TEAMS_AVATARS";
                $data = Stage::esp($parameters, $name, $method);

                $total = sizeof($data);

                if (!$total) {
                    return new MyArrayObject([]);
                }

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limitA = isset($p) ? $p->from() : $q->from;
            $limitB = isset($p) ? $p->to() : $q->to;

            $parameters = [[$idLeaderboard, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$fieldSnd, PDO::PARAM_INT],
                [$fieldTrd, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$filterSnd, PDO::PARAM_STR],
                [$filterTrd, PDO::PARAM_STR],
                [$orderBy, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT],
                [$limitA, PDO::PARAM_INT],
                [$limitB, PDO::PARAM_INT],
            ];

            $name = "OTT_LEADERBOARDS_TEAMS_AVATARS";
            $data = Stage::esp($parameters, $name, $method);

            $result = new MyArrayObject(array_map(function ($label) {
                return UserAvatar::createInstance($label);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachUserAvatars
     *
     * @param $idLeaderboard
     * @param $name
     * @param $idAsset
     * @param $filename
     * @param $avatar
     * @param $active
     * @param int $idAvatar
     * @return TeamAvatar|null
     * @throws Exception
     */
    public static function attachUserAvatars($idLeaderboard, $name, $idAsset, $filename, $avatar, $active, $idAvatar = 0)
    {
        $method = "SET";

        try {
            $parameters = [[$idAvatar, PDO::PARAM_INT],
                [$idLeaderboard, PDO::PARAM_INT],
                [$name, PDO::PARAM_STR],
                [$idAsset, PDO::PARAM_INT],
                [$filename, PDO::PARAM_STR],
                [$avatar, PDO::PARAM_STR],
                [$active, PDO::PARAM_BOOL]
            ];

            $name = "OTT_LEADERBOARDS_USERS_AVATARS";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code'])) {
                throw new Exception("The TeamAvatar id $idLeaderboard already exists.", $data[0]['return_code']);
            }

            if (!isset($data[0]) || !isset($data[0]['id_avatar'])) {
                return null;
            }

            return UserAvatar::createInstance($data[0]);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * queryUsersAvatars
     *
     * @param Query $q
     * @param bool $count
     * @param int $page
     * @param int $perPage
     * @return MyArrayObject
     * @throws Exception
     */
    public static function queryUsersAvatars(Query $q, $count = true, $page = 1, $perPage = 15)
    {
        $method = "GET";

        $idLeaderboard = $q->idLeaderboard;
        $field = $q->by;
        $fieldSnd = $q->sndBy;
        $fieldTrd = $q->trdBy;
        $query = $q->term;
        $filter = $q->filter;
        $filterSnd = $q->sndFilter;
        $filterTrd = $q->trdFilter;
        $limitA = $q->from;
        $limitB = $q->to;
        $orderBy = $q->orderBy;
        $order = $q->order;

        try {
            if ($count == true) {
                $parameters = [
                    [$idLeaderboard, PDO::PARAM_INT],
                    [$field, PDO::PARAM_INT],
                    [$fieldSnd, PDO::PARAM_INT],
                    [$fieldTrd, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_STR],
                    [$filterSnd, PDO::PARAM_STR],
                    [$filterTrd, PDO::PARAM_STR],
                ];

                $name = "OTT_LEADERBOARDS_USERS_AVATARS_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                if (!$total) {
                    return new MyArrayObject([]);
                }

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limitA = isset($p) ? $p->from() : $q->from;
            $limitB = isset($p) ? $p->to() : $q->to;

            $parameters = [[$idLeaderboard, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$fieldSnd, PDO::PARAM_INT],
                [$fieldTrd, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$filterSnd, PDO::PARAM_STR],
                [$filterTrd, PDO::PARAM_STR],
                [$orderBy, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT],
                [$limitA, PDO::PARAM_INT],
                [$limitB, PDO::PARAM_INT],
            ];

            $name = "OTT_LEADERBOARDS_USERS_AVATARS";
            $data = Stage::esp($parameters, $name, $method);

            $result = new MyArrayObject(array_map(function ($label) {
                return UserAvatar::createInstance($label);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * queryUsers
     *
     * @param Query $q
     * @param bool $count
     * @param int $page
     * @param int $perPage
     * @return MyArrayObject
     * @throws Exception
     */
    public static function queryUsers(Query $q, $count = true, $page = 1, $perPage = 15)
    {
        $method = "GET";

        $idLeaderboard = $q->idLeaderboard;
        $field = $q->by;
        $fieldSnd = $q->sndBy;
        $fieldTrd = $q->trdBy;
        $query = $q->term;
        $filter = $q->filter;
        $filterSnd = $q->sndFilter;
        $filterTrd = $q->trdFilter;
        $limitA = $q->from;
        $limitB = $q->to;
        $orderBy = $q->orderBy;
        $order = $q->order;

        try {
            if ($count == true) {
                $parameters = [
                    [$idLeaderboard, PDO::PARAM_INT],
                    [$field, PDO::PARAM_INT],
                    [$fieldSnd, PDO::PARAM_INT],
                    [$fieldTrd, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_STR],
                    [$filterSnd, PDO::PARAM_STR],
                    [$filterTrd, PDO::PARAM_STR],
                ];

                $name = "OTT_LEADERBOARDS_USERS_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                if (!$total) {
                    return new MyArrayObject([]);
                }

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limitA = isset($p) ? $p->from() : $q->from;
            $limitB = isset($p) ? $p->to() : $q->to;

            $parameters = [[$idLeaderboard, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$fieldSnd, PDO::PARAM_INT],
                [$fieldTrd, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$filterSnd, PDO::PARAM_STR],
                [$filterTrd, PDO::PARAM_STR],
                [$orderBy, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT],
                [$limitA, PDO::PARAM_INT],
                [$limitB, PDO::PARAM_INT],
            ];

            $name = "OTT_LEADERBOARDS_USERS";
            $data = Stage::esp($parameters, $name, $method);

            $result = new MyArrayObject(array_map(function ($label) {
                return User::createInstance($label);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "updateDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" || $attribute == "nameStripped" || $attribute == "imageTitle" || $attribute == "imageAlterText" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * @param $asset
     * @return Leaderboard
     * @throws Exception
     */
    public static function createInstance($asset)
    {
        if (isset($asset[0]))
            $asset = $asset[0];

        $instance = new self();
        $instance->setId($asset['id_leaderboard']);
        $instance->setName(html_entity_decode($asset['name'], ENT_COMPAT, 'UTF-8'));
        $instance->setNameStripped($asset['name_stripped']);
        if (isset($asset['leaderboard_token'])) {
            $instance->setToken($asset['leaderboard_token']);
        }
        $instance->setAllowTeams($asset['allow_teams']);
        $instance->setAllowFriends($asset['allow_friends']);
        $instance->setAllowOptimizeResults($asset['allow_optimize_results']);
        $instance->setMaxUsersPerTeam($asset['max_users_per_team']);
        $instance->setTotalTeamsPerUser($asset['total_teams_per_user']);
        $instance->setMaxFriendsPerUser($asset['max_friends_per_user']);
        $instance->setMaxOptimizationAttempts($asset['max_optimization_attempts']);
        $instance->setActive($asset['active']);

        if (isset($asset['update_date']) && Helper::isValidDate($asset['update_date'], "Y-m-d H:i:s")) {
            $instance->setUpdateDate(new DateTime($asset['update_date']));
        }

        if (isset($asset['register_date']) && Helper::isValidDate($asset['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($asset['register_date']));
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }

}