<?php namespace Skoiy\Ott\Plan;

use PDO;
use Stage\Stage;
use DateTime;
use Stage\Helper;
use Exception;

class Plan
{
    const ID = 1;

    private $id = -1;
    private $planType;
    private $managementName = "";
    private $nameStripped = "";
    private $billingType;
    private $billingCycle;
    private $firstBillDate;
    private $trialActive = true;
    private $trialType;
    private $trialPeriod;
    private $trialForNewUsers = true;
    private $timeActive = true;
    private $timeType;
    private $timePeriod;
    private $graceActive = true;
    private $graceType;
    private $gracePeriod;
    private $unlimitedAccess = true;
    private $acceptPayCycle = true;
    private $acceptPayMore = true;
    private $isActive = true;
    private $allowPromotions = true;
    private $totalUsers = 0;
    private $isDefault = false;
    private $position;

    /**
     * @var
     */
    private $updateDate;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'planType', 'managementName', 'nameStripped', 'billingType', 'billingCycle', 'firstBillDate', 'trialActive', 'trialType', 'trialPeriod', 'trialForNewUsers', 'timeActive', 'timeType', 'timePeriod', 'graceActive', 'graceType', 'allowPromotions', 'totalUsers', 'isActive', 'unlimitedAccess', 'unlimitedAccess', 'acceptPayCycle', 'gracePeriod', 'acceptPayMore', 'isDefault', 'position', 'updateDate', 'registerDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getPlanType()
    {
        return $this->planType;
    }

    public function getManagementName()
    {
        return $this->managementName;
    }

    public function getNameStripped()
    {
        return $this->nameStripped;
    }

    public function getBillingType()
    {
        return $this->billingType;
    }

    public function getBillingCycle()
    {
        return $this->billingCycle;
    }

    public function getFirstBillDate()
    {
        return $this->firstBillDate;
    }

    public function getTrialActive()
    {
        return $this->trialActive;
    }

    public function getTrialType()
    {
        return $this->trialType;
    }

    public function getTrialPeriod()
    {
        return $this->trialPeriod;
    }

    public function getTrialForNewUsers()
    {
        return $this->trialForNewUsers;
    }

    public function getTimeActive()
    {
        return $this->timeActive;
    }

    public function getTimeType()
    {
        return $this->timeType;
    }

    public function getTimePeriod()
    {
        return $this->timePeriod;
    }

    public function getGraceActive()
    {
        return $this->graceActive;
    }

    public function getGraceType()
    {
        return $this->graceType;
    }

    public function getAcceptPayMore()
    {
        return $this->acceptPayMore;
    }

    public function getGracePeriod()
    {
        return $this->gracePeriod;
    }

    public function getUnlimitedAccess()
    {
        return $this->unlimitedAccess;
    }

    public function getAcceptPayCycle()
    {
        return $this->acceptPayCycle;
    }

    public function isActive()
    {
        return (bool)$this->isActive;
    }

    public function allowPromotions()
    {
        return (bool)$this->allowPromotions;
    }

    public function getTotalUsers()
    {
        return $this->totalUsers;
    }

    public function isDefault()
    {
        return (bool)$this->isDefault;
    }

    public function getPosition()
    {
        return $this->position;
    }

    /**
     * @return mixed
     */
    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    /**
     * @return mixed
     */
    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    /**
     * @param mixed $active
     */
    public function setActive($active)
    {
        $this->isActive = (bool)$active;
    }

    public function setAllowPromotions($allow_promotions)
    {
        $this->allowPromotions = (bool)$allow_promotions;
    }

    public function setTotalUsers($total)
    {
        $this->totalUsers = (int) $total;
    }

    public function setIsDefault($is)
    {
        $this->isDefault = (bool)$is;
    }

    public function setPosition($pos)
    {
        $this->position = $pos;
    }

    public function setAcceptPayCycle($accept_pay_cycle)
    {
        $this->acceptPayCycle = (bool)$accept_pay_cycle;
    }

    public function setAcceptPayMore($accept_pay_more)
    {
        $this->acceptPayMore = (bool)$accept_pay_more;
    }

    public function setUnlimitedAccess($unlimited_access)
    {
        $this->unlimitedAccess = (bool)$unlimited_access;
    }

    public function setGracePeriod($grace_period)
    {
        $this->gracePeriod = (int)$grace_period;
    }

    public function setGraceType($grace_type)
    {
        $this->graceType = (int)$grace_type;
    }

    public function setGraceActive($grace_active)
    {
        $this->graceActive = (bool)$grace_active;
    }

    public function setTimePeriod($time_period)
    {
        $this->timePeriod = (int)$time_period;
    }

    public function setTimeType($time_type)
    {
        $this->timeType = (int)$time_type;
    }

    public function setTimeActive($time_active)
    {
        $this->timeActive = (bool)$time_active;
    }

    public function setTrialForNewUsers($trial_for_new_users)
    {
        $this->trialForNewUsers = (bool)$trial_for_new_users;
    }

    public function setTrialPeriod($trial_period)
    {
        $this->trialPeriod = (int)$trial_period;
    }

    public function setTrialType($trial_type)
    {
        $this->trialType = (int)$trial_type;
    }

    public function setTrialActive($trial_active)
    {
        $this->trialActive = (bool)$trial_active;
    }

    public function setFirstBillDate($first_bill_date)
    {
        $this->firstBillDate = $first_bill_date;
    }

    public function setBillingCycle($billing_cycle)
    {
        $this->billingCycle = (int)$billing_cycle;
    }

    public function setBillingType($billing_type)
    {
        $this->billingType = (int)$billing_type;
    }

    public function setNameStripped($name_stripped)
    {
        $this->nameStripped = $name_stripped;
    }

    public function setManagementName($management_name)
    {
        $this->managementName = $management_name;
    }

    public function setPlanType($plan_type)
    {
        $this->planType = (int)$plan_type;
    }

    public function setId($id_plan)
    {
        $this->id = (int)$id_plan;
    }

    /**
     * @param mixed $updateDate
     */
    private function setUpdateDate($updateDate)
    {
        $this->updateDate = $updateDate;
    }

    /**
     * @param mixed $registerDate
     */
    private function setRegisterDate($registerDate)
    {
        $this->registerDate = $registerDate;
    }

    /**
     * @return $this
     * @throws Exception
     */
    public function save($idPlan = null)
    {
        try {
            $name = "OTT_PLAN";
            $method = "SET";
            $parameters = [[$idPlan ? $idPlan : $this->getId(), PDO::PARAM_INT],
                [$this->getPlanType(), PDO::PARAM_INT],
                [$this->getManagementName(), PDO::PARAM_STR],
                [$this->getNameStripped(), PDO::PARAM_STR],
                [$this->getBillingType(), PDO::PARAM_INT],
                [$this->getBillingCycle(), PDO::PARAM_INT],
                [$this->getFirstBillDate(), PDO::PARAM_STR],
                [$this->getTrialActive(), PDO::PARAM_BOOL],
                [$this->getTrialType(), PDO::PARAM_BOOL],
                [$this->getTrialPeriod(), PDO::PARAM_INT],
                [$this->getTrialForNewUsers(), PDO::PARAM_INT],
                [$this->getTimeActive(), PDO::PARAM_INT],
                [$this->getTimeType(), PDO::PARAM_INT],
                [$this->getTimePeriod(), PDO::PARAM_INT],
                [$this->getGraceActive(), PDO::PARAM_INT],
                [$this->getGraceType(), PDO::PARAM_INT],
                [$this->getGracePeriod(), PDO::PARAM_INT],
                [$this->getUnlimitedAccess(), PDO::PARAM_INT],
                [$this->getAcceptPayMore(), PDO::PARAM_INT],
                [$this->getAcceptPayCycle(), PDO::PARAM_INT],
                [$this->isActive(), PDO::PARAM_INT],
                [$this->AllowPromotions(), PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            if (!isset($data[0]) && !isset($data[0]['id_plan'])) {
                throw new Exception("Error saving a new Plan");
            }

            $this->setId($data[0]['id_plan']);
            if (isset($data[0]['update_date']) && Helper::isValidDate($data[0]['update_date'], "Y-m-d H:i:s")) {
                $this->setRegisterDate(new DateTime($data[0]['update_date']));
            }
            if (isset($data[0]['register_date']) && Helper::isValidDate($data[0]['register_date'], "Y-m-d H:i:s")) {
                $this->setRegisterDate(new DateTime($data[0]['register_date']));
            }

            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @return Plan
     * @throws Exception
     */
    public static function duplicate($id)
    {
        try {
            $parameters = [[$id, PDO::PARAM_INT]];
            $name = "OTT_PLAN_CLONE";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            return self::createInstance($data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idPlan
     * @return bool
     * @throws Exception
     */
    public static function delete($idPlan)
    {
        try {
            $parameters = [[$idPlan, PDO::PARAM_INT]];
            $name = "OTT_PLAN";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("Plan {$idPlan} not found");
            } else if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0") {
                return true;
            }
            return false;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @return array
     * @throws Exception
     */
    public static function sizeof()
    {
        $name = "OTT_PLANS_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * @param $idPlan
     * @return Plan
     * @throws Exception
     */
    public static function find($idPlan)
    {
        try {
            $name = "OTT_PLAN";
            $method = "GET";
            $parameters = [[$idPlan, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Plan id $idPlan not exists.", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idPlan
     * @return array
     * @throws Exception
     */
    public static function isoLanguages($idPlan)
    {
        try {
            $name = "OTT_PLAN_ISOLANGUAGES";
            $method = "GET";
            $parameters = [[$idPlan, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Plan id $idPlan not exists.", $data[0]['return_code']);
            } else {
                return $data;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Get all L10n's for this Plan
     *
     * @param $idPlan
     * @return L10n[]
     * @throws Exception
     */
    public static function l10n($idPlan)
    {
        try {
            return L10n::find($idPlan);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idPlan
     * @return array
     * @throws Exception
     */
    public static function prices($idPlan)
    {
        try {
            return Price::find($idPlan);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param Query $q
     * @return array|User[]
     * @throws Exception
     */
    public static function query(Query $q)
    {
        // TODO
        $field = $q->by;
        $fieldSnd = $q->sndBy;
        $fieldTrd = $q->trdBy;
        $query = $q->term;
        $filter = $q->filter;
        $filterSnd = $q->sndFilter;
        $filterTrd = $q->trdFilter;
        $limitA = $q->from;
        $limitB = $q->to;
        $orderBy = $q->orderBy;
        $order = $q->order;

        try {
            $parameters = [[$field, PDO::PARAM_INT],
                [$fieldSnd, PDO::PARAM_INT],
                [$fieldTrd, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$filterSnd, PDO::PARAM_STR],
                [$filterTrd, PDO::PARAM_STR],
                [$orderBy, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT],
                [$limitA, PDO::PARAM_INT],
                [$limitB, PDO::PARAM_INT],
            ];

            $name = "OTT_PLANS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return array_map(function ($user) {
                return self::createInstance($user);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param Query $q
     * @return mixed
     * @throws Exception
     */
    public static function queryCount(Query $q)
    {
        // TODO
        $field = $q->by;
        $fieldSnd = $q->sndBy;
        $fieldTrd = $q->trdBy;
        $query = $q->term;
        $filter = $q->filter;
        $filterSnd = $q->sndFilter;
        $filterTrd = $q->trdFilter;
        $limitA = $q->from;
        $limitB = $q->to;
        $orderBy = $q->orderBy;
        $order = $q->order;

        try {
            $parameters = [[$field, PDO::PARAM_INT],
                [$fieldSnd, PDO::PARAM_INT],
                [$fieldTrd, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$filterSnd, PDO::PARAM_STR],
                [$filterTrd, PDO::PARAM_STR]];

            $name = "OTT_PLANS_COUNT";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]['total']) ? $data[0]['total'] : 0;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "updateDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" || $attribute == "nameStripped" || $attribute == "imageTitle" || $attribute == "imageAlterText" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * @param $asset
     * @return Plan
     * @throws Exception
     */
    public static function createInstance($asset)
    {
        if (isset($asset[0]))
            $asset = $asset[0];

        $instance = new self();
        $instance->setId($asset['id_plan']);
        $instance->setPlanType($asset['plan_type']);
        $instance->setManagementName(html_entity_decode($asset['management_name'], ENT_COMPAT, 'UTF-8'));
        $instance->setNameStripped($asset['name_stripped']);
        $instance->setBillingType($asset['billing_type']);
        $instance->setBillingCycle($asset['billing_cycle']);
        if (isset($asset['first_bill_date']) && Helper::isValidDate($asset['first_bill_date'], "Y-m-d H:i:s")) {
            $instance->setFirstBillDate($asset['first_bill_date']);
        }
        $instance->setTrialActive($asset['trial_active']);
        $instance->setTrialType($asset['trial_type']);
        $instance->setTrialPeriod($asset['trial_period']);
        $instance->setTrialForNewUsers($asset['trial_for_new_users']);
        $instance->setTimeActive($asset['time_active']);
        $instance->setTimeType($asset['time_type']);
        $instance->setTimePeriod($asset['time_period']);
        $instance->setGraceActive($asset['grace_active']);
        $instance->setGraceType($asset['grace_type']);
        $instance->setGracePeriod($asset['grace_period']);
        $instance->setUnlimitedAccess($asset['unlimited_access']);
        $instance->setAcceptPayMore($asset['accept_pay_more']);
        $instance->setAcceptPayCycle($asset['accept_pay_cycle']);
        $instance->setActive($asset['active']);
        $instance->setAllowPromotions($asset['allow_promotions']);
        if( isset($asset['total_users']) ) {
            $instance->setTotalUsers($asset['total_users']);
        }

        if (isset($asset['is_default']))
            $instance->setIsDefault($asset['is_default']);

        if (isset($asset['position']))
            $instance->setPosition($asset['position']);

//        $instance->setName(html_entity_decode($asset['name'], ENT_COMPAT, 'UTF-8'));

        if (isset($asset['update_date']) && Helper::isValidDate($asset['update_date'], "Y-m-d H:i:s")) {
            $instance->setUpdateDate(new DateTime($asset['update_date']));
        }

        if (isset($asset['register_date']) && Helper::isValidDate($asset['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($asset['register_date']));
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }

}