<?php namespace Skoiy\Ott\Plan;

use PDO;
use Stage\Stage;
use DateTime;
use Stage\Helper;
use Exception;

class Price
{
    const DELETE_ONE = 1;
    const DELETE_ALL = 2;

    const ID = 1;

    private $id = -1;
    private $idPlan;
    private $price;
    private $trialPrice;
    private $currencyIso = "";
    private $isDefault;
    private $position;

    /**
     * @var
     */
    private $updateDate;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'price', 'trialPrice', 'currencyIso', 'isDefault', 'position', 'updateDate', 'registerDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdPlan()
    {
        return $this->idPlan;
    }

    public function getPrice()
    {
        return $this->price;
    }

    public function getTrialPrice()
    {
        return $this->trialPrice;
    }

    public function getCurrencyIso()
    {
        return $this->currencyIso;
    }

    public function isDefault()
    {
        return (bool)$this->isDefault;
    }

    public function getPosition()
    {
        return $this->position;
    }

    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function setIdPlan($id)
    {
        $this->idPlan = (int)$id;
    }

    public function setPrice($price)
    {
        $this->price = $price;
    }

    public function setTrialPrice($price)
    {
        $this->trialPrice = $price;
    }

    public function setCurrencyIso($iso)
    {
        $this->currencyIso = $iso;
    }

    public function setPosition($pos)
    {
        $this->position = (int)$pos;
    }

    public function setIsDefault($is)
    {
        $this->isDefault = (bool)$is;
    }

    private function setId($id_plan)
    {
        $this->id = (int)$id_plan;
    }

    /**
     * @param mixed $updateDate
     */
    private function setUpdateDate($updateDate)
    {
        $this->updateDate = $updateDate;
    }

    /**
     * @param mixed $registerDate
     */
    private function setRegisterDate($registerDate)
    {
        $this->registerDate = $registerDate;
    }

    /**
     * @param $idPlan
     * @param null $idPrice
     * @return array|Price|Price[]
     * @throws Exception
     */
    public static function find($idPlan, $idPrice = null)
    {
        try {
            $name = "OTT_PLAN_PRICES";
            $method = "GET";
            $parameters = [[$idPlan, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Plan id $idPlan not exists.", $data[0]['return_code']);
            } else {
                $prices = array_map(function ($l10n) {
                    return self::createInstance($l10n);
                }, $data);

                if ($idPrice) {
                    $price = array_values(array_filter($prices, function ($price) use ($idPrice) {
                        return $price->getId() == $idPrice;
                    }));
                    if (isset($price[0]) && $price[0] instanceof Price) {
                        return $price[0];
                    } else {
                        throw new Exception("Price $idPrice not found.");
                    }
                } else {
                    return $prices;
                }
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @return $this
     * @throws Exception
     * @description Save on table plans_price_related
     */
    public function save()
    {
        $idPlan = $this->getIdPlan();
        try {
            $name = "OTT_PLAN_PRICES";
            $method = "SET";
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                [$idPlan, PDO::PARAM_INT],
                [$this->getPrice(), PDO::PARAM_INT],
                [$this->getTrialPrice(), PDO::PARAM_INT],
                [$this->getCurrencyIso(), PDO::PARAM_STR],
                [$this->isDefault(), PDO::PARAM_BOOL],
                [$this->getPosition(), PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            if (!isset($data[0]) && !isset($data[0]['id_plan'])) {
                throw new Exception("Error saving a new Plan Price");
            } else if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "-3") {
                throw new Exception("Plan Price with currency iso {$this->getCurrencyIso()} already exists");
            }

            $this->setId($data[0]['id']);
            $this->setIdPlan($data[0]['id_plan']);
            if (isset($data[0]['update_date']) && Helper::isValidDate($data[0]['update_date'], "Y-m-d H:i:s")) {
                $this->setUpdateDate(new DateTime($data[0]['update_date']));
            }
            if (isset($data[0]['register_date']) && Helper::isValidDate($data[0]['register_date'], "Y-m-d H:i:s")) {
                $this->setRegisterDate(new DateTime($data[0]['register_date']));
            }

            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idPlan
     * @param int $type
     * @param int $id
     * @return bool
     * @throws Exception
     */
    public static function delete($idPlan, $id = 0, $type = self::DELETE_ONE)
    {
        try {
            $parameters = [[$type, PDO::PARAM_INT],
                [$idPlan, PDO::PARAM_INT],
                [$id, PDO::PARAM_INT]];
            $name = "OTT_PLAN_PRICES";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("Plan {$idPlan} not found");
            } else if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0") {
                return true;
            }
            return false;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Delete all Prices from a Plan
     *
     * @param $idPlan
     * @return bool
     * @throws Exception
     */
    public static function deleteAll($idPlan)
    {
        try {
            return self::delete($idPlan, 0, self::DELETE_ALL);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "updateDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" || $attribute == "nameStripped" || $attribute == "imageTitle" || $attribute == "imageAlterText" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * @param $asset
     * @return Price
     * @throws Exception
     */
    public static function createInstance($asset)
    {
        $instance = new self();
        $instance->setId($asset['id']);
        $instance->setIdPlan($asset['id_plan']);
        $instance->setCurrencyIso($asset['currency_iso']);
        $instance->setPrice($asset['price']);
        $instance->setTrialPrice($asset['trial_price']);
        $instance->setIsDefault($asset['is_default']);

        if (isset($asset['update_date']) && Helper::isValidDate($asset['update_date'], "Y-m-d H:i:s")) {
            $instance->setUpdateDate(new DateTime($asset['update_date']));
        }

        if (isset($asset['register_date']) && Helper::isValidDate($asset['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($asset['register_date']));
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }

}