<?php namespace Skoiy\Ott\Plan\Set;

use PDO;
use Stage\Stage;
use DateTime;
use Stage\Helper;
use Exception;

class L10n
{
    const ID = 1;

    private $id = -1;
    private $idPlanSet;
    private $isoLanguage;
    private $name = "";
    private $description = "";
    private $details = "";
    private $actionName = "";
    private $actionTarget = "";

    /**
     * @var
     */
    private $updateDate;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'idPlanSet', 'isoLanguage', 'name', 'description', 'details', 'actionName', 'actionTarget', 'updateDate', 'registerDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdPlanSet()
    {
        return $this->idPlanSet;
    }

    public function getIsoLanguage()
    {
        return $this->isoLanguage;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getDescription()
    {
        return $this->description;
    }

    public function getDetails()
    {
        return $this->details;
    }

    public function getActionName()
    {
        return $this->actionName;
    }

    public function getActionTarget()
    {
        return $this->actionTarget;
    }

    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setIdPlanSet($id)
    {
        $this->idPlanSet = (int)$id;
    }

    public function setIsoLanguage($iso)
    {
        $this->isoLanguage = $iso;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setDescription($desc)
    {
        $this->description = $desc;
    }

    public function setDetails($details)
    {
        $this->details = $details;
    }

    public function setActionName($action)
    {
        $this->actionName = $action;
    }

    public function setActionTarget($target)
    {
        $this->actionTarget = $target;
    }

    private function setUpdateDate($updateDate)
    {
        $this->updateDate = $updateDate;
    }

    private function setRegisterDate($registerDate)
    {
        $this->registerDate = $registerDate;
    }

    /**
     * @param $idPlanSet
     * @param false $idL10n
     * @return array|L10n|L10n[]
     * @throws Exception
     */
    public static function find($idPlanSet, $idL10n = false) {
        try {
            $name = "OTT_PLAN_SET_L10N";
            $method = "GET";
            $parameters = [[$idPlanSet, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Plan id $idPlanSet not exists.", $data[0]['return_code']);
            } else {
                $l10ns = array_map(function ($l10n) {
                    return self::createInstance($l10n);
                }, $data);

                if( $idL10n ) {
                    $l10n = array_values(array_filter($l10ns, function($l10n) use ($idL10n) {
                        return $l10n->getId() == $idL10n;
                    }));
                    if( isset($l10n[0]) && $l10n[0] instanceof L10n ) {
                        return $l10n[0];
                    } else {
                        throw new Exception("L10n $idL10n not found.");
                    }
                } else {
                    return $l10ns;
                }
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        try {
            $name = "OTT_PLAN_SET_L10N";
            $method = "SET";
            $parameters = [[$this->getId(), PDO::PARAM_INT],//id_l10n
                [$this->getIdPlanSet(), PDO::PARAM_INT],
                [$this->getIsoLanguage(), PDO::PARAM_STR],
                [$this->getName(), PDO::PARAM_STR],
                [$this->getDescription(), PDO::PARAM_STR],
                [$this->getDetails(), PDO::PARAM_STR],
                [$this->getActionName(), PDO::PARAM_STR],
                [$this->getActionTarget(), PDO::PARAM_STR]];

            // NOTE: Se a isoLanguage ja existir para o idPlanSet vai dar -2
            $data = Stage::esp($parameters, $name, $method);
            if (!isset($data[0]) && !isset($data[0]['id_l10n'])) {
                throw new Exception("Error saving a new Plan Set L10n");
            } else if(isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("Iso language repeated or id {$this->getIdPlanSet()} plan set not found");
            }

            $this->setId($data[0]['id_l10n']);
            if (isset($data[0]['update_date']) && Helper::isValidDate($data[0]['update_date'], "Y-m-d H:i:s")) {
                $this->setRegisterDate(new DateTime($data[0]['update_date']));
            }
            if (isset($data[0]['register_date']) && Helper::isValidDate($data[0]['register_date'], "Y-m-d H:i:s")) {
                $this->setRegisterDate(new DateTime($data[0]['register_date']));
            }

            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idPlanSetL10n
     * @return bool
     * @throws Exception
     * @description Writes on table plans_sets_l10n
     */
    public static function delete($idPlanSetL10n)
    {
        try {
            $parameters = [[$idPlanSetL10n, PDO::PARAM_INT]];
            $name = "OTT_PLAN_SET_L10N";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("PlanSet L10n {$idPlanSetL10n} not found");
            } else if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0") {
                return true;
            }
            return false;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "updateDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" || $attribute == "nameStripped" || $attribute == "description" || $attribute == "details" || $attribute == "actionName" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * @param $asset
     * @return SetL10n
     * @throws Exception
     */
    public static function createInstance($asset)
    {
        if (isset($asset[0]) && isset($asset[0]['id_l10n']))
            $asset = $asset[0];

        $instance = new self();
        $instance->setId($asset['id_l10n']);
        $instance->setIdPlanSet($asset['id_plan_set']);
        $instance->setIsoLanguage($asset['iso_language']);
        $instance->setName(html_entity_decode($asset['name'], ENT_COMPAT, 'UTF-8'));
        $instance->setDescription(html_entity_decode($asset['description'], ENT_COMPAT, 'UTF-8'));
        $instance->setDetails(html_entity_decode($asset['details'], ENT_COMPAT, 'UTF-8'));
        $instance->setActionName(html_entity_decode($asset['action_name'], ENT_COMPAT, 'UTF-8'));
        $instance->setActionTarget($asset['action_target']);

        if (isset($asset['update_date']) && Helper::isValidDate($asset['update_date'], "Y-m-d H:i:s")) {
            $instance->setUpdateDate(new DateTime($asset['update_date']));
        }

        if (isset($asset['register_date']) && Helper::isValidDate($asset['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($asset['register_date']));
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}