<?php namespace Skoiy\Ott\Plan\Set;

use PDO;
use Stage\Stage;
use DateTime;
use Stage\Helper;
use Exception;

class SecurityCountry
{
    const ID = 1;

    /**
     * @var
     */
    private $id = -1;
    private $idPlanSet;
    private $exclude;
    private $countryIso;
    /**
     * @var
     */
    private $updateDate;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'exclude', 'countryIso', 'updateDate', 'registerDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdPlanSet()
    {
        return $this->idPlanSet;
    }

    public function getExclude()
    {
        return (bool)$this->exclude;
    }

    public function getCountryIso()
    {
        return $this->countryIso;
    }

    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setIdPlanSet($id)
    {
        $this->idPlanSet = (int) $id;
    }

    public function setExclude($exclude)
    {
        $this->exclude = (bool)$exclude;
    }

    public function setCountryIso($iso)
    {
        $this->countryIso = $iso;
    }

    /**
     * @param mixed $updateDate
     */
    public function setUpdateDate($updateDate)
    {
        $this->updateDate = $updateDate;
    }

    /**
     * @param mixed $registerDate
     */
    public function setRegisterDate($registerDate)
    {
        $this->registerDate = $registerDate;
    }


    /**
     * @param $idPlanSet
     * @param $iso
     * @param $exclude
     * @return SecurityCountry
     * @throws Exception
     * @description Save on table plans_sets_security_countries
     */
    public static function createSecurityCountries($idPlanSet, $iso, $exclude)
    {
        try {
            $parameters = [[$idPlanSet, PDO::PARAM_INT],
                [$iso, PDO::PARAM_STR],
                [$exclude, PDO::PARAM_BOOL]];
            $name = "OTT_PLAN_SET_SECURITY_COUNTRIES";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "-2")
                throw new Exception("Promotion {$idPlanSet} not found");

            return self::createInstance($data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idPlanSet
     * @return boolean
     * @throws Exception
     * @description Delete from table plans_sets_security_countries
     */
    public static function removeAllSecurityCountries($idPlanSet)
    {
        try {
            $parameters = [[$idPlanSet, PDO::PARAM_INT]];
            $name = "OTT_PLAN_SET_SECURITY_COUNTRIES";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("Plan Set {$idPlanSet} not found");
            } else if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0") {
                return true;
            }
            return false;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "updateDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" || $attribute == "nameStripped" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * @param $asset
     * @return SecurityCountry
     * @throws Exception
     */
    public static function createInstance($asset)
    {
        if (isset($asset[0]) && isset($asset[0]['id_security']))
            $asset = $asset[0];

        $instance = new self();
        $instance->setId($asset['id_security']);
        $instance->setIdPlanSet($asset['id_plan_set']);
        $instance->setExclude($asset['exclude']);
        $instance->setCountryIso($asset['country_iso']);

        if (isset($asset['update_date']) && Helper::isValidDate($asset['update_date'], "Y-m-d H:i:s")) {
            $instance->setUpdateDate(new DateTime($asset['update_date']));
        }

        if (isset($asset['register_date']) && Helper::isValidDate($asset['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($asset['register_date']));
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}