<?php namespace Skoiy\Ott\Plan\Set;

use PDO;
use Skoiy\Ott\Plan\Plan;
use Stage\Stage;
use DateTime;
use Stage\Helper;
use Exception;

class Set
{
    const ID = 1;

    private $id = -1;
    private $managementName = "";
    private $nameStripped = "";
    private $active;
    private $domains = "";
    private $exclude;
    private $totalPlans = 0;

    /**
     * @var
     */
    private $updateDate;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'managementName', 'nameStripped', 'active', 'domains', 'exclude', 'totalPlans', 'updateDate', 'registerDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }


    public function getManagementName()
    {
        return $this->managementName;
    }

    public function getNameStripped()
    {
        return $this->nameStripped;
    }

    public function isActive()
    {
        return $this->active;
    }

    public function getDomains()
    {
        return $this->domains;
    }

    public function exclude()
    {
        return (bool)$this->exclude;
    }

    public function totalPlans()
    {
        return $this->totalPlans;
    }

    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    /**
     * @return mixed
     */
    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    private function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setManagementName($name)
    {
        $this->managementName = $name;
    }

    public function setNameStripped($name)
    {
        $this->nameStripped = $name;
    }

    public function setActive($active)
    {
        $this->active = (bool)$active;
    }

    public function setDomains($domains)
    {
        $this->domains = $domains;
    }

    public function setExclude($exclude)
    {
        $this->exclude = (bool)$exclude;
    }

    public function setTotalPlans($total)
    {
        $this->totalPlans = $total;
    }

    /**
     * @param mixed $updateDate
     */
    private function setUpdateDate($updateDate)
    {
        $this->updateDate = $updateDate;
    }

    /**
     * @param mixed $registerDate
     */
    private function setRegisterDate($registerDate)
    {
        $this->registerDate = $registerDate;
    }

    /**
     * @return array
     * @throws Exception
     */
    public static function sizeof()
    {
        $name = "OTT_PLAN_SETS_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * @param $idPlan
     * @return Set
     * @throws Exception
     */
    public static function find($idPlan)
    {
        try {
            $name = "OTT_PLAN_SET";
            $method = "GET";
            $parameters = [[$idPlan, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The PlanSet id $idPlan not exists.", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param Query $q
     * @return array|User[]
     * @throws Exception
     */
    public static function query(Query $q)
    {
        // TODO
        $field = $q->by;
        $fieldSnd = $q->sndBy;
        $fieldTrd = $q->trdFilter;
        $query = $q->term;
        $filter = $q->filter;
        $filterSnd = $q->sndFilter;
        $filterTrd = $q->trdFilter;
        $limitA = $q->from;
        $limitB = $q->to;
        $orderBy = $q->orderBy;
        $order = $q->order;

        try {
            $parameters = [[$field, PDO::PARAM_INT],
                [$fieldSnd, PDO::PARAM_INT],
                [$fieldTrd, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$filterSnd, PDO::PARAM_STR],
                [$filterTrd, PDO::PARAM_STR],
                [$orderBy, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT],
                [$limitA, PDO::PARAM_INT],
                [$limitB, PDO::PARAM_INT],];

            $name = "OTT_PLAN_SETS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return array_map(function ($user) {
                return self::createInstance($user);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param Query $q
     * @return mixed
     * @throws Exception
     */
    public static function queryCount(Query $q)
    {
        // TODO
        $field = $q->by;
        $fieldSnd = $q->sndBy;
        $fieldTrd = $q->trdFilter;
        $query = $q->term;
        $filter = $q->filter;
        $filterSnd = $q->sndFilter;
        $filterTrd = $q->trdFilter;
        $limitA = $q->from;
        $limitB = $q->to;
        $orderBy = $q->orderBy;
        $order = $q->order;

        try {
            $parameters = [[$field, PDO::PARAM_INT],
                [$fieldSnd, PDO::PARAM_INT],
                [$fieldTrd, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$filterSnd, PDO::PARAM_STR],
                [$filterTrd, PDO::PARAM_STR]];

            $name = "OTT_PLAN_SETS_COUNT";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]['total']) ? $data[0]['total'] : 0;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idPlan
     * @return array
     * @throws Exception
     */
    public static function isoLanguages($idPlan)
    {
        try {
            $name = "OTT_PLAN_SET_ISOLANGUAGES";
            $method = "GET";
            $parameters = [[$idPlan, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Plan id $idPlan not exists.", $data[0]['return_code']);
            } else {
                return $data;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Get all L10n's for the PlanSet
     *
     * @param $idPlanSet
     * @return L10n[]
     * @throws Exception
     */
    public static function l10n($idPlanSet)
    {
        try {
            return L10n::find($idPlanSet);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Get all the security countries for the PlanSet
     *
     * @param $idPlanSet
     * @return array|SecurityCountry[]
     * @throws Exception
     */
    public static function securityCountries($idPlanSet)
    {
        try {
            $name = "OTT_PLAN_SET_SECURITY_COUNTRIES";
            $method = "GET";
            $parameters = [[$idPlanSet, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                return [];
            } else {
                return array_map(function ($l10n) {
                    return SecurityCountry::createInstance($l10n);
                }, $data);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Get all the Plans related with the PlanSet $idPlanSet
     * @param $idPlanSet
     * @return Plan[]
     * @throws Exception
     */
    public static function plans($idPlanSet)
    {
        try {
            $name = "OTT_PLAN_SET_PLANS";
            $method = "GET";
            $parameters = [[$idPlanSet, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                return [];
            } else {
                return array_map(function ($l10n) {
                    return Plan::createInstance($l10n);
                }, $data);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Persist on database the PlanSet
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        try {
            $name = "OTT_PLAN_SET";
            $method = "SET";
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                [$this->getManagementName(), PDO::PARAM_STR],
                [$this->getNameStripped(), PDO::PARAM_STR],
                [$this->isActive(), PDO::PARAM_BOOL],
                [$this->getDomains(), PDO::PARAM_STR],
                [$this->exclude(), PDO::PARAM_BOOL]];

            $data = Stage::esp($parameters, $name, $method);

            if (!isset($data[0]) && !isset($data[0]['id_plan_set'])) {
                throw new Exception("Error saving a new Plan L10n");
            }

            $this->setId($data[0]['id_plan_set']);
            if (isset($data[0]['update_date']) && Helper::isValidDate($data[0]['update_date'], "Y-m-d H:i:s")) {
                $this->setRegisterDate(new DateTime($data[0]['update_date']));
            }
            if (isset($data[0]['register_date']) && Helper::isValidDate($data[0]['register_date'], "Y-m-d H:i:s")) {
                $this->setRegisterDate(new DateTime($data[0]['register_date']));
            }

            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @return Set
     * @throws Exception
     */
    public static function duplicate($idPlanSet)
    {
        try {
            $parameters = [[$idPlanSet, PDO::PARAM_INT]];
            $name = "OTT_PLAN_SET_CLONE";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            return self::createInstance($data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idPlanSet
     * @return bool
     * @throws Exception
     * @description Writes on table plans_sets
     */
    public static function delete($idPlanSet)
    {
        try {
            $parameters = [[$idPlanSet, PDO::PARAM_INT]];
            $name = "OTT_PLAN_SET";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("Plan {$idPlanSet} not found");
            } else if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0") {
                return true;
            }
            return false;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @return boolean
     * @throws Exception
     * @description Save on table plans_sets_plans_related
     */
    public static function addPlan($idPlanSet, $idPlan, $isDefault, $position)
    {
        try {
            $parameters = [[$idPlanSet, PDO::PARAM_INT],
                [$idPlan, PDO::PARAM_INT],
                [$isDefault, PDO::PARAM_BOOL],
                [$position, PDO::PARAM_INT]];
            $name = "OTT_PLAN_SET_PLANS_RELATED";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0" ? true : false;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $type
     * @param $idPlanSet
     * @param $idPlan
     * @return bool
     * @throws Exception
     */
    public static function removePlan($type, $idPlanSet, $idPlan)
    {
        try {
            $parameters = [[$type, PDO::PARAM_INT],
                [$idPlanSet, PDO::PARAM_INT],
                [$idPlan, PDO::PARAM_INT]];
            $name = "OTT_PLAN_SETS_PLAN";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("Plan {$idPlanSet} not found");
            } else if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0") {
                return true;
            }
            return false;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "updateDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" || $attribute == "nameStripped" || $attribute == "imageTitle" || $attribute == "imageAlterText" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * @param $asset
     * @return Freebie
     * @throws Exception
     */
    public static function createInstance($asset)
    {
        if (isset($asset[0]) && isset($asset[0]['id_plan_set']))
            $asset = $asset[0];

        $instance = new self();
        $instance->setId($asset['id_plan_set']);
        $instance->setManagementName(html_entity_decode($asset['management_name'], ENT_COMPAT, 'UTF-8'));
        $instance->setNameStripped($asset['name_stripped']);
        $instance->setActive($asset['active']);
        $instance->setDomains($asset['domains']);
        $instance->setExclude($asset['exclude']);
        if (isset($asset['total_plans'])) {
            $instance->setTotalPlans($asset['total_plans']);
        }

        if (isset($asset['update_date']) && Helper::isValidDate($asset['update_date'], "Y-m-d H:i:s")) {
            $instance->setUpdateDate(new DateTime($asset['update_date']));
        }

        if (isset($asset['register_date']) && Helper::isValidDate($asset['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($asset['register_date']));
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }

}