<?php namespace Skoiy\Ott\Promotion;

use PDO;
use Stage\Stage;
use DateTime;
use Stage\Helper;
use Exception;

class Email
{
    const ID = 1;

    /**
     * @var
     */
    private $id = -1;
    private $idPromotion;
    private $email;
    private $active;

    /**
     * @var
     */
    private $updateDate;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'idPromotion', 'email', 'active', 'updateDate', 'registerDate');

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdPromotion()
    {
        return $this->idPromotion;
    }

    public function getEmail()
    {
        return $this->email;
    }

    public function isActive()
    {
        return (bool)$this->active;
    }

    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function setId($id_promotion)
    {
        $this->id = $id_promotion;
    }

    public function setIdPromotion($id_promotion)
    {
        $this->idPromotion = $id_promotion;
    }

    public function setEmail($email)
    {
        $this->email = $email;
    }

    public function setActive($active)
    {
        $this->active = (bool)$active;
    }

    /**
     * @param mixed $updateDate
     */
    public function setUpdateDate($updateDate)
    {
        $this->updateDate = $updateDate;
    }

    /**
     * @param mixed $registerDate
     */
    public function setRegisterDate($registerDate)
    {
        $this->registerDate = $registerDate;
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "updateDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" || $attribute == "nameStripped" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * @param $asset
     * @return Email
     * @throws Exception
     */
    public static function createInstance($asset)
    {
        if (isset($asset[0]) && isset($asset[0]['id_code']))
            $asset = $asset[0];

        $instance = new self();
        $instance->setId($asset['id']);
        $instance->setIdPromotion($asset['id_promotion']);
        $instance->setEmail($asset['email']);
        $instance->setActive($asset['active']);

        if (isset($asset['update_date']) && Helper::isValidDate($asset['update_date'], "Y-m-d H:i:s")) {
            $instance->setUpdateDate(new DateTime($asset['update_date']));
        }

        if (isset($asset['register_date']) && Helper::isValidDate($asset['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($asset['register_date']));
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }

}