<?php namespace Skoiy\Ott\Promotion;

use PDO;
use Skoiy\Ott\Plan\Plan;
use Stage\Stage;
use DateTime;
use Stage\Helper;
use Exception;

class Promotion
{
    const ID = 1;

    /**
     * @var
     */
    private $id = -1;
    private $promotionType;
    private $percentOff;
    private $name;
    private $priceOff;
    private $currencyIso;
    private $nameStripped;
    private $beginDate;
    private $endDate;
    private $appliedOnRenewal;
    private $appliedToAllPeriods;
    private $billingPeriod;
    private $codeRequired;
//    private $emailListEnabled;
    private $emailDomains;
    private $active;
    private $totalPlans = 0;

    /**
     * @var
     */
    private $updateDate;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'promotionType', 'percentOff', 'currencyIso', 'name', 'priceOff', 'nameStripped', 'beginDate', 'endDate', 'appliedOnRenewal', 'appliedToAllPeriods', 'billingPeriod', 'codeRequired', 'emailDomains', 'active', 'totalPlans', 'updateDate', 'registerDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getPromotionType()
    {
        return $this->promotionType;
    }

    public function getPercentOff()
    {
        return $this->percentOff;
    }

    public function getPriceOff()
    {
        return $this->priceOff;
    }

    public function getCurrencyIso()
    {
        return $this->currencyIso;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getNameStripped()
    {
        return $this->nameStripped;
    }

    public function getBeginData()
    {
        return $this->beginDate;
    }

    public function getEndDate()
    {
        return $this->endDate;
    }

    public function getAppliedOnRenewal()
    {
        return $this->appliedOnRenewal;
    }

    public function getAppliedToAllPeriods()
    {
        return $this->appliedToAllPeriods;
    }

    public function getBillingPeriod()
    {
        return $this->billingPeriod;
    }

    public function getCodeRequired()
    {
        return $this->codeRequired;
    }

    public function getEmailListEnabled()
    {
        return $this->emailListEnabled;
    }

    public function getEmailDomains()
    {
        return $this->emailDomains;
    }

    public function isActive()
    {
        return (bool)$this->active;
    }

    public function getTotalPlans()
    {
        return $this->totalPlans;
    }

    /**
     * @return mixed
     */
    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    /**
     * @return mixed
     */
    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function setId($id_promotion)
    {
        $this->id = (int)$id_promotion;
    }

    public function setPromotionType($promotion_type)
    {
        $this->promotionType = $promotion_type;
    }

    public function setPercentOff($percent_off)
    {
        $this->percentOff = $percent_off;
    }

    public function setPriceOff($price_off)
    {
        $this->priceOff = $price_off;
    }

    public function setCurrencyIso($iso)
    {
        $this->currencyIso = $iso;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setNameStripped($name_stripped)
    {
        $this->nameStripped = $name_stripped;
    }

    public function setBeginDate($begin_date)
    {
        $this->beginDate = $begin_date;
    }

    public function setEndDate($end_date)
    {
        $this->endDate = $end_date;
    }

    public function setAppliedOnRenewal($applied_on_renewal)
    {
        $this->appliedOnRenewal = (bool)$applied_on_renewal;
    }

    public function setAppliedToAllPeriods($applied_to_all_periods)
    {
        $this->appliedToAllPeriods = (bool)$applied_to_all_periods;
    }

    public function setBillingPeriod($billing_period)
    {
        $this->billingPeriod = (int)$billing_period;
    }

    public function setCodeRequired($code_required)
    {
        $this->codeRequired = (bool)$code_required;
    }

    public function setEmailListEnabled($email_list_enabled)
    {
        $this->emailListEnabled = (bool)$email_list_enabled;
    }

    public function setEmailDomains($email_domains)
    {
        $this->emailDomains = $email_domains;
    }

    public function setActive($active)
    {
        $this->active = (bool)$active;
    }

    public function setTotalPlans($total)
    {
        $this->totalPlans = (int)$total;
    }

    /**
     * @param mixed $updateDate
     */
    public function setUpdateDate($updateDate)
    {
        $this->updateDate = $updateDate;
    }

    /**
     * @param mixed $registerDate
     */
    public function setRegisterDate($registerDate)
    {
        $this->registerDate = $registerDate;
    }

    /**
     * @return array
     * @throws Exception
     */
    public static function sizeof()
    {
        $name = "OTT_PROMOTIONS_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * @param $idPromotion
     * @return Promotion
     * @throws Exception
     */
    public static function find($idPromotion)
    {
        try {
            $name = "OTT_PROMOTION";
            $method = "GET";
            $parameters = [[$idPromotion, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Promotion id $idPromotion not exists.", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        try {
            $name = "OTT_PROMOTION";
            $method = "SET";
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                [$this->getPromotionType(), PDO::PARAM_INT],
                [$this->getPercentOff(), PDO::PARAM_INT],
                [$this->getPriceOff(), PDO::PARAM_INT],
                [$this->getCurrencyIso(), PDO::PARAM_STR],
                [$this->getName(), PDO::PARAM_STR],
                [$this->getNameStripped(), PDO::PARAM_STR],
                [$this->getBeginData(), PDO::PARAM_STR],
                [$this->getEndDate(), PDO::PARAM_STR],
                [$this->getAppliedOnRenewal(), PDO::PARAM_INT],
                [$this->getAppliedToAllPeriods(), PDO::PARAM_INT],
                [$this->getBillingPeriod(), PDO::PARAM_INT],
                [$this->getCodeRequired(), PDO::PARAM_BOOL],
//                [$this->getEmailListEnabled(), PDO::PARAM_BOOL],
//                [$this->getEmailDomains(), PDO::PARAM_STR],
                [$this->isActive(), PDO::PARAM_BOOL]];

            $data = Stage::esp($parameters, $name, $method);

            if (!isset($data[0]) && !isset($data[0]['id_plan_set'])) {
                throw new Exception("Error saving a new Plan L10n");
            }

            $this->setId($data[0]['id_promotion']);
            if (isset($data[0]['update_date']) && Helper::isValidDate($data[0]['update_date'], "Y-m-d H:i:s")) {
                $this->setRegisterDate(new DateTime($data[0]['update_date']));
            }
            if (isset($data[0]['register_date']) && Helper::isValidDate($data[0]['register_date'], "Y-m-d H:i:s")) {
                $this->setRegisterDate(new DateTime($data[0]['register_date']));
            }

            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @return Promotion
     * @throws Exception
     */
    public static function duplicate($idPromotion)
    {
        try {
            $parameters = [[$idPromotion, PDO::PARAM_INT]];
            $name = "OTT_PROMOTION_CLONE";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "-2")
                throw new Exception("Promotion {$idPromotion} not found");

            return self::createInstance($data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @return boolean
     * @throws Exception
     * @description Writes on table promotions_plans_related
     */
    public static function addPlan($idPromotion, $idPlan, $position)
    {
        try {
            $parameters = [[$idPromotion, PDO::PARAM_INT],
                [$idPlan, PDO::PARAM_INT],
                [$position, PDO::PARAM_INT]];
            $name = "OTT_PROMOTION_PLANS_RELATED";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0" ? true : false;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @return boolean
     * @throws Exception
     * @description Writes on table promotions_plans_related
     */
    public static function removePlan($type, $idPromotion, $idPlan)
    {
        try {
            $parameters = [[$type, PDO::PARAM_INT],
                [$idPromotion, PDO::PARAM_INT],
                [$idPlan, PDO::PARAM_INT]];
            $name = "OTT_PROMOTION_PLAN";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0" ? true : false;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idPromotion
     * @param $code
     * @param $quantity
     * @param $ulimited boolean
     * @param $active boolean
     * @return Code
     * @throws Exception
     * @description Writes on table promotions_codes_related
     */
    public static function addCode($idPromotion, $code, $quantity, $unlimited, $active, $searchable, $randomCode, $prefix, $numbers, $lowercase, $uppercase, $length, $totalCodes, $idCode = -1)
    {
        try {
            return Code::save($idCode, $idPromotion, $code, $quantity, $unlimited, $active, $searchable, $randomCode, $prefix, $numbers, $lowercase, $uppercase, $length, $totalCodes);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $type
     * @param $idPromotion
     * @param $idCode
     * @return bool
     * @throws Exception
     * @description Writes on table
     */
    public static function removeCode($type, $idPromotion, $idCode)
    {
        try {
            $parameters = [[$type, PDO::PARAM_INT],
                [$idPromotion, PDO::PARAM_INT],
                [$idCode, PDO::PARAM_INT]];
            $name = "OTT_PROMOTION_CODE";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0" ? true : false;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * @param $idPromotion
     * @return bool
     * @throws Exception
     */
    public static function delete($idPromotion)
    {
        try {
            $parameters = [[$idPromotion, PDO::PARAM_INT]];
            $name = "OTT_PROMOTION";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("Promotion {$idPromotion} not found");
            } else if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0") {
                return true;
            }
            return false;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idPromotion
     * @return Plan[]
     * @throws Exception
     */
    public static function plans($idPromotion)
    {
        try {
            $name = "OTT_PROMOTION_PLANS";
            $method = "GET";
            $parameters = [[$idPromotion, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Promotion id $idPromotion not exists.", $data[0]['return_code']);
            } else {
                return array_map(function ($l10n) {
                    return Plan::createInstance($l10n);
                }, $data);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idPromotion
     * @return array|SecurityCountry[]
     * @throws Exception
     */
    public static function securityCountries($idPromotion)
    {
        try {
            $name = "OTT_PROMOTION_SECURITY_COUNTRIES";
            $method = "GET";
            $parameters = [[$idPromotion, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                return [];
            } else {
                return array_map(function ($l10n) {
                    return SecurityCountry::createInstance($l10n);
                }, $data);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param Query $q
     * @return array|Code[]|Email[]|Used[]
     * @throws Exception
     * @tables promotions_emails_related promotions_codes_related promotions_used
     */
    public static function query(Query $q)
    {
        $field = $q->by;
        $fieldSnd = $q->sndBy;
        $fieldTrd = $q->trdBy;
        $query = $q->term;
        $filter = $q->filter;
        $filterSnd = $q->sndFilter;
        $filterTrd = $q->trdFilter;
        $limitA = $q->from;
        $limitB = $q->to;
        $orderBy = $q->orderBy;
        $order = $q->order;

        try {
            $parameters = [[$field, PDO::PARAM_INT],
                [$fieldSnd, PDO::PARAM_INT],
                [$fieldTrd, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$filterSnd, PDO::PARAM_STR],
                [$filterTrd, PDO::PARAM_STR],
                [$orderBy, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT],
                [$limitA, PDO::PARAM_INT],
                [$limitB, PDO::PARAM_INT]];

            if ($q instanceof CodeQuery) {
                $name = "OTT_PROMOTION_CODES";
//                [':field', ':field_snd', ':field_trd', ':query', ':filter', ':filter_snd', ':filter_trd', ':limit_a', ':limit_b', ':order', ':order_by']
            } else if ($q instanceof EmailQuery) {
                $name = "OTT_PROMOTION_WHITELIST";
//                [':field', ':field_snd', ':field_trd', ':query', ':filter', ':filter_snd', ':filter_trd', ':limit_a', ':limit_b', ':order', ':order_by']
            } else if ($q instanceof UsedQuery) {
                $name = "OTT_PROMOTION_USED";
//                [':field', ':field_snd', ':field_trd', ':query', ':filter', ':filter_snd', ':filter_trd', ':limit_a', ':limit_b', ':order', ':order_by']
            } else if ($q instanceof PromotionQuery) {
                $name = "OTT_PROMOTIONS";
//                [':field', ':field_snd', ':field_trd', ':query', ':filter', ':filter_snd', ':filter_trd', ':limit_a', ':limit_b', ':order', ':order_by']
            }

            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return array_map(function ($user) use ($q) {
                if ($q instanceof CodeQuery) {
                    return Code::createInstance($user);
                } else if ($q instanceof EmailQuery) {
                    return WhiteList::createInstance($user);
                } else if ($q instanceof UsedQuery) {
                    return Used::createInstance($user);
                } else if ($q instanceof PromotionQuery) {
                    return self::createInstance($user);
                }
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param Query $q
     * @return int|mixed
     * @throws Exception
     * @tables promotions_emails_related promotions_codes_related promotions_used
     */
    public static function queryCount(Query $q)
    {
        $field = $q->by;
        $fieldSnd = $q->by;
        $fieldTrd = $q->by;
        $query = $q->term;
        $filter = $q->filter;
        $filterSnd = $q->filter;
        $filterTrd = $q->filter;
        $limitA = $q->from;
        $limitB = $q->to;
        $orderBy = $q->orderBy;
        $order = $q->order;

        try {
            $parameters = [[$field, PDO::PARAM_INT],
                [$fieldSnd, PDO::PARAM_INT],
                [$fieldTrd, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$filterSnd, PDO::PARAM_STR],
                [$filterTrd, PDO::PARAM_STR]];

            if ($q instanceof CodeQuery) {
                $name = "OTT_PROMOTION_CODES_COUNT";
//                  [':field', ':field_snd', ':field_trd', ':query', ':filter', ':filter_snd', ':filter_trd']
            } else if ($q instanceof EmailQuery) {
                $name = "OTT_PROMOTION_WHITELIST_COUNT";
//                  [':field', ':field_snd', ':field_trd', ':query', ':filter', ':filter_snd', ':filter_trd']
            } else if ($q instanceof UsedQuery) {
                $name = "OTT_PROMOTION_USED_COUNT";
//                  [':field', ':field_snd', ':field_trd', ':query', ':filter', ':filter_snd', ':filter_trd']
            } else if ($q instanceof PromotionQuery) {
                $name = "OTT_PROMOTIONS_COUNT";
//                  [':field', ':field_snd', ':field_trd', ':query', ':filter', ':filter_snd', ':filter_trd']
            }

            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]['total']) ? $data[0]['total'] : 0;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $id
     * @param $idPromotion
     * @param $value
     * @param $type // type1 = email type2=domain
     * @param $active boolean
     * @return WhiteList
     * @throws Exception
     * @description Writes on table promotions_codes_related
     */
    public static function addToWhitelist($id, $idPromotion, $value, $type, $active)
    {
        try {
            return WhiteList::save($id, $idPromotion, $value, $type, $active);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $type // type1 = email type2=domain
     * @param $idPromotion
     * @param $email
     * @return bool
     * @throws Exception
     * @description Writes on table
     */
    public static function removeFromWhitelist($type, $idPromotion, $id)
    {
        try {
            $parameters = [[$type, PDO::PARAM_INT],
                [$idPromotion, PDO::PARAM_INT],
                [$id, PDO::PARAM_INT]];
            $name = "OTT_PROMOTION_WHITELIST";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0";
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * codeExist
     *
     * @param $code
     * @return array
     * @throws Exception
     */
    public static function codeExist($code)
    {
        try {
            $parameters = [[$code, PDO::PARAM_STR]];
            $name = "OTT_PROMOTION_CODE_NUM";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "updateDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" || $attribute == "nameStripped" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * @param $asset
     * @return Promotion
     * @throws Exception
     */
    public static function createInstance($asset)
    {
        if (isset($asset[0]) && isset($asset[0]['id_promotion']))
            $asset = $asset[0];

        $instance = new self();
        $instance->setId($asset['id_promotion']);
        $instance->setPromotionType($asset['promotion_type']);
        $instance->setPercentOff($asset['percent_off']);
        if (isset($asset['currency_iso'])) {
            $instance->setCurrencyIso($asset['currency_iso']);
        }
        $instance->setPriceOff($asset['price_off']);
        $instance->setName(html_entity_decode($asset['name'], ENT_COMPAT, 'UTF-8'));
        $instance->setNameStripped($asset['name_stripped']);
        $instance->setBeginDate($asset['begin_date']);
        $instance->setEndDate($asset['end_date']);
        $instance->setAppliedOnRenewal($asset['applied_on_renewal']);
        $instance->setAppliedToAllPeriods($asset['applied_to_all_periods']);
        $instance->setBillingPeriod($asset['billing_period']);
        $instance->setCodeRequired($asset['code_required']);
        //$instance->setEmailListEnabled($asset['email_list_enabled']);
        //$instance->setEmailDomains($asset['email_domains']);
        $instance->setActive($asset['active']);

        if (isset($asset['total_plans']))
            $instance->setTotalPlans($asset['total_plans']);

        if (isset($asset['update_date']) && Helper::isValidDate($asset['update_date'], "Y-m-d H:i:s")) {
            $instance->setUpdateDate(new DateTime($asset['update_date']));
        }

        if (isset($asset['register_date']) && Helper::isValidDate($asset['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($asset['register_date']));
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}