<?php namespace Skoiy\Ott\Scan;

use PDO;
use Stage\Stage;
use DateTime;
use Stage\Helper;
use Exception;

class Code
{
    const ID = 1;

    /**
     * @var
     */
    private $id = -1;
    private $idScan;
    private $code;
    private $quantity;
    private $unlimited;
    private $active;
    private $maxScansPerUser;
    private $dailyLimitPerUser;
    private $scanIntervalPerUser;
    private $total;

    /**
     * @var
     */
    private $updateDate;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'idScan', 'code', 'quantity', 'unlimited', 'active', 'maxScansPerUser', 'dailyLimitPerUser', 'scanIntervalPerUser', 'total', 'updateDate', 'registerDate');

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdScan()
    {
        return $this->idScan;
    }

    public function getCode()
    {
        return $this->code;
    }

    public function getQuantity()
    {
        return $this->quantity;
    }

    public function isUnlimited()
    {
        return (bool)$this->unlimited;
    }

    public function isActive()
    {
        return (bool)$this->active;
    }

    public function getMxScansPerUser()
    {
        return $this->maxScansPerUser;
    }

    public function getDailyLimitPerUser()
    {
        return $this->dailyLimitPerUser;
    }

    public function getScanIntervalPerUser()
    {
        return $this->scanIntervalPerUser;
    }

    public function getTotal()
    {
        return (int)$this->total;
    }

    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setIdScan($id)
    {
        $this->idScan = (int)$id;
    }

    public function setCode($code)
    {
        $this->code = $code;
    }

    public function setQuantity($qty)
    {
        $this->quantity = (int)$qty;
    }

    public function setUnlimited($ulimited)
    {
        $this->unlimited = (bool)$ulimited;
    }

    public function setActive($active)
    {
        $this->active = (bool)$active;
    }

    public function setTotal($total)
    {
        $this->total = (int)$total;
    }

    public function setMaxScansPerUser($total)
    {
        $this->maxScansPerUser = (int)$total;
    }

    public function setDailyLimitPerUser($total)
    {
        $this->dailyLimitPerUser = (int)$total;
    }

    public function setScanIntervalPerUser($total)
    {
        $this->scanIntervalPerUser = (int)$total;
    }

    /**
     * @param mixed $updateDate
     */
    public function setUpdateDate($updateDate)
    {
        $this->updateDate = $updateDate;
    }

    /**
     * @param mixed $registerDate
     */
    public function setRegisterDate($registerDate)
    {
        $this->registerDate = $registerDate;
    }

    /**
     * save
     *
     * @param $idCode
     * @param $idScan
     * @param $code
     * @param $quantity
     * @param $unlimited
     * @param $active boolean
     * @param $maxScansPerUser
     * @param $dailyLimitPerUser
     * @param $scanIntervalPerUser
     * @param $randomCode
     * @param $prefix
     * @param $numbers
     * @param $lowercase
     * @param $uppercase
     * @param $length
     * @param $totalCodes
     * @return Code
     * @throws Exception
     * @description Writes on table scans_codes_related
     */
    public static function save($idCode, $idScan, $code, $quantity, $unlimited, $active, $maxScansPerUser, $dailyLimitPerUser, $scanIntervalPerUser, $randomCode, $prefix, $numbers, $lowercase, $uppercase, $length, $totalCodes)
    {
        try {
            $parameters = [[$idCode, PDO::PARAM_INT],
                [$idScan, PDO::PARAM_INT],
                [$code, PDO::PARAM_STR],
                [$quantity, PDO::PARAM_INT],
                [$unlimited, PDO::PARAM_BOOL],
                [$active, PDO::PARAM_BOOL],
                [$maxScansPerUser, PDO::PARAM_INT],
                [$dailyLimitPerUser, PDO::PARAM_INT],
                [$scanIntervalPerUser, PDO::PARAM_INT],
                [$randomCode, PDO::PARAM_INT],
                [$prefix, PDO::PARAM_STR],
                [$numbers, PDO::PARAM_INT],
                [$lowercase, PDO::PARAM_INT],
                [$uppercase, PDO::PARAM_INT],
                [$length, PDO::PARAM_INT],
                [$totalCodes, PDO::PARAM_INT]];

            $name = "OTT_SCAN_CODES_RELATED";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("Scan {$idScan} not found");
            } else if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "-3") {
                throw new Exception("Scan Code {$code} duplicated");
            }
            return self::createInstance($data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "updateDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" || $attribute == "nameStripped" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * @param $asset
     * @return Code
     * @throws Exception
     */
    public static function createInstance($asset)
    {
        if (isset($asset[0]) && isset($asset[0]['id_code']))
            $asset = $asset[0];

        $instance = new self();
        $instance->setId($asset['id_code']);
        $instance->setIdScan($asset['id_scan']);
        $instance->setCode($asset['code']);
        $instance->setQuantity($asset['quantity']);
        $instance->setUnlimited($asset['unlimited']);
        $instance->setActive($asset['active']);
        $instance->setMaxScansPerUser($asset['max_scans_per_user']);
        $instance->setDailyLimitPerUser($asset['daily_limit_per_user']);
        $instance->setScanIntervalPerUser($asset['scan_interval_per_user']);
        $instance->setTotal($asset['total']);

        if (isset($asset['update_date']) && Helper::isValidDate($asset['update_date'], "Y-m-d H:i:s")) {
            $instance->setUpdateDate(new DateTime($asset['update_date']));
        }

        if (isset($asset['register_date']) && Helper::isValidDate($asset['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($asset['register_date']));
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }

}