<?php namespace Skoiy\Ott\Scan;

use PDO;
use Skoiy\Ott\MyArrayObject;
use Stage\Paginator;
use Stage\Stage;
use DateTime;
use Stage\Helper;
use Exception;

class Scan
{
    const ID = 1;

    private $id = -1;

    /**
     * @var
     */
    private $managementName;

    private $scanType;
    private $scanToken;
    private $isActive;
    private $startDate;
    private $endDate;

    /**
     * @var
     */
    private $updateDate;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'managementName', 'scanType', 'scanToken', 'startDate', 'endDate', 'isActive', 'updateDate', 'registerDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getManagementName()
    {
        return $this->managementName;
    }

    public function getScanType()
    {
        return $this->scanType;
    }

    public function getStartDate()
    {
        return $this->startDate;
    }

    public function getEndDate()
    {
        return $this->endDate;
    }

    public function isActive()
    {
        return (bool)$this->isActive;
    }

    /**
     * @return mixed
     */
    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    /**
     * @return mixed
     */
    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setManagementName($name)
    {
        $this->managementName = $name;
    }

    public function setScanType($type)
    {
        $this->scanType = $type;
    }

    public function setScanToken($token)
    {
        $this->scanToken = $token;
    }

    /**
     * @param mixed $active
     */
    public function setActive($active)
    {
        $this->isActive = (bool)$active;
    }

    public function setStartDate($startDate)
    {
        $this->startDate = $startDate;
    }

    public function setEndDate($endDate)
    {
        $this->endDate = $endDate;
    }

    /**
     * @param mixed $updateDate
     */
    private function setUpdateDate($updateDate)
    {
        $this->updateDate = $updateDate;
    }

    /**
     * @param mixed $registerDate
     */
    private function setRegisterDate($registerDate)
    {
        $this->registerDate = $registerDate;
    }

    /**
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        try {
            // [':id_scan', ':management_name', ':scan_type', ':start_date', ':end_date', ':active']
            $name = "OTT_SCANS";
            $method = "SET";
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                [$this->getManagementName(), PDO::PARAM_STR],
                [$this->getScanType(), PDO::PARAM_STR],
                [$this->getStartDate(), PDO::PARAM_STR],
                [$this->getEndDate(), PDO::PARAM_STR],
                [$this->isActive(), PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            if (!isset($data[0]) && !isset($data[0]['id_scan'])) {
                throw new Exception("Error saving a new Scan");
            }

            $this->setId($data[0]['id_scan']);
            if (isset($data[0]['update_date']) && Helper::isValidDate($data[0]['update_date'], "Y-m-d H:i:s")) {
                $this->setRegisterDate(new DateTime($data[0]['update_date']));
            }
            if (isset($data[0]['register_date']) && Helper::isValidDate($data[0]['register_date'], "Y-m-d H:i:s")) {
                $this->setRegisterDate(new DateTime($data[0]['register_date']));
            }

            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @return array
     * @throws Exception
     */
    public static function sizeof()
    {
        $name = "OTT_SCANS_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * @param $id
     * @return self
     * @throws Exception
     */
    public static function find($id)
    {
        try {
            $name = "OTT_SCAN";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Scan id $id not exists.", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * isoLanguages
     *
     * @param $idScan
     * @return array
     * @throws Exception
     */
    public static function isoLanguages($idScan)
    {
        try {
            $name = "OTT_SCANS_ISOLANGUAGES";
            $method = "GET";
            $parameters = [[$idScan, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Scan id $idScan not exists.", $data[0]['return_code']);
            } else {
                return $data;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * query
     *
     * @param Query $q
     * @param bool $count
     * @param int $page
     * @param int $perPage
     * @return MyArrayObject
     * @throws Exception
     */
    public static function query(Query $q, $count = true, $page = 1, $perPage = 15)
    {
        $method = "GET";

        $field = $q->by;
        $fieldSnd = $q->sndBy;
        $fieldTrd = $q->trdBy;
        $query = $q->term;
        $filter = $q->filter;
        $filterSnd = $q->sndFilter;
        $filterTrd = $q->trdFilter;
        $limitA = $q->from;
        $limitB = $q->to;
        $orderBy = $q->orderBy;
        $order = $q->order;

        try {
            if ($count == true) {
                $parameters = [
                    [$field, PDO::PARAM_INT],
                    [$fieldSnd, PDO::PARAM_INT],
                    [$fieldTrd, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_STR],
                    [$filterSnd, PDO::PARAM_STR],
                    [$filterTrd, PDO::PARAM_STR],
                ];

                $name = "OTT_SCANS_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limitA = isset($p) ? $p->from() : $q->from;
            $limitB = isset($p) ? $p->to() : $q->to;

            $parameters = [[$field, PDO::PARAM_INT],
                [$fieldSnd, PDO::PARAM_INT],
                [$fieldTrd, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$filterSnd, PDO::PARAM_STR],
                [$filterTrd, PDO::PARAM_STR],
                [$orderBy, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT],
                [$limitA, PDO::PARAM_INT],
                [$limitB, PDO::PARAM_INT],
            ];

            $name = "OTT_SCANS";
            $data = Stage::esp($parameters, $name, $method);

            $result = new MyArrayObject(array_map(function ($label) {
                return self::createInstance($label);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachL10n
     *
     * @param $idScan
     * @param $isoLanguage
     * @param $name
     * @param $nameStripped
     * @param $description
     * @param $image
     * @param int $idL10n
     * @return L10n|null
     * @throws Exception
     */
    public static function attachL10n($idScan, $isoLanguage, $name, $nameStripped, $description, $image, $idL10n = 0)
    {
        $method = "SET";

        try {

            $parameters = [[$idL10n, PDO::PARAM_INT],
                [$idScan, PDO::PARAM_INT],
                [$isoLanguage, PDO::PARAM_STR],
                [$name, PDO::PARAM_STR],
                [$nameStripped, PDO::PARAM_STR],
                [$description, PDO::PARAM_STR],
                [$image, PDO::PARAM_STR],
            ];

            $name = "OTT_SCANS_L10N";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code'])) {
                throw new Exception("The L10n iso language $isoLanguage already exists.", $data[0]['return_code']);
            }

            if (!isset($data[0]) || !isset($data[0]['id_l10n'])) {
                return null;
            }

            return L10n::createInstance($data[0]);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * l10ns
     *
     * @param $idScan
     * @return array
     * @throws Exception
     */
    public static function l10ns($idScan)
    {
        try {

            $parameters = [
                [$idScan, PDO::PARAM_INT],
            ];

            $method = "GET";
            $name = "OTT_SCANS_L10N";
            $data = Stage::esp($parameters, $name, $method);
            return array_map(function ($user) {
                return L10n::createInstance($user);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * queryTeamsAvatars
     *
     * @param Query $q
     * @param bool $count
     * @param int $page
     * @param int $perPage
     * @return MyArrayObject
     * @throws Exception
     */
    public static function queryCodes(Query $q, $count = true, $page = 1, $perPage = 15)
    {
        $method = "GET";

        $field = $q->by;
        $fieldSnd = $q->sndBy;
        $fieldTrd = $q->trdBy;
        $query = $q->term;
        $filter = $q->filter;
        $filterSnd = $q->sndFilter;
        $filterTrd = $q->trdFilter;
        $limitA = $q->from;
        $limitB = $q->to;
        $orderBy = $q->orderBy;
        $order = $q->order;

        try {
            if ($count == true) {
                $parameters = [
                    [$field, PDO::PARAM_INT],
                    [$fieldSnd, PDO::PARAM_INT],
                    [$fieldTrd, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_STR],
                    [$filterSnd, PDO::PARAM_STR],
                    [$filterTrd, PDO::PARAM_STR],
                ];

                $name = "OTT_SCAN_CODES_COUNT";
                $data = Stage::esp($parameters, $name, $method);

                $total = sizeof($data);

                if (!$total) {
                    return new MyArrayObject([]);
                }

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limitA = isset($p) ? $p->from() : $q->from;
            $limitB = isset($p) ? $p->to() : $q->to;

            $parameters = [[$field, PDO::PARAM_INT],
                [$fieldSnd, PDO::PARAM_INT],
                [$fieldTrd, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$filterSnd, PDO::PARAM_STR],
                [$filterTrd, PDO::PARAM_STR],
                [$orderBy, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT],
                [$limitA, PDO::PARAM_INT],
                [$limitB, PDO::PARAM_INT],
            ];

            $name = "OTT_SCAN_CODES";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                $result = new MyArrayObject([]);
                $result->setCount(0);
                $result->setPaginator($p);
                return $result;
            }

            $result = new MyArrayObject(array_map(function ($label) {
                return Code::createInstance($label);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * addCode
     *
     * @param $idScan
     * @param $code
     * @param $quantity
     * @param $unlimited
     * @param $active boolean
     * @param $maxScansPerUser
     * @param $dailyLimitPerUser
     * @param $scanIntervalPerUser
     * @param $randomCode
     * @param $prefix
     * @param $numbers
     * @param $lowercase
     * @param $uppercase
     * @param $length
     * @param $totalCodes
     * @param int $idCode
     * @return Code
     * @throws Exception
     * @description Writes on table promotions_codes_related
     */
    public static function addCode($idScan, $code, $quantity, $unlimited, $active, $maxScansPerUser, $dailyLimitPerUser, $scanIntervalPerUser, $randomCode, $prefix, $numbers, $lowercase, $uppercase, $length, $totalCodes, $idCode = -1)
    {
        try {
            return Code::save($idCode, $idScan, $code, $quantity, $unlimited, $active, $maxScansPerUser, $dailyLimitPerUser, $scanIntervalPerUser, $randomCode, $prefix, $numbers, $lowercase, $uppercase, $length, $totalCodes);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * codeExists
     *
     * @param $code
     * @return int
     * @throws Exception
     */
    public static function codeExists($code)
    {
        try {

            $parameters = [
                [$code, PDO::PARAM_STR],
            ];

            $method = "GET";
            $name = "OTT_SCAN_CODE_NUM";
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['total']) ? $data[0]['total'] : 0;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * @param $idScan
     * @return bool
     * @throws Exception
     */
    public static function delete($idScan)
    {
        try {
            $parameters = [[$idScan, PDO::PARAM_INT]];
            $name = "OTT_SCAN";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("Scan {$idScan} not found");
            } else if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0") {
                return true;
            }
            return false;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * deleteScanCode
     *
     * @param $type
     * @param $idScan
     * @param $idCode
     * @return bool
     * @throws Exception
     */
    public static function deleteScanCode($type, $idScan, $idCode)
    {
        try {
            $parameters = [[$type, PDO::PARAM_INT],
                [$idScan, PDO::PARAM_INT],
                [$idCode, PDO::PARAM_INT]];
            $name = "OTT_SCAN_CODE";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("Scan {$idScan} not found");
            } else if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0") {
                return true;
            }
            return false;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "updateDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" || $attribute == "nameStripped" || $attribute == "imageTitle" || $attribute == "imageAlterText" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * @param $asset
     * @return self
     * @throws Exception
     */
    public static function createInstance($asset)
    {
        if (isset($asset[0]))
            $asset = $asset[0];

        $instance = new self();
        $instance->setId($asset['id_scan']);
        $instance->setManagementName(html_entity_decode($asset['management_name'], ENT_COMPAT, 'UTF-8'));
        $instance->setScanType($asset['scan_type']);
        $instance->setScanToken($asset['scan_token']);
        $instance->setStartDate($asset['start_date']);
        $instance->setEndDate($asset['end_date']);
        $instance->setActive($asset['active']);

        if (isset($asset['update_date']) && Helper::isValidDate($asset['update_date'], "Y-m-d H:i:s")) {
            $instance->setUpdateDate(new DateTime($asset['update_date']));
        }

        if (isset($asset['register_date']) && Helper::isValidDate($asset['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($asset['register_date']));
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }

}