<?php namespace Skoiy\Ott\User\Metric;

use PDO;
use Stage\Stage;
use DateTime;
use Exception;

class Metric
{
    /**
     * @var
     */
    private $value;

    /**
     * @var
     */
    private $total;


    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('value', 'total');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getValue()
    {
        return $this->value;
    }

    /**
     * @return mixed
     */
    public function getTotal()
    {
        return $this->total;
    }

    /**
     * @param mixed $value
     */
    public function setValue($value)
    {
        $this->value = $value;
    }

    /**
     * @param mixed $total
     */
    public function setTotal($total)
    {
        $this->total = $total;
    }

    /**
     * @param Query $q
     * @return array|Metric[]
     * @throws Exception
     */
    public static function query(Query $q)
    {
        $field = $q->by;
        $sndField = $q->sndBy;
        $trdField = $q->trdBy;
        $sndFilter = $q->sndFilter;
        $trdFilter = $q->trdFilter;
        $term = $q->term;
        $filter = $q->filter;
        $limitTrdA = $q->limitTrdA;
        $limitTrdB = $q->limitTrdB;

        try {
            $parameters = [[$field, PDO::PARAM_INT],
                [$sndField, PDO::PARAM_INT],
                [$trdField, PDO::PARAM_INT],
                [$term, PDO::PARAM_STR],
                [$filter, PDO::PARAM_INT],
                [$sndFilter, PDO::PARAM_INT],
                [$trdFilter, PDO::PARAM_INT],
                [$limitTrdA, PDO::PARAM_STR],
                [$limitTrdB, PDO::PARAM_STR]];

            $name = "OTT_USERS_PERFORMANCE";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return array_map(function ($data) {
                return self::createInstance($data);
            }, is_array($data) ? $data : []);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "imageAlterText" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $asset
     */
    public static function createInstance($data)
    {
        if (isset($data[0]))
            $data = $data[0];

        $instance = new self();
        $instance->setValue($data['value']);
        $instance->setTotal($data['total']);

        return $instance;
    }
}