<?php

use Skoiy\Ott\Audience\Audience;
use Skoiy\Ott\Audience\Group;
use Stage\Sps;
use Stage\Stage;
use Service\Connection;
use Stage\Account\Account;
use Stage\StoredProcedure\Manager as StoredProcedureManager;

class AudiencesTest extends \PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        $stageInstance = Stage::getInstance();
        $host = "mysqldb";

        $credentials = [
            'host' => "$host",
            'dbname' => "StageAccount",
            'username' => "root",
            'password' => "secret"
        ];

        $clientCredentials = [
            'host' => "$host",
            'dbname' => "bd1",
            'username' => "root",
            'password' => "secret"
        ];

        $tasksCredentials = [
            'host' => "$host",
            'dbname' => "SkoiyTasks",
            'username' => "root",
            'password' => "secret"
        ];

        $ottCredentials = [
            'host' => "$host",
            'dbname' => "bd279_OTT",
            'username' => "root",
            'password' => "secret"
        ];

        $default = new Connection($credentials);
        $client = new Connection($clientCredentials);
        $tasks = new Connection($tasksCredentials);
        $ott = new Connection($ottCredentials);
        $stageInstance->addConnection('default', $default);
        $stageInstance->addConnection('client', $client);
        $stageInstance->addConnection('tasks', $tasks);
        $stageInstance->addConnection('ott', $ott);

        // 3. Set the Stage stored procedures
        $stageInstance->setSps(Sps::load());

        // Load Skoiy OTT routines ( stored procedures ) and attach to the Stage instance
        $ottRoutines = \Skoiy\Ott\Ott::routines();
        $stageInstance->attachRoutines($ottRoutines);

        // Set the StoredProcedure\Manager instance
        $spm = StoredProcedureManager::getInstance();

        // Set the Stored Procedure Manager
        $stageInstance->setStoredProcedureManager($spm);

        try {
            $idAccount = 3;
            $account = Account::find($idAccount);

            // Get a Stage instance
            $stageInstance->setAccount($account);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function tearDown()
    {
    }

    public function testSaveAudience()
    {
        try {
            $audience = new Skoiy\Ott\Audience\Audience();
            $audience->setName("name");
            $audience->save();
            $this->assertTrue($audience instanceof Audience);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testFindAudience()
    {
        try {
            $audience = \Skoiy\Ott\Audience\Audience::find(1);
            $this->assertTrue($audience instanceof Audience);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testAudienceQuery()
    {
        try {
            $query = new Skoiy\Ott\Audience\Query();
            $result = Audience::query($query);
            foreach ($result as $audience) {
                $this->assertTrue($audience instanceof Audience);
            }
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testDeleteAudience()
    {
        try {
            $status = \Skoiy\Ott\Audience\Audience::delete(4);
            $this->assertTrue($status);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testAudienceSaveCommand()
    {
        try {
            $status = \Skoiy\Ott\Audience\Command::synchronize(1);
            $this->assertTrue($status);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testSaveAudienceGroup()
    {
        try {
            $group = new Skoiy\Ott\Audience\Group();
            $group->save(1, 1);
            $this->assertTrue($group instanceof Group);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testQueryAudienceOptions()
    {
        try {
            $query = new Skoiy\Ott\Audience\Query();
            $result = \Skoiy\Ott\Audience\Option::query($query);
            foreach ($result as $option) {
                $this->assertTrue($option instanceof \Skoiy\Ott\Audience\Option);
            }
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testDeleteAudienceGroup()
    {
        try {
            $status = \Skoiy\Ott\Audience\Group::delete(7, 1);
            $this->assertTrue($status);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testAttachAudienceGroupOption()
    {
        try {
            $idGroup = 1;
            $idAudience = 1;
            $idOption = 1;
            $optionValue = "value";
            $status = \Skoiy\Ott\Audience\Group::attachOption($idGroup, $idAudience, $idOption, $optionValue);
            $this->assertTrue($status);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testDetachAudienceGroupOption()
    {
        try {
            $idGroup = 1;
            $idAudience = 1;
            $idOption = 1;
            $status = \Skoiy\Ott\Audience\Group::detachOption($idGroup, $idAudience, $idOption);
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
            $this->assertTrue($e instanceof Exception);
        }
    }

}
