<?php

use Skoiy\Ott\Freebie\CodeQuery;
use Skoiy\Ott\Freebie\EmailQuery;
use Skoiy\Ott\Freebie\Freebie;
use Skoiy\Ott\Freebie\FreebieQuery;
use Skoiy\Ott\Freebie\L10n;
use Skoiy\Ott\Freebie\SecurityCountry;
use Skoiy\Ott\Freebie\UsedFreebieQuery;
use Skoiy\Ott\Plan\Plan;
use Stage\Sps;
use Stage\Stage;
use Service\Connection;
use Stage\Account\Account;
use Stage\StoredProcedure\Manager as StoredProcedureManager;

class FreebieTest extends \PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        $stageInstance = Stage::getInstance();

        $credentials = [
            'host' => "mysqldb",
            'dbname' => "StageAccount",
            'username' => "root",
            'password' => "secret"
        ];

        $clientCredentials = [
            'host' => "mysqldb",
            'dbname' => "bd1",
            'username' => "root",
            'password' => "secret"
        ];

        $tasksCredentials = [
            'host' => "mysqldb",
            'dbname' => "SkoiyTasks",
            'username' => "root",
            'password' => "secret"
        ];

        $ottCredentials = [
            'host' => "mysqldb",
            'dbname' => "bd279_OTT",
            'username' => "root",
            'password' => "secret"
        ];

        $default = new Connection($credentials);
        $client = new Connection($clientCredentials);
        $tasks = new Connection($tasksCredentials);
        $ott = new Connection($ottCredentials);
        $stageInstance->addConnection('default', $default);
        $stageInstance->addConnection('client', $client);
        $stageInstance->addConnection('tasks', $tasks);
        $stageInstance->addConnection('ott', $ott);

        // 3. Set the Stage stored procedures
        $stageInstance->setSps(Sps::load());

        // Load Skoiy OTT routines ( stored procedures ) and attach to the Stage instance
        $ottRoutines = \Skoiy\Ott\Ott::routines();
        $stageInstance->attachRoutines($ottRoutines);

        // Set the StoredProcedure\Manager instance
        $spm = StoredProcedureManager::getInstance();

        // Set the Stored Procedure Manager
        $stageInstance->setStoredProcedureManager($spm);

        try {
            $idAccount = 3;
            $account = Account::find($idAccount);

            // Get a Stage instance
            $stageInstance->setAccount($account);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function tearDown()
    {
    }

    public function testCreateFreebie()
    {
        try {
            $freebie = new Skoiy\Ott\Freebie\Freebie();
            $freebie->setName("new");
            $freebie->setNameStripped("new");
            $freebie->setBeginDate("2020-12-12");
            $freebie->setEndDate("2020-12-31");
            $freebie->setAppliedToAllPeriods(true);
            $freebie->setBillingPeriod(1);
            $freebie->setOneUsePerUser(true);
            $freebie->save();
            $this->assertTrue($freebie instanceof Freebie);

        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testCreateL10nFreebie()
    {
        try {
            $l10n = new Skoiy\Ott\Freebie\L10n();
            $l10n->setIdFreebie(1);
            $l10n->setIsoLanguage("en");
            $l10n->setName("name");
            $l10n->setDescription("name");
            $l10n->setDetails("details");
            $l10n->setActionName("action");
            $l10n->setActionTarget("target");
            $l10n->save();
            $this->assertTrue($l10n instanceof L10n);

        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testGetFreebieL10ns()
    {
        try {
            $results = Freebie::l10n(1);
            foreach ($results as $result) {
                $this->assertTrue($result instanceof L10n);
            }
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testDeleteFreebieL10n()
    {
        try {
            $freebie = \Skoiy\Ott\Freebie\L10n::delete(3);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testFindFreebie()
    {
        try {
            $freebie = Freebie::find(1);
            $this->assertTrue($freebie instanceof Freebie);

        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testDupliateFreebie()
    {
        try {
            $freebie = Freebie::duplicate(1);
            $this->assertTrue($freebie instanceof Freebie);

        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testNumFreebie()
    {
        try {
            $total = Freebie::sizeof();
            echo $total;
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testIsoLanguagesFromFreebie()
    {
        try {
            $total = Freebie::isoLanguages(1);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testDeleteFreebie()
    {
        try {
            $deleted = Freebie::delete(3);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testQueryFreebies()
    {
        try {
            $q = new \Skoiy\Ott\Freebie\FreebieQuery();
            $results = Freebie::query($q);
            foreach ($results as $result) {
                $this->assertTrue($result instanceof Freebie);
            }
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testCreateSecurityCountryFreebie()
    {
        try {
            $s = SecurityCountry::createSecurityCountries(1, "pt", false);
            $this->assertTrue($s instanceof SecurityCountry);

        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testFindSecurityCountryFreebie()
    {
        try {
            $f = Freebie::securityCountries(1);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testRemoveSecurityCountryFreebie()
    {
        try {
            $f = SecurityCountry::removeAllSecurityCountries(1);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testFreebieAddCode()
    {
        try {
            $code = Freebie::addCode(1, "PT_256", 1, true, true);
            $this->assertTrue($code instanceof \Skoiy\Ott\Promotion\Code);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testFreebieRemoveCode()
    {
        try {
            Freebie::removeCode(1, 1, 1);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testGetFreebieCodes()
    {
        try {
            Freebie::removeCode(1, 1, 1);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testAddToWhitelistFreebie()
    {
        try {
            Freebie::addToWhitelist(-1, 1, "email@gmail.com", 1, true);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testRemoveFromWhitelistFreebie()
    {
        try {
            Freebie::removeFromWhitelist(1, 1, 1);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testAddPlanToFreebie()
    {
        try {
            Freebie::addPlan(1, 1, 1);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testRemovePlanToFreebie()
    {
        try {
            Freebie::removePlan(1, 1, 1);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testGetPlansFromFreebie()
    {
        try {
            $results = Freebie::plans(1);
            foreach ($results as $result) {
                $this->assertTrue($result instanceof Plan);
            }
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }
}