<?php

use Skoiy\Ott\Leaderboard\Leaderboard;
use Skoiy\Ott\Leaderboard\User;
use Stage\Sps;
use Stage\Stage;
use Service\Connection;
use Stage\Account\Account;
use Stage\StoredProcedure\Manager as StoredProcedureManager;

class LeaderBoardTest extends \PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        $stageInstance = Stage::getInstance();

        $credentials = [
            'host' => "mysqldb",
            'dbname' => "StageAccount",
            'username' => "root",
            'password' => "secret"
        ];

        $clientCredentials = [
            'host' => "mysqldb",
            'dbname' => "bd1",
            'username' => "root",
            'password' => "secret"
        ];

        $tasksCredentials = [
            'host' => "mysqldb",
            'dbname' => "SkoiyTasks",
            'username' => "root",
            'password' => "secret"
        ];

        $ottCredentials = [
            'host' => "mysqldb",
            'dbname' => "bd279_OTT",
            'username' => "root",
            'password' => "secret"
        ];

        $default = new Connection($credentials);
        $client = new Connection($clientCredentials);
        $tasks = new Connection($tasksCredentials);
        $ott = new Connection($ottCredentials);
        $stageInstance->addConnection('default', $default);
        $stageInstance->addConnection('client', $client);
        $stageInstance->addConnection('tasks', $tasks);
        $stageInstance->addConnection('ott', $ott);

        // 3. Set the Stage stored procedures
        $stageInstance->setSps(Sps::load());

        // Load Skoiy OTT routines ( stored procedures ) and attach to the Stage instance
        $ottRoutines = \Skoiy\Ott\Ott::routines();
        $stageInstance->attachRoutines($ottRoutines);

        // Set the StoredProcedure\Manager instance
        $spm = StoredProcedureManager::getInstance();

        // Set the Stored Procedure Manager
        $stageInstance->setStoredProcedureManager($spm);

        try {
            $idAccount = 3;
            $account = Account::find($idAccount);

            // Get a Stage instance
            $stageInstance->setAccount($account);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function tearDown()
    {
    }

//    public function testFindPlan()
//    {
//        try {
//            $plan = Freebie::find(1);
//            $this->assertTrue($plan instanceof Freebie);
//
//        } catch (Exception $e) {
//            $this->assertTrue($e instanceof Exception);
//            echo $e->getMessage();
//        }
//    }
//
//    public function testPlanIsoLanguages()
//    {
//        try {
//            $planL10ns = Freebie::isoLanguages(1);
//            print_r($planL10ns);
//        } catch (Exception $e) {
//            $this->assertTrue($e instanceof Exception);
//            echo $e->getMessage();
//        }
//    }
//
//    public function testPlanGetPrices()
//    {
//        try {
//            $planL10ns = Freebie::prices(1);
//            print_r($planL10ns);
//        } catch (Exception $e) {
//            $this->assertTrue($e instanceof Exception);
//            echo $e->getMessage();
//        }
//    }

    public function testCreateLeaderboard()
    {
        try {
            $l = new Leaderboard();
            $l->setName("name");
            $l->setNameStripped("management-striped");
            $l->setActive(true);
            $l->setAllowFriends(true);
            $l->setAllowOptimizeResults(true);
            $l->setAllowTeams(true);
            $l->setMaxFriendsPerUser(10);
            $l->setMaxOptimizationAttempts(20);
            $l->setMaxUsersPerTeam(30);
            $l->setTotalTeamsPerUser(40);

            $l->save();
            $this->assertTrue($l instanceof Leaderboard);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testLeaderboardFind()
    {
        try {
            $l = Leaderboard::find(1);
            $this->assertTrue($l instanceof Leaderboard);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testLeaderboardNum()
    {
        try {
            $result = Leaderboard::sizeof();
            $this->assertTrue($result > 0);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testLeaderboardQuery()
    {
        try {
            $q = new \Skoiy\Ott\Leaderboard\Query();
            $l = Leaderboard::query($q);
            foreach ($l as $l1) {
                $this->assertTrue($l1 instanceof Leaderboard);
            }
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testLeaderboardAttachL10n()
    {
        try {
            $l10n = \Skoiy\Ott\Leaderboard\Leaderboard::attachL10n(1, "pt", "1", "1");
            $this->assertInstanceOf('\Skoiy\Ott\Leaderboard\L10n', $l10n);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testLeaderboardL10ns()
    {
        try {
            $l10ns = \Skoiy\Ott\Leaderboard\Leaderboard::l10ns(1);
            foreach ($l10ns as $l1) {
                $this->assertTrue($l1 instanceof \Skoiy\Ott\Leaderboard\L10n);
            }
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testLeaderboardAttachTeamAvatar()
    {
        try {
            $l10n = \Skoiy\Ott\Leaderboard\Leaderboard::attachTeamAvatars(1, "name", 1, "fname", "avatar", true);
            $this->assertInstanceOf('\Skoiy\Ott\Leaderboard\TeamAvatar', $l10n);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testLeaderboardTeamsAvatarsQuery()
    {
        try {
            $q = new \Skoiy\Ott\Leaderboard\Query();
            $q->idLeaderboard = 1;
            $l = Leaderboard::queryTeamsAvatars($q);
            foreach ($l as $l1) {
                $this->assertTrue($l1 instanceof \Skoiy\Ott\Leaderboard\UserAvatar);
            }
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testLeaderboardAttachUserAvatar()
    {
        try {
            $l10n = \Skoiy\Ott\Leaderboard\Leaderboard::attachUserAvatars(1, "name", 1, "fname", "avatar", true);
            $this->assertInstanceOf('\Skoiy\Ott\Leaderboard\UserAvatar', $l10n);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testLeaderboardUsersAvatarsQuery()
    {
        try {
            $q = new \Skoiy\Ott\Leaderboard\Query();
            $q->idLeaderboard = 1;
            $l = Leaderboard::queryUsersAvatars($q);
            print_r($l->toArray());
            foreach ($l as $l1) {
                $this->assertTrue($l1 instanceof \Skoiy\Ott\Leaderboard\UserAvatar);
            }
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testLeaderboardIsoLanguages()
    {
        try {
            $isoLanguages = \Skoiy\Ott\Leaderboard\Leaderboard::isoLanguages(1);
            print_r($isoLanguages);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testLeaderboardUsersQuery()
    {
        try {
            $q = new \Skoiy\Ott\Leaderboard\Query();
            $q->idLeaderboard = 2;
            $l = Leaderboard::queryUsers($q);
            foreach ($l as $l1) {
                $this->assertTrue($l1 instanceof \Skoiy\Ott\Leaderboard\User);
            }
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }
}