<?php

use Stage\Sps;
use Stage\Stage;
use Service\Connection;
use Stage\Account\Account;
use Skoiy\Ott\Plan\Freebie;
use Skoiy\Ott\Plan\Set\Set;
use Skoiy\Ott\Plan\Set\L10n as SetL10n;
use Skoiy\Ott\Plan\L10n;
use Skoiy\Ott\Plan\Price;
use Skoiy\Ott\Plan\Set\SecurityCountry;
use Stage\StoredProcedure\Manager as StoredProcedureManager;

class PlanTest extends \PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        $stageInstance = Stage::getInstance();

        $credentials = [
            'host' => "mysqldb",
            'dbname' => "StageAccount",
            'username' => "root",
            'password' => "secret"
        ];

        $clientCredentials = [
            'host' => "mysqldb",
            'dbname' => "bd1",
            'username' => "root",
            'password' => "secret"
        ];

        $tasksCredentials = [
            'host' => "mysqldb",
            'dbname' => "SkoiyTasks",
            'username' => "root",
            'password' => "secret"
        ];

        $ottCredentials = [
            'host' => "mysqldb",
            'dbname' => "bd279_OTT",
            'username' => "root",
            'password' => "secret"
        ];

        $default = new Connection($credentials);
        $client = new Connection($clientCredentials);
        $tasks = new Connection($tasksCredentials);
        $ott = new Connection($ottCredentials);
        $stageInstance->addConnection('default', $default);
        $stageInstance->addConnection('client', $client);
        $stageInstance->addConnection('tasks', $tasks);
        $stageInstance->addConnection('ott', $ott);

        // 3. Set the Stage stored procedures
        $stageInstance->setSps(Sps::load());

        // Load Skoiy OTT routines ( stored procedures ) and attach to the Stage instance
        $ottRoutines = \Skoiy\Ott\Ott::routines();
        $stageInstance->attachRoutines($ottRoutines);

        // Set the StoredProcedure\Manager instance
        $spm = StoredProcedureManager::getInstance();

        // Set the Stored Procedure Manager
        $stageInstance->setStoredProcedureManager($spm);

        try {
            $idAccount = 3;
            $account = Account::find($idAccount);

            // Get a Stage instance
            $stageInstance->setAccount($account);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function tearDown()
    {
    }

    public function testFindPlan()
    {
        try {
            $plan = Freebie::find(1);
            $this->assertTrue($plan instanceof Freebie);

        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testFindPlanL10n()
    {
        try {
            $planL10ns = Freebie::l10n(1);
            print_r($planL10ns);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testPlanIsoLanguages()
    {
        try {
            $planL10ns = Freebie::isoLanguages(1);
            print_r($planL10ns);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testPlanGetPrices()
    {
        try {
            $planL10ns = Freebie::prices(1);
            print_r($planL10ns);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testCreatePlan()
    {
        try {
            $plan = new Freebie();
            $plan->setPlanType(1);
            $plan->setManagementName("management name");
            $plan->setNameStripped("management-striped");
            $plan->setBillingType(1);
            $plan->setBillingCycle(1);
            $plan->setFirstBillDate("");
            $plan->setTrialActive(true);
            $plan->setTrialType(1);
            $plan->setTrialPeriod(1);
            $plan->setTrialForNewUsers(1);
            $plan->setTimeActive(1);
            $plan->setTimeType(1);
            $plan->setTimePeriod(1);
            $plan->setGraceActive(1);
            $plan->setGraceType(1);
            $plan->setAllowPromotions(1);
            $plan->setActive(true);
            $plan->setUnlimitedAccess(true);
            $plan->setAcceptPayMore(true);
            $plan->setAcceptPayCycle(true);
            $plan->setGracePeriod(1);


            $plan->save();
            $this->assertTrue($plan instanceof Freebie);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testDeletePlanL10n()
    {
        try {
            $deleted = L10n::delete(3);
            print_r($deleted);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testDuplicatePlan()
    {
        try {
            $duplicated = Freebie::duplicate(7);
            print_r($duplicated);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testSavePlanL10n()
    {
        try {

            $l10n = new L10n();
            $l10n->setIdPlan(1);
            $l10n->setIsoLanguage("pt");
            $l10n->setName("name");
            $l10n->setDescription("description");
            $l10n->setDetails("details");
            $l10n->setActionName("action_name");
            $l10n->setActionTarget("action_target");
            $l10n->save();

        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testCretePlanPrice()
    {
        try {
            $price = new Price();
            $price->setIdPlan(1);
            $price->setPrice(58.55);
            $price->setTrialPrice(12);
            $price->setCurrencyIso("pl");
            $price->setPosition(1);
            $price->save();
            $this->assertTrue($price instanceof Price);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testFindPlanPrice()
    {
        try {
            $result = Price::find(1, 3);
            print_r($result->toArray());
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testDeleteAllPlanPrices()
    {
        try {
            $result = Price::delete(7, Price::DELETE_ALL);
            print_r($result);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testDeleteSinglePlanPrice()
    {
        try {
            $result = Price::delete(1, 11);
            var_dump($result);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testFindPlanSet()
    {
        try {
            $plan = Set::find(1);
            $this->assertTrue($plan instanceof Set);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testPlanSetIsoLanguages()
    {
        try {
            $planL10ns = Set::isoLanguages(1);
            print_r($planL10ns);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testPlanSetL10n()
    {
        try {
            $planL10ns = Set::l10n(8);
            print_r($planL10ns);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testPlanSetFindPlans()
    {
        try {
            $planL10ns = Set::plans(8);
            print_r($planL10ns);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testPlanGetSecurityCountries()
    {
        try {
            $planL10ns = Set::securityCountries(1);
            print_r($planL10ns);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testCreatePlanSet()
    {
        try {
            $price = new Set();
            $price->setNameStripped("name stripped");
            $price->setManagementName("management_name");
            $price->setActive(true);
            $price->setDomains("http://www.sapo.pt");
            $price->setExclude(false);
            $price->save();
            print_r($price);
            $this->assertTrue($price instanceof Set);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testDuplicatePlanSet()
    {
        try {
            $duplicated = Set::duplicate(1);
        } catch (Exception $e) {

        }
    }

    public function testSetSecurityCountry()
    {
        try {
            $result = SecurityCountry::createSecurityCountries(1, "pt", true);
            print_r($result);
        } catch (Exception $e) {

        }
    }

    public function testCreatePlanSetl10n()
    {
        try {
            $price = new SetL10n();
            $price->setIdPlanSet(8);
            $price->setIsoLanguage("en");
            $price->setName("name");
            $price->setDescription("description");
            $price->setDetails("details");
            $price->setActionName("action_name");
            $price->setActionTarget("action_target");
            $price->save();
            $this->assertTrue($price instanceof SetL10n);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testFindPlanSetL10n()
    {
        try {
            $l10n = SetL10n::find(8, 8);
            print_r($l10n->toArray());
            die();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPlanSetAddPlan()
    {
        try {
            $return = Set::addPlan(1, 6, true);
        } catch (Exception $e) {

        }
    }

    public function testPlanSetRemovePlan()
    {
        try {
            $return = Set::removePlan(1, 1, 6);
        } catch (Exception $e) {

        }
    }

    public function testDeletePlanSet()
    {
        try {
            $deleted = Set::delete(4);
            print_r($deleted);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testDeletePlanSetL10n()
    {
        try {
            $deleted = SetL10n::delete(2);
            print_r($deleted);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testDeletePlan()
    {
        try {
            $deleted = Freebie::delete(7);
            print_r($deleted);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testDeletePlanSetSecurityCountries()
    {
        try {
            $deleted = SecurityCountry::removeAllSecurityCountries(1);
            print_r($deleted);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

//    public function testDuplicatePlanL10()
//    {
//        try {
//            $duplicated = L10n::duplicate(1,1);
//            print_r($duplicated);
//        } catch (Exception $e) {
//            echo $e->getMessage();
//        }
//    }

    public function testLanguages()
    {
        Skoiy\Ott\Language\Language::all();
    }

    public function testCurrency()
    {
        Skoiy\Ott\Currency\Currency::all();
    }

    public function testCountries()
    {
        $data = Skoiy\Ott\Country\Country::all();
        print_r($data);
    }
}