<?php

use Skoiy\Ott\User\Query;
use Stage\Sps;
use Stage\Stage;
use Service\Connection;
use Stage\Account\Account;
use Skoiy\Ott\Promotion\Promotion;
use Skoiy\Ott\Promotion\SecurityCountry;
use Stage\StoredProcedure\Manager as StoredProcedureManager;

class PromotionTest extends \PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        $stageInstance = Stage::getInstance();
        $host = "mysqldb";

        $credentials = [
            'host' => "$host",
            'dbname' => "StageAccount",
            'username' => "root",
            'password' => "secret"
        ];

        $clientCredentials = [
            'host' => "$host",
            'dbname' => "bd1",
            'username' => "root",
            'password' => "secret"
        ];

        $tasksCredentials = [
            'host' => "$host",
            'dbname' => "SkoiyTasks",
            'username' => "root",
            'password' => "secret"
        ];

        $ottCredentials = [
            'host' => "$host",
            'dbname' => "bd279_OTT",
            'username' => "root",
            'password' => "secret"
        ];

        $default = new Connection($credentials);
        $client = new Connection($clientCredentials);
        $tasks = new Connection($tasksCredentials);
        $ott = new Connection($ottCredentials);
        $stageInstance->addConnection('default', $default);
        $stageInstance->addConnection('client', $client);
        $stageInstance->addConnection('tasks', $tasks);
        $stageInstance->addConnection('ott', $ott);

        // 3. Set the Stage stored procedures
        $stageInstance->setSps(Sps::load());

        // Load Skoiy OTT routines ( stored procedures ) and attach to the Stage instance
        $ottRoutines = \Skoiy\Ott\Ott::routines();
        $stageInstance->attachRoutines($ottRoutines);

        // Set the StoredProcedure\Manager instance
        $spm = StoredProcedureManager::getInstance();

        // Set the Stored Procedure Manager
        $stageInstance->setStoredProcedureManager($spm);

        try {
            $idAccount = 3;
            $account = Account::find($idAccount);

            // Get a Stage instance
            $stageInstance->setAccount($account);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function tearDown()
    {
    }

    public function testFindPromotion()
    {
        try {
            $promotion = Promotion::find(1);
            print_r($promotion);
            $this->assertTrue($promotion instanceof Promotion);

        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testFindPromotionPlans()
    {
        try {
            $plans = Promotion::plans(1);
            print_r($plans);

        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testFindPromotionSecurityCountries()
    {
        try {
            $plans = Promotion::securityCountries(1);
            print_r($plans);

        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testQueryPromotionCodes()
    {
        try {
            $q = new \Skoiy\Ott\Promotion\CodeQuery();
            $q->filter = 1; // its the promotion id
            $results = Promotion::query($q);
            print_r($results);

        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testQueryPromotionEmails()
    {
        try {
            $q = new \Skoiy\Ott\Promotion\EmailQuery();
            $q->filter = 1; // its the promotion id
            $results = Promotion::query($q);
            print_r($results);

        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testQueryPromotionUsed()
    {
        try {
            $q = new \Skoiy\Ott\Promotion\UsedQuery();
            $q->filter = 1; // its the promotion id
            $results = Promotion::query($q);
            $count = Promotion::queryCount($q);
            print_r($count);
            print_r($results);

        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testCreatePromotion()
    {
        try {
            $plan = new Promotion();
            $plan->setPromotionType(1);
            $plan->setPercentOff(1);
            $plan->setPriceOff(1);
            $plan->setName("name");
            $plan->setNameStripped("management-striped");
            $plan->setBeginDate("2020-12-12");
            $plan->setEndDate("2020-12-31");
            $plan->setAppliedOnRenewal(false);
            $plan->setAppliedToAllPeriods(true);
            $plan->setBillingPeriod(1);
            $plan->setCodeRequired(true);
            $plan->setEmailListEnabled(true);
            $plan->setEmailDomains("http://www.sapo.pt");
            $plan->setActive(true);

            $plan->save();
            $this->assertTrue($plan instanceof Promotion);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testClonePromotion()
    {
        try {
            $promotion = Promotion::duplicate(2);
            $this->assertTrue($promotion instanceof Promotion);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testDeletePromotion()
    {
        try {
            $deleted = Promotion::delete(3);
            print_r($deleted);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testCreateSecurityCountry()
    {
        try {
            $deleted = SecurityCountry::createSecurityCountries(2, "pt", false);
            print_r($deleted);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testDeletePromotionSecurityCountries()
    {
        try {
            $deleted = SecurityCountry::removeAllSecurityCountries(2);
            print_r($deleted);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testPromotionAddPlan()
    {
        try {
            $added = Promotion::addPlan(2, 6);
            $this->assertTrue($added);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testPromotionRemovePlan()
    {
        try {
            $added = Promotion::removePlan(1, 2, 6);
            $this->assertTrue($added);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testPromotionAddCode()
    {
        try {
            $code = Promotion::addCode(2, "PT_256", 5, true, true);
            $this->assertTrue($code instanceof \Skoiy\Ott\Promotion\Code);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testPromotionRemoveCode()
    {
        try {
            $added = Promotion::removeCode(1, 2, 1);
            $this->assertTrue($added);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testPromotionAddEmailToWhiteList()
    {
        try {
            $code = Promotion::addToWhitelist(-1, 2, "email@gmail.com", 1, 1);
            $this->assertTrue($code instanceof \Skoiy\Ott\Promotion\WhiteList);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testPromotionRemoveEmailFromWhiteList()
    {
        try {
            $status = Promotion::removeFromWhitelist(1, 2, 3);
            $this->assertTrue($status);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

}