<?php

use Skoiy\Ott\Scan\Code;
use Stage\Sps;
use Stage\Stage;
use Service\Connection;
use Skoiy\Ott\Scan\Scan;
use Stage\Account\Account;
use Skoiy\Ott\Leaderboard\Leaderboard;
use Stage\StoredProcedure\Manager as StoredProcedureManager;

class ScanTest extends \PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        $stageInstance = Stage::getInstance();

        $credentials = [
            'host' => "mysqldb",
            'dbname' => "StageAccount",
            'username' => "root",
            'password' => "secret"
        ];

        $clientCredentials = [
            'host' => "mysqldb",
            'dbname' => "bd1",
            'username' => "root",
            'password' => "secret"
        ];

        $tasksCredentials = [
            'host' => "mysqldb",
            'dbname' => "SkoiyTasks",
            'username' => "root",
            'password' => "secret"
        ];

        $ottCredentials = [
            'host' => "mysqldb",
            'dbname' => "bd279_OTT",
            'username' => "root",
            'password' => "secret"
        ];

        $default = new Connection($credentials);
        $client = new Connection($clientCredentials);
        $tasks = new Connection($tasksCredentials);
        $ott = new Connection($ottCredentials);
        $stageInstance->addConnection('default', $default);
        $stageInstance->addConnection('client', $client);
        $stageInstance->addConnection('tasks', $tasks);
        $stageInstance->addConnection('ott', $ott);

        // 3. Set the Stage stored procedures
        $stageInstance->setSps(Sps::load());

        // Load Skoiy OTT routines ( stored procedures ) and attach to the Stage instance
        $ottRoutines = \Skoiy\Ott\Ott::routines();
        $stageInstance->attachRoutines($ottRoutines);

        // Set the StoredProcedure\Manager instance
        $spm = StoredProcedureManager::getInstance();

        // Set the Stored Procedure Manager
        $stageInstance->setStoredProcedureManager($spm);

        try {
            $idAccount = 3;
            $account = Account::find($idAccount);

            // Get a Stage instance
            $stageInstance->setAccount($account);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function tearDown()
    {
    }

    public function testCreateScan()
    {
        try {
            $l = new \Skoiy\Ott\Scan\Scan();
            $l->setManagementName("management_name");
            $l->setScanType(1);
            $l->setStartDate("2020-12-12");
            $l->setEndDate("2020-12-31");
            $l->setActive(true);
            $l->save();
            $this->assertTrue($l instanceof Scan);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Scan);
            echo $e->getMessage();
        }
    }

    public function testScanFind()
    {
        try {
            $l = Scan::find(1);
            $this->assertTrue($l instanceof Scan);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testScanNum()
    {
        try {
            $result = Scan::sizeof();
            $this->assertTrue($result > 0);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testScanQuery()
    {
        try {
            $q = new \Skoiy\Ott\Scan\Query();
            $l = Scan::query($q);
            foreach ($l as $l1) {
                $this->assertTrue($l1 instanceof Scan);
            }
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testScanCodesQuery()
    {
        try {
            $q = new \Skoiy\Ott\Scan\Query();
            $q->filter = 2; // it's the id scan
            $l = Scan::queryCodes($q);
            foreach ($l as $l1) {
                $this->assertTrue($l1 instanceof Code);
            }
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testScanCodesNum()
    {
        try {
            $totalCodes = Scan::codeExists("prefx1P1n1eD8zZsiyhYhnxNzqI");
            $this->assertTrue($totalCodes > 0);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testScanAttachL10n()
    {
        try {
            $l10n = \Skoiy\Ott\Scan\Scan::attachL10n(1, "pt", "1", "stripped", "desc", "image.xoa");
            $this->assertInstanceOf('\Skoiy\Ott\Scan\L10n', $l10n);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testScanL10ns()
    {
        try {
            $l10ns = \Skoiy\Ott\Scan\Scan::l10ns(1);
            foreach ($l10ns as $l1) {
                $this->assertTrue($l1 instanceof \Skoiy\Ott\Scan\L10n);
            }
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testScanAddCode()
    {
        try {
            // $idScan, $code, $quantity, $unlimited, $active, $maxScansPerUser, $dailyLimitPerUser, $scanIntervalPerUser, $randomCode, $prefix, $numbers, $lowercase, $uppercase, $length, $totalCodes, $idCode = -1
            $code = Scan::addCode(2, "PT_256", 5, true, true, true, 20, 10, true, "prefx", 1, true, true,22,99);
            $this->assertTrue($code instanceof \Skoiy\Ott\Scan\Code);
            $this->assertTrue($code->getScanIntervalPerUser() == 10);
            $this->assertTrue($code->getDailyLimitPerUser() == 20);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testDeleteScan()
    {
        try {
            $deleted = Scan::delete(3);
            $this->assertTrue($deleted);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testDeleteScanCode()
    {
        try {
            $deleted = Scan::deleteScanCode(1,2,4);
            $this->assertTrue($deleted);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

}