<?php

use Skoiy\Ott\User\PlanQueryResponse;
use Skoiy\Ott\User\Query;
use Skoiy\Ott\User\TransactionQueryResponse;
use Stage\Sps;
use Stage\Stage;
use Service\Connection;
use Stage\Account\Account;
use Skoiy\Ott\User\User;
use Stage\StoredProcedure\Manager as StoredProcedureManager;

class UserTest extends \PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        $stageInstance = Stage::getInstance();
        $host = "mysqldb";

        $credentials = [
            'host' => "$host",
            'dbname' => "StageAccount",
            'username' => "root",
            'password' => "secret"
        ];

        $clientCredentials = [
            'host' => "$host",
            'dbname' => "bd1",
            'username' => "root",
            'password' => "secret"
        ];

        $tasksCredentials = [
            'host' => "$host",
            'dbname' => "SkoiyTasks",
            'username' => "root",
            'password' => "secret"
        ];

        $ottCredentials = [
            'host' => "$host",
            'dbname' => "bd279_OTT",
            'username' => "root",
            'password' => "secret"
        ];

        $default = new Connection($credentials);
        $client = new Connection($clientCredentials);
        $tasks = new Connection($tasksCredentials);
        $ott = new Connection($ottCredentials);
        $stageInstance->addConnection('default', $default);
        $stageInstance->addConnection('client', $client);
        $stageInstance->addConnection('tasks', $tasks);
        $stageInstance->addConnection('ott', $ott);

        // 3. Set the Stage stored procedures
        $stageInstance->setSps(Sps::load());

        // Load Skoiy OTT routines ( stored procedures ) and attach to the Stage instance
        $ottRoutines = \Skoiy\Ott\Ott::routines();
        $stageInstance->attachRoutines($ottRoutines);

        // Set the StoredProcedure\Manager instance
        $spm = StoredProcedureManager::getInstance();

        // Set the Stored Procedure Manager
        $stageInstance->setStoredProcedureManager($spm);

        try {
            $idAccount = 3;
            $account = Account::find($idAccount);

            // Get a Stage instance
            $stageInstance->setAccount($account);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function tearDown()
    {
    }

    public function testFindUser()
    {
        try {
            $user = User::find(1);
            print_r($user);
            $this->assertTrue($user instanceof User);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testQueryUsers()
    {
        try {
            $q = new Query();
            $q->filter = 0;
            $q->from = 0;
            $q->to = 2;
            /** @var []User $users */
            $users = User::query($q);
            print_r($users);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testQueryUserTransactions()
    {
        try {
            $q = new Query();
            $q->idUser = 5;
            $q->from = 0;
            $q->to = -1;
            /** @var []User $users */
            $results = User::queryTransactions($q);
            foreach ($results as $result){
                $this->assertTrue($result instanceof TransactionQueryResponse);
            }
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testQueryUserPlans()
    {
        try {
            $q = new Query();
            $q->idUser = 43;
            $q->from = 0;
            $q->to = -1;
            /** @var []User $users */
            $results = User::queryPlans($q);
            foreach ($results as $result){
                $this->assertTrue($result instanceof PlanQueryResponse);
            }
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testSizeofUser()
    {
        try {
            $totalUsers = User::sizeof();
            print_r($totalUsers);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testGetUserMetrics()
    {
        try {
            $q = new \Skoiy\Ott\User\Metric\Query();
            $q->by = 2;
            $q->trdBy = 1002;
            $q->limitTrdA = '2020-09-12';
            $q->limitTrdB = '2020-12-31';
            $metric = \Skoiy\Ott\User\Metric\Metric::query($q);
            print_r($metric->toArray());
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

    public function testCreateUpdateUser()
    {
        try {
            $user = Skoiy\Ott\User\User::find(16);
            $user->setName("name");
            $user->setEmail("email@email.com");
            $user->setGender(1);
            $user->setBirthdate("1986-05-21");
            $user->setIdentification("1111");
            $user->setTaxNumber("aaa");
            $user->setNationality("portuguese");
            $user->setCity("barcelos");
            $user->setZipcode("4750");
            $user->setCountryIso("pt");
            $user->setPhone(1);
            $user->setActive(1);
            $user->setVerified(1);

            $user->save();
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
            echo $e->getMessage();
        }
    }

}
